Package: gemtc
Version: 1.1-1
Date: 2026-03-25
Title: Network Meta-Analysis Using Bayesian Methods
Author: Gert van Valkenhoef [aut, cre],
  Joel Kuiper [aut]
Authors@R: c(person(given = "Gert",
                    family = "van Valkenhoef",
                    role = c("aut", "cre"),
                    email = "gert@gertvv.nl"),
             person(given = "Joel",
                    family = "Kuiper",
                    role = "aut"))
Maintainer: Gert van Valkenhoef <gert@gertvv.nl>
Description: Network meta-analyses (mixed treatment comparisons) in the Bayesian
    framework using JAGS. Includes methods to assess heterogeneity and
    inconsistency, and a number of standard visualizations.
    van Valkenhoef et al. (2012) <doi:10.1002/jrsm.1054>;
    van Valkenhoef et al. (2015) <doi:10.1002/jrsm.1167>.
Depends: R (>= 3.5.0), coda (>= 0.13)
Imports: igraph (>= 1.5), meta (>= 2.1), plyr (>= 1.8), graphics,
        grDevices, stats, utils, grid, rjags (>= 4-0), truncnorm,
        Rglpk, forcats (>= 0.5.0)
Suggests: testthat (>= 0.8), Matrix
URL: https://github.com/gertvv/gemtc
License: GPL-3
LazyData: true
Collate: 'anohe.R' 'arrayize.R' 'blobbogram.R' 'template.R' 'code.R'
        'data.R' 'deviance.R' 'forest.R' 'solveLP.R' 'inits.R'
        'likelihoods.R' 'll-helper.counts.R' 'll.binom.cloglog.R'
        'll.binom.log.R' 'll.binom.logit.R' 'll.call.R'
        'll.normal.identity.R' 'll.normal.smd.R' 'll.poisson.log.R'
        'minimum.diameter.spanning.tree.R' 'mtc.data.studyrow.R'
        'mtc.hy.prior.R' 'mtc.model.R' 'mtc.model.consistency.R'
        'mtc.model.nodesplit.R' 'mtc.model.regression.R'
        'mtc.model.ume.R' 'mtc.model.use.R' 'mtc.network.R'
        'stopIfNotConsistent.R' 'mtc.result.R' 'mtc.run.R'
        'nodesplit.R' 'plotCovariateEffect.R' 'priors.R'
        'rank.probability.R' 'regression.R' 'relative.effect.R'
        'relative.effect.table.R'
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2026-03-25 19:50:07 UTC; gert
Repository: CRAN
Date/Publication: 2026-03-26 13:10:14 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 18:14:40 UTC; unix
Archs: gemtc.so.dSYM

useDynLib(gemtc, .registration = TRUE)

import(graphics)
import(grDevices)
import(stats)
import(utils)

import(grid)
import(coda)
importFrom(igraph, E, V, "V<-", edge, edges, vertex, ends, path, set.edge.attribute, are.connected, get.shortest.paths, graph.empty, simplify, clusters, induced.subgraph, get.edges, degree, get.edge.attribute, shortest.paths, as.undirected, graph.edgelist, vertices)
importFrom(plyr, aaply, laply)

S3method(print, mtc.network)
S3method(summary, mtc.network)
S3method(plot, mtc.network)

S3method(print, mtc.model)
S3method(summary, mtc.model)
S3method(plot, mtc.model)

S3method(print, mtc.result)
S3method(summary, mtc.result)
S3method(plot, mtc.result)
S3method(as.mcmc.list, mtc.result)
S3method(forest, mtc.result)

S3method(print, summary.mtc.result)

S3method(print, mtc.anohe)
S3method(plot, mtc.anohe)
S3method(summary, mtc.anohe)

S3method(print, mtc.anohe.summary)
S3method(plot, mtc.anohe.summary)

S3method(print, mtc.nodesplit)
S3method(plot, mtc.nodesplit)
S3method(summary, mtc.nodesplit)

S3method(print, mtc.nodesplit.summary)
S3method(plot, mtc.nodesplit.summary)

S3method(print, mtc.rank.probability)
S3method(plot, mtc.rank.probability)

S3method(as.character, mtc.hy.prior)

S3method(as.data.frame, mtc.relative.effect.table)
S3method(print, mtc.relative.effect.table)
S3method(forest, mtc.relative.effect.table)

S3method(plot, mtc.deviance)

export(
	mtc.network, mtc.data.studyrow,
	mtc.model, mtc.hy.prior, mtc.hy.empirical.lor,
	mtc.run,
	mtc.deviance, mtc.devplot, mtc.levplot,
	relative.effect, relative.effect.table,
	rank.probability, sucra, rank.quantiles,
	blobbogram, forest,
	plotCovariateEffect,
	mtc.anohe,
	mtc.nodesplit, mtc.nodesplit.comparisons,
	ll.call)

#' @include likelihoods.R

# Arm-level effect estimate (given a one-row data frame)
mtc.arm.mle.normal.identity <- function(data, k=0.5) {
  c('mean'=as.numeric(data['mean']), 'sd'=as.numeric(data['std.err']))
}

# Relative effect estimate (given a two-row data frame)
mtc.rel.mle.normal.identity <- function(data, correction.force=TRUE, correction.type="constant", correction.magnitude=1) {
  e1 <- mtc.arm.mle.normal.identity(data[1,])
  e2 <- mtc.arm.mle.normal.identity(data[2,])
  c(e2['mean'] - e1['mean'], sqrt(e1['sd']^2 + e2['sd']^2))
}

mtc.code.likelihood.normal.identity <- function(powerAdjust) {
  paste("theta[i, k] <- $armLinearModel$", likelihood.code.normal[powerAdjust + 1], sep="\n")
}

fitted.values.parameter.normal.identity <- fitted.values.parameter.normal
deviance_fn.normal.identity <- deviance_fn.normal

scale.log.normal.identity <- function() { FALSE }
scale.name.normal.identity <- function() { "Mean Difference" }

inits.info.normal.identity <- function() {
  list(
    limits=c(-Inf, +Inf),
    param='mu',
    transform=identity)
}

required.columns.ab.normal.identity <- function() {
  c('m'='mean', 'e'='std.err')
}

validate.data.normal.identity <- function(data.ab) {
  stopifnot(all(data.ab[['std.err']] > 0))
}

study.baseline.priors.normal.identity <- function() {
"for (i in studies.a) {
  mu[i] ~ dnorm(0, prior.prec)
}
"
}

context("mtc.model (column handling)")

test_that("std.dev + sampleSize is rewritten to std.err", {
  data <- read.table(textConnection('study treatment mean std.dev sampleSize
s01 A 0.0 1.5 12
s01 B 1.0 2.0 31'), header=T)
  network <- mtc.network(data)
  model <- mtc.model(network, likelihood='normal', link='identity')
  expect_that(model$network[['data.ab']]$std.err, equals(c(1.5/sqrt(12), 2.0/sqrt(31))))
})

test_that("normal.identity requires the right columns", {
  expect_that(required.columns.ab.normal.identity(), equals(c('m'='mean', 'e'='std.err')))
})

test_that("data.ab missing sampleSize throws error", {
  data <- read.table(textConnection('study treatment mean std.dev
s01 A 0.0 1.5
s01 B 1.0 2.0'), header=T)
  network <- mtc.network(data)
  expect_error(mtc.model(network, likelihood='normal', link='identity'))
})

test_that("data.ab missing std.dev throws error", {
  data <- read.table(textConnection('study treatment mean sampleSize 
s01 A 0.0 12
s01 B 1.0 31'), header=T)
  network <- mtc.network(data)
  expect_error(mtc.model(network, likelihood='normal', link='identity'))
})

test_that("data.ab missing mean throws error", {
  data <- read.table(textConnection('study treatment std.dev sampleSize 
s01 A 0.0 12
s01 B 1.0 31'), header=T)
  network <- mtc.network(data)
  expect_error(mtc.model(network, likelihood='normal', link='identity'))
})

test_that("data.ab with std.err does not throw error", {
  data <- read.table(textConnection('study treatment mean std.err 
s01 A 0.0 0.5 
s01 B 1.0 0.7'), header=T)
  network <- mtc.network(data)
  model <- mtc.model(network, likelihood='normal', link='identity')
  succeed()
})

test_that("data.re missing diff throws error", {
  data <- read.table(textConnection('study treatment diff std.err
s01 A NA 0.5
s01 B 1.0 0.7'), header=T)
  network <- mtc.network(data.re=data)
  network$data.re$diff <- NULL
  expect_error(mtc.model(network, likelihood='normal', link='identity'))
})

test_that("data.re missing std.err throws error", {
  data <- read.table(textConnection('study treatment diff std.err
s01 A NA 0.5
s01 B 1.0 0.7'), header=T)
  network <- mtc.network(data.re=data)
  network$data.re$std.err <- NULL
  expect_error(mtc.model(network, likelihood='normal', link='identity'))
})

#'@include arrayize.R

study.seq.ab <- function(data) {
  data[['studies.a']]
}

study.seq.re <- function(data) {
  c(data[['studies.r2']], data[['studies.rm']])
}

deviance_monitors_residuals_ab <- function(model) {
  do.call(c, lapply(study.seq.ab(model[['data']]), function(i) { paste0("dev[", i, ",", 1:model[['data']][['na']][i], "]") }))
}

deviance_monitors_residuals_re <- function(model) {
  do.call(c, lapply(study.seq.re(model[['data']]), function(i) { paste0("dev[", i, ",", 1, "]") }))
}

deviance_monitors_fitted_ab <- function(model) {
  fpname <- ll.call("fitted.values.parameter", model)
  do.call(c, lapply(study.seq.ab(model[['data']]), function(i) { paste0(fpname, "[", i, ",", 1:model[['data']][['na']][i], "]") }))
}

alpha.ab <- function(model) {
  if (!is.na(model[['powerAdjust']]) && !is.null(model[['powerAdjust']])) {
    do.call(c, lapply(study.seq.ab(model[['data']]), function(i) { rep(model[['data']][['alpha']][i], model[['data']][['na']][i]) }))
  } else {
    1
  }
}

deviance_monitors_fitted_re <- function(model) {
  do.call(c, lapply(study.seq.re(model[['data']]), function(i) { paste0("delta[", i, ",", 2:model[['data']][['na']][i], "]") }))
}

deviance_monitors <- function(model) {
  c(deviance_monitors_residuals_ab(model),
    deviance_monitors_residuals_re(model),
    deviance_monitors_fitted_ab(model),
    deviance_monitors_fitted_re(model))
}

devfit.ab <- function(model, fit.ab) {
  studies <- study.seq.ab(model[['data']])
  if (length(studies) > 0) {
    cols <- names(ll.call("required.columns.ab", model))
    data <- model[['data']][cols]
    data <- lapply(data, function(el) {
      x <- as.vector(t(el[studies, , drop=FALSE]))
      x[!is.na(x)]
    })
    ll.call("deviance_fn", model, data, fit.ab, alpha=alpha.ab(model))
  } else {
    c()
  }
}

devfit.re <- function(model, mfit) {
  data <- model[['data']]
  s <- study.seq.re(data)
  if (length(s) > 0) {
    sapply(s, function(i) {
      na <- data[['na']][i]
      prev <- s[1:which(s == i)]
      start <- sum(data[['na']][prev] - 1) - na + 2
      ifit <- mfit[start:(start + na - 2)]

      cov <- if (!is.na(data[['e']][i, 1])) data[['e']][i, 1]^2 else 0
      Sigma <- matrix(cov, nrow=(na-1), ncol=(na-1))
      diag(Sigma) <- data[['e']][i, 2:na]^2
      Omega <- solve(Sigma)
      m <- data[['m']][i, 2:na]

      mdiff <- m - ifit

      alpha <-
        if (!is.na(model[['powerAdjust']]) && !is.null(model[['powerAdjust']])) data[['alpha']][i]
        else 1

      alpha * t(mdiff) %*% Omega %*% mdiff
    })
  } else {
    c()
  }
}

computeDeviance <- function(model, stats) {
  arms <- arm.index.matrix(model[['network']])
  studies.ab <- study.seq.ab(model[['data']])
  studies.re <- study.seq.re(model[['data']])

  nd <- apply(!is.na(arms), 1, sum)
  nd.ab <- nd[studies.ab]
  nd.re <- nd[studies.re] - 1
  dp <- sum(nd.ab) + sum(nd.re)

  shape.ab <- function(x) {
    if (length(x) > 0) {
      tpl <- arms[studies.ab,]
      x <- unname(x)
      y <- t(tpl)
      y[!is.na(y)] <- x
      t(y)
    } else {
      c()
    }
  }

  name.ab <- function(x) {
    if (length(x) > 0) {
      names(x) <- rownames(arms)[studies.ab]
      x
    } else {
      c()
    }
  }

  name.re <- function(x) {
    if (length(x) > 0) {
      names(x) <- rownames(arms)[studies.re]
      x
    } else {
      c()
    }
  }

  dev.ab <- stats[deviance_monitors_residuals_ab(model)]
  if (length(dev.ab) == 0) {
    dev.ab <- NULL
  }
  dev.re <- stats[deviance_monitors_residuals_re(model)]
  if (length(dev.re) == 0) {
    dev.re <- NULL
  }

  Dbar <- sum(c(dev.ab, dev.re))

  fit.ab <- devfit.ab(model, stats[deviance_monitors_fitted_ab(model)])
  if (length(fit.ab) == 0) {
    fit.ab <- NULL
  }
  fit.re <- devfit.re(model, stats[deviance_monitors_fitted_re(model)])
  if (length(fit.re) == 0) {
    fit.re <- NULL
  }

  lev.ab <- dev.ab - fit.ab
  if (length(lev.ab) == 0) {
    lev.ab <- NULL
  }
  lev.re <- dev.re - fit.re
  if (length(lev.re) == 0) {
    lev.re <- NULL
  }

  pD <- Dbar - sum(c(fit.ab, fit.re))

  fitted <- c(stats[deviance_monitors_fitted_ab(model)], stats[deviance_monitors_fitted_re(model)])

  info <- list(Dbar=Dbar, pD=pD, DIC=Dbar+pD, "data points"=dp,
               dev.ab=shape.ab(dev.ab), dev.re=name.re(dev.re),
               fit.ab=shape.ab(fit.ab), fit.re=name.re(fit.re),
               lev.ab=shape.ab(lev.ab), lev.re=name.re(lev.re),
               nd.ab=name.ab(nd.ab), nd.re=name.re(nd.re),
               fitted=arrayize(fitted))
  class(info) <- "mtc.deviance"
  info
}

mtc.deviance <- function(result) {
  stopifnot(class(result) == "mtc.result")
  result[['deviance']]
}

mtc.devplot <- function(x, ...) {
  stopifnot(class(x) == "mtc.deviance")

  if (is.null(x[['dev.re']])) {
    tpl <- x[['dev.ab']]
    study <- matrix(rep(1:nrow(tpl), times=ncol(tpl)), nrow=nrow(tpl), ncol=ncol(tpl))
    study <- t(study)[t(!is.na(tpl))]
    devbar <- t(x[['dev.ab']])[t(!is.na(tpl))]
    title <- "Per-arm residual deviance"
    xlab <- "Arm"
  } else {
    nd <- c(x[['nd.ab']], x[['nd.re']])
    devbar <- c(apply(x[['dev.ab']], 1, sum, na.rm=TRUE), x[['dev.re']]) / nd
    study <- 1:length(devbar)
    title <- "Per-study mean per-datapoint residual deviance"
    xlab <- "Study"
  }

  plot(devbar, ylim=c(0,max(devbar, na.rm=TRUE)),
      ylab="Residual deviance", xlab=xlab,
      main=title, pch=c(1, 22)[(study%%2)+1],
      ...)
  for (i in 1:length(devbar)) {
    lines(c(i, i), c(0, devbar[i]))
  }
}

mtc.levplot <- function(x, ...) {
  stopifnot(class(x) == "mtc.deviance")

  fit.ab <- apply(x[['fit.ab']], 1, sum, na.rm=TRUE)
  dev.ab <- apply(x[['dev.ab']], 1, sum, na.rm=TRUE)
  lev.ab <- dev.ab - fit.ab
  fit.re <- x[['fit.re']]
  dev.re <- x[['dev.re']]
  lev.re <- dev.re - fit.re
  nd <- c(x[['nd.ab']], x[['nd.re']])
  w <- sqrt(c(dev.ab, dev.re) / nd)
  lev <- c(lev.ab, lev.re) / nd

  plot(w, lev, xlim=c(0, max(c(w, 2.5))), ylim=c(0, max(c(lev, 4))),
       xlab="Square root of residual deviance", ylab="Leverage",
       main="Leverage versus residual deviance",
       ...)
  mtext("Per-study mean per-datapoint contribution")

  x <- seq(from=0, to=3, by=0.05)
  for (c in 1:4) { lines(x, c - x^2) }
}

plot.mtc.deviance <- function(x, auto.layout=TRUE, ...) {
  if (auto.layout) {
    par(mfrow=c(2,1))
  }
  mtc.devplot(x, ...)
  mtc.levplot(x, ...)
}

# Unrelated study effects model
mtc.model.use <- function(model) {
  model[['linearModel']] <- 'random'

  model[['data']] <- mtc.model.data(model)
  model[['data']][['nt']] <- NULL
  model[['data']][['t']] <- NULL
  model[['inits']] <- lapply(mtc.init(model), function(inits) {
    inits[-which(names(inits) == "sd.d")]
  })

  model[['code']] <- mtc.model.code(model, c(), '', template='gemtc.model.use.template.txt')

  monitors <- inits.to.monitors(model[['inits']][[1]])
  model[['monitors']] <- list(
    available=monitors,
    enabled=monitors[grep('^delta\\[', monitors)]
  )

    class(model) <- "mtc.model"

  model
}

mtc.model.name.use <- function(model) {
  "unrelated study effects"
}

func.param.matrix.use <- function(model, t1, t2) {
  stop('func.param.matrix.use not implemented')
}

#' @include ll-helper.counts.R
#' @include likelihoods.R

# Arm-level effect estimate (given a one-row data frame)
# Returns mean, standard deviation.
mtc.arm.mle.binom.logit <- function(data, k=0.5) {
  s <- unname(data['responders'] + k)
  f <- unname(data['sampleSize'] - s + 2 * k)
  c('mean'=log(s/f), 'sd'=sqrt(1/s + 1/f))
}

# Relative effect estimate (given a two-row data frame)
mtc.rel.mle.binom.logit <- function(data, correction.force=TRUE, correction.type="constant", correction.magnitude=1) {
  correction <- correction.counts(data, correction.force, correction.type, correction.magnitude)

  e1 <- mtc.arm.mle.binom.logit(data[1,], k=correction[1])
  e2 <- mtc.arm.mle.binom.logit(data[2,], k=correction[2])

  c(e2['mean'] - e1['mean'], sqrt(e1['sd']^2 + e2['sd']^2))
}

mtc.code.likelihood.binom.logit <- function(powerAdjust) {
  paste("logit(p[i, k]) <- $armLinearModel$", likelihood.code.binom[powerAdjust + 1], sep="\n")
}

fitted.values.parameter.binom.logit <- fitted.values.parameter.binom
deviance_fn.binom.logit <- deviance_fn.binom

scale.log.binom.logit <- function() { TRUE }
scale.name.binom.logit <- function() { "Odds Ratio" }

# Initial values outside this range result in probability 0 or 1 for the
# binomial, which may lead to BUGS/JAGS rejecting the data
inits.info.binom.logit <- function() {
  list(
    limits=c(-745, 36.8),
    param='mu',
    transform=identity)
}

required.columns.ab.binom.logit <- required.columns.counts
validate.data.binom.logit <- validate.data.counts

study.baseline.priors.binom.logit <- function() {
"for (i in studies.a) {
  mu[i] ~ dnorm(0, prior.prec)
}
"
}

context("ll.binom.logit")

test_that("mtc.arm.mle (default correction)", {
  expect_equal(mtc.arm.mle.binom.logit(c('responders'=0, 'sampleSize'=25)),
               c('mean'=log(0.5/25.5), 'sd'=sqrt(1/0.5 + 1/25.5)))

  expect_equal(mtc.arm.mle.binom.logit(c('responders'=12, 'sampleSize'=25)),
               c('mean'=log(12.5/13.5), 'sd'=sqrt(1/12.5 + 1/13.5)))

  expect_equal(mtc.arm.mle.binom.logit(c('responders'=25, 'sampleSize'=25)),
               c('mean'=log(25.5/0.5), 'sd'=sqrt(1/25.5 + 1/0.5)))
})

test_that("mtc.arm.mle (no correction)", {
  expect_equal(mtc.arm.mle.binom.logit(c('responders'=12, 'sampleSize'=25), k=0),
               c('mean'=log(12/13), 'sd'=sqrt(1/12 + 1/13)))
})

test_that("mtc.arm.mle (other correction)", {
  expect_equal(mtc.arm.mle.binom.logit(c('responders'=0, 'sampleSize'=25), k=0.05),
               c('mean'=log(0.05/25.05), 'sd'=sqrt(1/0.05 + 1/25.05)))

  expect_equal(mtc.arm.mle.binom.logit(c('responders'=12, 'sampleSize'=25), k=0.05),
               c('mean'=log(12.05/13.05), 'sd'=sqrt(1/12.05 + 1/13.05)))

  expect_equal(mtc.arm.mle.binom.logit(c('responders'=25, 'sampleSize'=25), k=0.05),
               c('mean'=log(25.05/0.05), 'sd'=sqrt(1/25.05 + 1/0.05)))
})

test_that("mtc.rel.mle (forced default correction)", {
  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=0, 'sampleSize'=25), c('responders'=3, 'sampleSize'=25))),
               c('mean'=log((3.5/22.5)/(0.5/25.5)), 'sd'=sqrt(1/3.5 + 1/0.5 + 1/22.5 + 1/25.5)))

  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=0, 'sampleSize'=25), c('responders'=0, 'sampleSize'=25))),
               c('mean'=0, 'sd'=sqrt(2/0.5 + 2/25.5)))
})

test_that("mtc.rel.mle (as-needed default correction)", {
  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=1, 'sampleSize'=25), c('responders'=3, 'sampleSize'=24)), correction.force=FALSE),
               c('mean'=log((3/21)/(1/24)), 'sd'=sqrt(1/3 + 1/1 + 1/21 + 1/24)))

  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=0, 'sampleSize'=25), c('responders'=3, 'sampleSize'=24)), correction.force=FALSE),
               c('mean'=log((3.5/21.5)/(0.5/25.5)), 'sd'=sqrt(1/3.5 + 1/0.5 + 1/21.5 + 1/25.5)))

  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=1, 'sampleSize'=25), c('responders'=24, 'sampleSize'=24)), correction.force=FALSE),
               c('mean'=log((24.5/0.5)/(1.5/24.5)), 'sd'=sqrt(1/24.5 + 1/1.5 + 1/0.5 + 1/24.5)))
})

test_that("mtc.rel.mle (alternative magnitude correction)", {
  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=0, 'sampleSize'=25), c('responders'=3, 'sampleSize'=25)), correction.magnitude=0.1),
               c('mean'=log((3.05/22.05)/(0.05/25.05)), 'sd'=sqrt(1/3.05 + 1/0.05 + 1/22.05 + 1/25.05)))
})

test_that("mtc.rel.mle (reciprocal correction)", {
  # no correction
  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=1, 'sampleSize'=25), c('responders'=3, 'sampleSize'=24)), correction.type="reciprocal", correction.force=FALSE),
               c('mean'=log((3/21)/(1/24)), 'sd'=sqrt(1/3 + 1/1 + 1/21 + 1/24)))

  # 1:2 group ratio (R = 2), correction for the control is R/(R+1) = 2/3, for the treatment 1/(R+1) = 1/3
  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=0, 'sampleSize'=50), c('responders'=3, 'sampleSize'=25)), correction.type="reciprocal"),
               c('mean'=log(((3+1/3)/(22+1/3))/((0+2/3)/(50+2/3))), 'sd'=sqrt(1/(3+1/3) + 1/(0+2/3) + 1/(22+1/3) + 1/(50+2/3))))

  # 1:4 group ratio (R = 4), correction for the control is R/(R+1) = 4/5, for the treatment 1/(R+1) = 1/5
  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=0, 'sampleSize'=100), c('responders'=3, 'sampleSize'=25)), correction.type="reciprocal"),
               c('mean'=log((3.2/22.2)/(0.8/100.8)), 'sd'=sqrt(1/3.2 + 1/0.8 + 1/22.2 + 1/100.8)))

  # 1:4 group ratio (R = 4), correction for the control is 0.1 R/(R+1) = 0.4/5, for the treatment 0.1/(R+1) = 0.1/5
  expect_equal(mtc.rel.mle.binom.logit(rbind(c('responders'=0, 'sampleSize'=100), c('responders'=3, 'sampleSize'=25)), correction.type="reciprocal", correction.magnitude=0.1),
               c('mean'=log((3.02/22.02)/(0.08/100.08)), 'sd'=sqrt(1/3.02 + 1/0.08 + 1/22.02 + 1/100.08)))
})

#'From a named vector with "BUGS format" names (v[1], m[2,3]), create a list of vectors and matrices
arrayize <- function(x) {
  # parse expressions into array indexes
  exprs <- parse(text=names(x))
  assgn <- lapply(exprs, function(expr) {
    if (inherits(expr, "name")) {
      list(name=as.character(expr), index=c())
    } else if (expr[[1]] == "[") {
      list(name=as.character(expr[[2]]), index=sapply(3:length(expr), function(i) { expr[[i]] }))
    } else {
      stop("Unrecognized expression")
    }
  })

  # find the unique variables and their dimension
  vars <- list()
  for (a in assgn) {
    name <- a[['name']]
    index <- a[['index']]
    if (!is.null(vars[[name]])) {
      stopifnot(length(index) == length(vars[[name]]))
      vars[[name]] <- pmax(vars[[name]], index)
    } else {
      vars[[name]] <- index
    }
  }

  # allocate the variables
  vars <- lapply(vars, function(dim) {
    if (length(dim) == 0) {
      NA
    } else if (length(dim) == 1) {
      rep(NA, dim[1])
    } else if (length(dim) == 2) {
      matrix(NA, nrow=dim[1], ncol=dim[2])
    } else {
      stop("higher dimensional objects not supported")
    }
  })

  # assign values
  for (i in 1:length(x)) {
    name <- assgn[[i]][['name']]
    index <- assgn[[i]][['index']]
    if (length(index) == 0) {
      vars[[name]] <- unname(x[i])
    } else if (length(index) == 1) {
      vars[[name]][index[1]] <- unname(x[i])
    } else if (length(index) == 2) {
      vars[[name]][index[1], index[2]] <- unname(x[i])
    }
  }

  vars[order(names(vars))]
}

context("arrayize")

test_that("it preserves scalar values", {
  expect_equal(arrayize(c("a"=3)), list("a"=3))
  expect_equal(arrayize(c("a"=3, "b"=2)), list("a"=3, "b"=2))
})

test_that("it parses vectors", {
  expect_equal(arrayize(c("a[1]"=3)), list("a"=3))
  expect_equal(arrayize(c("a[1]"=3, "a[2]"=4)), list("a"=c(3,4)))
  expect_equal(arrayize(c("a[2]"=4)), list("a"=c(NA,4)))
})

test_that("it parses matrices", {
  expect_equal(arrayize(c("m[1,2]"=8,"m[2,1]"=4)), list("m"=rbind(c(NA,8),c(4,NA))))
})

test_that("it handles multiple variables", {
  result <- arrayize(c("m[1,2]"=8,"a"=1,"beta[2]"=3,"m[2,1]"=4))
  expect_equal(result, list("a"=1, "beta"=c(NA,3), "m"=rbind(c(NA,8),c(4,NA))))
})

#' @include template.R

fixna <- function(x, v) {
  x[is.na(x)] <- v
  x
}

likelihood.code.binom <- list(
  read.template("gemtc.likelihood.binom.txt"),
  read.template("gemtc.likelihood.binom.power.txt"))

deviance_fn.binom <- function(data, val, alpha=1) {
  r <- data$r
  n <- data$n
  rfit <- val
  2 * alpha * (fixna(r * log(r / rfit), 0) + fixna((n - r) * log((n - r) / (n - rfit)), 0))
}

fitted.values.parameter.binom <- function() { "rhat" }

likelihood.code.poisson <- list(
  read.template("gemtc.likelihood.poisson.txt"),
  read.template("gemtc.likelihood.poisson.power.txt"))

deviance_fn.poisson <- function(data, val, alpha=1) {
  r <- data$r
  rfit <- val
  2 * alpha * ((rfit - r) + fixna(r * log(r / rfit), 0))
}

fitted.values.parameter.poisson <- function() { "theta" }

likelihood.code.normal <- list(
  read.template("gemtc.likelihood.normal.txt"),
  read.template("gemtc.likelihood.normal.power.txt"))

deviance_fn.normal <- function(data, val, alpha=1) {
  if (is.null(data[['e']])) {
    data[['e']] <- data[['s']] / sqrt(data[['n']])
  }
  alpha * (data[['m']] - val)^2 / data[['e']]^2
}

fitted.values.parameter.normal <- function() { "theta" }

get.row.groups <- function(data, group.labels) {
  if (is.factor(data[['group']])) {
    data[['group']] <- as.character(data[['group']])
  }
  groups <- rle(data[['group']])
  lapply(1:length(groups[['lengths']]), function(i) {
    v <- groups[['values']][i]
    l <- groups[['lengths']][i]
    i0 <- if (i == 1) 1 else sum(groups[['lengths']][1:max(1, (i - 1))]) + 1
    list(label=group.labels[v], data=data[i0:(i0 + l - 1), , drop=FALSE])
  })
}

add.group.label <- function(label, layout.row) {
  pushViewport(viewport(layout.pos.row=layout.row, layout.pos.col=1))
  if (!is.null(label)) {
    grid.draw(label)
  }
  popViewport()
  layout.row + 1
}

add.group <- function(columns, ci.data, layout.row) {
  nc <- length(columns)
  for (row in 1:length(ci.data[['labels']])) {
    for (col in columns) {
      if (!is.null(ci.data[['labels']][[row]][[col]])){
        pushViewport(viewport(layout.pos.row=layout.row, layout.pos.col=2 * which(columns == col) - 1))
        grid.draw(ci.data[['labels']][[row]][[col]])
        popViewport()
      }
    }
    if (!is.null(ci.data[['ci.plot']][[row]])) {
      pushViewport(viewport(layout.pos.row=layout.row, layout.pos.col=2*nc+1))
      grid.draw(ci.data[['ci.plot']][[row]])
      popViewport()
      pushViewport(viewport(layout.pos.row=layout.row, layout.pos.col=2*nc+3))
      grid.draw(ci.data[['ci.label']][[row]])
      popViewport()
    }
    layout.row <- layout.row + 1
  }
  layout.row
}

grob.ci <- function(pe, ci.l, ci.u, xrange, style) {
  if (is.na(pe) || is.na(ci.l) || is.na(ci.u)) {
    return(grob())
  }

  grob.pe <- NULL
  if(style[['pe.style']] == "circle") {
    grob.pe <- circleGrob(x=unit(pe, "native"), y=0.5, r=unit(0.2, "snpc"), gp=gpar(col="black"))
  } else { # Default is square
    grob.pe <- rectGrob(x=unit(pe, "native"), y=0.5, width=unit(0.2, "snpc"), height=unit(0.2, "snpc"), gp=gpar(fill="black",col="black"))
  }

  build.arrow <- function(ends) {
    arrow(ends=ends, length=unit(0.5, "snpc"))
  }

  arrow <- if (ci.l < xrange[[1]] && ci.u > xrange[[2]]) build.arrow("both")
          else if(ci.l < xrange[[1]]) build.arrow("first")
          else if (ci.u > xrange[[2]]) build.arrow("last")
          else NULL

  line <- linesGrob(x=unit(c(max(ci.l, xrange[[1]]), min(ci.u, xrange[[2]])), "native"), arrow=arrow, y=0.5, gp=gpar(lty=style[['lty']]))

  if (pe > xrange[1] && pe < xrange[2]) {
    ciGrob <- gTree(children=gList(
      line,
      grob.pe
    ))
  } else {
    ciGrob <- gTree(children=gList(
      line
    ))
  }
  ciGrob[['vp']] <- viewport(xscale=xrange)
  ciGrob
}

text_style  <- function(styles) {
  function(text, style) {
    ff <- styles[as.character(style), ,drop=TRUE][['font.weight']]
    ff <- if (is.na(ff)) "plain" else ff
    textGrob(text, x=unit(0, "npc"), just="left", gp=gpar(fontface=ff))
  }
}

draw.page <- function(ci.data, colwidth, rowheights, ci.label, grouped,
                      columns, column.groups, column.group.labels,
                      header.labels, text.fn, xrange, scale.trf, left.label,
                      right.label, center.label, draw.no.effect=TRUE) {
  columns.grouped <- !is.null(column.groups)
  row.offset <- if (columns.grouped) 2 else 1

  # Initialize plot and layout
  rowheight <- unit.c(unit(rep(1, row.offset), "lines"), rowheights, unit(c(0.5, 1), "lines"), unit(c(0.2, 1), "lines"))
  layout <- grid.layout(length(rowheight), length(colwidth), widths=colwidth, heights=rowheight)
  pushViewport(viewport(layout=layout))

  # Draw column labels (left-hand side)
  if (columns.grouped) {
    groups <- names(column.group.labels)
    for (group in groups) {
      label <- textGrob(column.group.labels[group])
      pushViewport(viewport(layout.pos.row=1, layout.pos.col=which(column.groups == group) * 2 + 1))
      grid.draw(label)
      popViewport()
    }
  }
  for (i in 1:length(header.labels)) {
    pushViewport(viewport(layout.pos.row=row.offset, layout.pos.col=2 * i - 1))
    grid.draw(header.labels[[i]])
    popViewport()
  }

  # CI column label
  pushViewport(viewport(layout.pos.row=row.offset, layout.pos.col=length(colwidth)))
  grid.draw(textGrob(ci.label))
  popViewport()

  # Main content
  nc <- length(columns)
  layout.row <- row.offset + 1
  groupNames <- names(ci.data)
  for (grp in 1:length(ci.data)) {
    if (grouped) {
      layout.row <- add.group.label(if (!is.na(groupNames[[grp]])) text.fn(groupNames[[grp]], 'group'), layout.row)
    }
    layout.row <- add.group(columns, ci.data[[grp]], layout.row)
  }
  nr <- layout.row

  if (draw.no.effect) {
      ## No-effect line
      pushViewport(viewport(layout.pos.col=2*nc+1, layout.pos.row=(row.offset+1):(nr), xscale=xrange))
      grid.lines(x=unit(c(0, 0), "native"), y=unit(c(0, 1), "npc"))
      popViewport()
  }

  # Axis and ticks
  pushViewport(viewport(layout.pos.col=2*nc+1, layout.pos.row=nr, xscale=xrange))
  grid.lines(x=unit(c(0, 1), "npc"), y=unit(1, "npc"))
  grid.lines(x=unit(0, "npc"), y=unit(c(0, 1), "npc"))
  grid.lines(x=unit(1, "npc"), y=unit(c(0, 1), "npc"))
  popViewport()

  # Tick labels
  pushViewport(viewport(layout.pos.col=2*nc+1, layout.pos.row=nr+1, xscale=xrange))
  if(draw.no.effect) {
      grid.draw(textGrob(scale.trf(0), just="center", x=unit(0, "native")))
  }
  grid.draw(textGrob(scale.trf(xrange[1]), just="center", x=unit(0, "npc")))
  grid.draw(textGrob(scale.trf(xrange[2]), just="center", x=unit(1, "npc")))
  popViewport()

  x.offset <- if(draw.no.effect) 0 else mean(xrange)

  pushViewport(viewport(layout.pos.col=2*nc+1, layout.pos.row=nr+3, xscale=xrange))
  if (!is.null(left.label))
    grid.draw(textGrob(left.label, just="right", x=sum(unit(x.offset, "native"), unit(-0.2, "npc"))))
  if (!is.null(right.label))
    grid.draw(textGrob(right.label, just="left", x=sum(unit(x.offset, "native"), unit(0.2, "npc"))))
  if (!is.null(center.label))
    grid.draw(textGrob(center.label, just="center", x=sum(unit(x.offset, "native"), unit(0, "npc"))))  
  popViewport()
}

blobbogram.styles.default <- function() {
  styles <- data.frame(
    style=c('normal', 'pooled', 'group'),
    font.weight=c('plain', 'plain', 'bold'),
    row.height=c(1, 1, 1.5),
    pe.style=c('circle', 'square', NA),
    pe.scale=c(FALSE, FALSE, NA),
    lty=c(1,1,NA))
  rownames(styles) <- styles[['style']]
  styles
}

formatCI <- function(ci, scale.trf=identity, digits=2) {
  if (all(!is.na(ci))) {
    fmt <- lapply(ci, function(x) { formatC(scale.trf(x), digits=digits, zero.print="0", flag="#") })
    paste(fmt[['pe']], " (", fmt[['ci.l']], ", ", fmt[['ci.u']], ")", sep="")
  } else {
    "NA"
  }
}

nice.value <- function(x, round.fun, log.scale) {
  # Scale transform and its inverse
  scale.trf <- if (log.scale) exp else identity
  scale.inv <- if (log.scale) log else identity

  # Round to a single significant digit, according to round.fun
  y <- scale.trf(x)
  p <- 10^floor(log10(abs(y)))
  l <- scale.inv(round.fun(y / p) * p)
  if (is.na(l) || !is.finite(l)) x else l
}

blobbogram <- function(data, id.label='Study', ci.label="Mean (95% CI)",
                       left.label=NULL, right.label=NULL, center.label=NULL,
                       log.scale=FALSE, xlim=NULL, styles=NULL,
                       grouped=TRUE, group.labels=NULL,
                       columns=NULL, column.labels=NULL,
                       column.groups=NULL, column.group.labels=NULL,
                       digits=2,
                       ask=dev.interactive(orNone=TRUE),
                       draw.no.effect=TRUE) {

  devAskNewPage(FALSE)
  plot.new()
  devAskNewPage(ask)

  grouped <- !is.null(group.labels) && isTRUE(grouped)

  if (is.null(styles)) {
    styles <- blobbogram.styles.default()
  }
  text.fn <- text_style(styles)

  # Rewrite input: split into groups
  data <- if (grouped) {
    get.row.groups(data, group.labels)
  } else {
    list(list(label=NULL, data=data))
  }

  columns.grouped <- !is.null(column.groups)

  # Add default ('id') column
  columns <- c('id', columns)
  column.labels <- c(id.label, column.labels)
  
  # Scale transform and its inverse
  scale.trf <- if (log.scale) exp else identity
  scale.inv <- if (log.scale) log else identity

  # Round to a single significant digit, according to round.fun
  nice <- function(x, round.fun) {
    nice.value(x, round.fun, log.scale)
  }

  # Calculate plot range
  xrange <- if (is.null(xlim)) {
    ci.l <- do.call(c, lapply(data, function(datagrp) { datagrp[['data']][, 'ci.l', drop=TRUE]}))
    ci.u <- do.call(c, lapply(data, function(datagrp) { datagrp[['data']][, 'ci.u', drop=TRUE]}))
    c(min(nice(min(ci.l, na.rm=TRUE), floor), 0), max(nice(max(ci.u, na.rm=TRUE), ceiling), 0))
  } else {
    xlim
  }

  rowToGrobs <- function(row) {
    lapply(row, function(x) {
        if (!is.na(x)) text.fn(x, row['style'])
      })
  }

  header.labels <- rowToGrobs(column.labels)
  forest.data <- lapply(data, function(datagrp) {
      # Create labels
      labels <- apply(datagrp[['data']], 1, rowToGrobs)

      ci.data <- lapply(1:nrow(datagrp[['data']]), function(i) {
        # Create CI plots
        fmt <- datagrp[['data']][i, c('pe', 'ci.l', 'ci.u'), drop=TRUE]
        ci <- grob.ci(fmt[['pe']], fmt[['ci.l']], fmt[['ci.u']], xrange,
          styles[as.character(datagrp[['data']][i, 'style', drop=TRUE]), , drop=TRUE])

        # Create CI interval labels (right side)
        text <- formatCI(fmt, scale.trf, digits=digits)
        label <-  text.fn(text, as.character(datagrp[['data']][i, 'style', drop=TRUE]))

        list(ci=ci, label=label)
      })
      ci.plot <- lapply(ci.data, function(x) { x[['ci']] })
      ci.label <- lapply(ci.data, function(x) { x[['label']] })

      list(labels=labels, ci.plot=ci.plot, ci.label=ci.label)
    })
  names(forest.data) <- lapply(data, function(grp) { grp[['label']] })

  forest.data.index <- function(fd, idx) {
    list(labels=fd[['labels']][idx], ci.plot=fd[['ci.plot']][idx], ci.label=fd[['ci.label']][idx])
  }

  if (columns.grouped) {
    column.group.labels.grob <- rowToGrobs(column.group.labels)
  }

  # Calculate column widths
  colgap <- unit(3, "mm")
  colwidth <- do.call(unit.c, lapply(columns, function(col) {
    col <- c(header.labels[col], do.call(c, lapply(forest.data, function(grp) { sapply(grp[['labels']], function(row) { row[col] }) })))
    col <- col[!sapply(col, is.null)]
    unit.c(max(unit(rep(1, length(col)), "grobwidth", col)), colgap)
  }))

  # Adjust column widths so column group labels fit
  if (columns.grouped) {
    groups <- names(column.group.labels)
    if (is.null(groups)) {
      groups <- 1:length(column.group.labels)
    }
    for (group in groups) {
      gl <- column.group.labels.grob[group]
      select <- column.groups == group
      for (i in which(select)) {
        colwidth[[2 * i + 1]] <- max(unit(1.0 / sum(select), "grobwidth", gl), colwidth[[2 * i + 1]])
      }
    }
  }

  # Adjust column widths so row group labels fit
  if (grouped) {
    labels <- lapply(group.labels, text.fn, 'group')
    for (i in 1:length(columns))
    colwidth[[2 * i - 1]] <- max(max(unit(rep(1.0 / length(columns), length(labels)), "grobwidth", labels)), colwidth[[2 * i - 1]])
  }

  graphwidth <- unit(5, "cm")
  all.ci.labels <- do.call(c, lapply(forest.data, function(x) { x[['ci.label']] }) )

  ci.colwidth <- max(unit(rep(1, length(all.ci.labels)), "grobwidth", all.ci.labels))
  colwidth <- unit.c(colwidth, graphwidth, colgap, ci.colwidth)

  groupHeight <- function(grp) {
    if (grouped) {
      unit(
        c(styles['group', 'row.height'],
          styles[as.character(grp[['data']][['style']]), 'row.height', drop=TRUE]
        ), "lines")
    } else {
      unit(styles[as.character(grp[['data']][['style']]), 'row.height', drop=TRUE], "lines")
    }
  }

  groupHeightNPC <- function(grp) {
    convertY(sum(groupHeight(grp)), "npc", valueOnly=TRUE)
  }

  # divide data into pages
  height <- sum(unit.c(unit(rep(1, if (columns.grouped) 2 else 1), "lines"), unit(c(0.5, 1), "lines"), unit(c(0.2, 1), "lines")))
  space <- 1.0 - convertY(height, "npc", valueOnly=TRUE)

  pages <- list(list())
  height <- 0
  for (i in 1:length(data)) {
    myHeight <- groupHeightNPC(data[[i]])
    if (myHeight > space) {
      # Create (sub) groups that fit
      i0 <- 1
      i1 <- 1
      while (i0 <= nrow(data[[i]][['data']])) {
        if (height / space > 0.7) { # create a new page
          pages <- c(pages, list(list()))
          height <- 0
        }
        while (groupHeightNPC(list(data=data[[i]][['data']][i0:i1, , drop=FALSE])) < (space - height) && i1 <= nrow(data[[i]][['data']])) {
          i1 <- i1 + 1
        }
        block <- list(list(
          forest.data=forest.data.index(forest.data[[i]], i0:(i1 - 1)),
          rowheight=groupHeight(list(data=data[[i]][['data']][i0:(i1 - 1), , drop=FALSE]))
          ))
        names(block) <- names(forest.data)[i]
        pages[[length(pages)]] <- c(pages[[length(pages)]], block)
        i0 <- i1
        height <- space
      }
    } else if (height + myHeight > space) { # new page
      height <- myHeight
      block <- list(list(
        forest.data=forest.data[[i]],
        rowheight=groupHeight(data[[i]])
        ))
      names(block) <- names(forest.data)[i]
      pages <- c(pages, list(block))
    } else { # append to this page
      height <- height + myHeight
      block <- list(list(
        forest.data=forest.data[[i]],
        rowheight=groupHeight(data[[i]])
        ))
      names(block) <- names(forest.data)[i]
      pages[[length(pages)]] <- c(pages[[length(pages)]], block)
    }
  }

  # Now plot each group
  for (i in 1:length(pages)) {
    page <- pages[[i]]
    if (length(page) > 0) {
      rowheights <- do.call(unit.c, lapply(page, function(grp) { grp[['rowheight']] }))
      fd <- lapply(page, function(grp) { grp[['forest.data']] })
      dev.hold()
      if (i > 1) grid.newpage()
      draw.page(fd, colwidth, rowheights, ci.label,
                grouped, columns, column.groups,
                column.group.labels, header.labels,
                text.fn, xrange, scale.trf,
                left.label, right.label, center.label, draw.no.effect)
      dev.flush()
    }
  }
}

#' Stop if the results were not derived from an internally consistent model type
stopIfNotConsistent <- function(result, method) {
  allowedTypes <- c('consistency', 'regression')
  if (!(tolower(result[['model']][['type']]) %in% allowedTypes)) stop(paste("Can only apply", method, "to models of the following types:", paste(allowedTypes, collapse=", ")))
}

required.columns.counts <- function() {
  c('r'='responders', 'n'='sampleSize')
}

validate.data.counts <- function(data.ab) {
  stopifnot(all(data.ab[['sampleSize']] >= data.ab[['responders']]))
  stopifnot(all(data.ab[['sampleSize']] > 0))
  stopifnot(all(data.ab[['responders']] >= 0))
}

correction.counts <- function(data, correction.force, correction.type, correction.magnitude) {
  correction.need <-
    data[1,'responders'] == 0 || data[1,'responders'] == data[1,'sampleSize'] ||
    data[2,'responders'] == 0 || data[2,'responders'] == data[2,'sampleSize']

  groupRatio <- if (correction.type == "reciprocal") {
    data[1,'sampleSize'] / data[2,'sampleSize']
  } else {
    1
  }

  if (correction.force || correction.need) {
    correction.magnitude * c(groupRatio/(groupRatio+1), 1/(groupRatio+1))
  } else {
    c(0, 0)
  }
}

#' @param obj Numeric vector of objective coefficients
#' @param mat Numeric matrix of constraint coefficients
#' @param rhs Numeric vector of constraint right-hand sides
#' @param eq Logical vector; TRUE for equality, FALSE for <=
#' @param max Logical scalar; TRUE for maximize, FALSE for minimize
solveLP <- function(obj, mat, rhs, eq, max=FALSE) {
  # Solution using RCDD (results in memory corruption...)
  # constraints <- cbind(eq, rhs, -mat)
  # sol <- rcdd::lpcdd(constraints, obj, minimize=!max)
  # if (sol$solution.type == "Optimal") {
  #   sol$optimal.value
  # } else if (sol$solution.type == "DualInconsistent" || sol$solution.type == "StrucDualInconsistent") {
  #   if (max) { +Inf } else { -Inf }
  # } else {
  #   stop(paste("LP solver:", sol$solution.type))
  # }

  # Solution using Rglpk
  GLP_OPT <- 5    # solution is optimal
  GLP_UNBND <- 6  # solution is unbounded
  status <- c("solution is undefined", "solution is feasible", "solution is infeasible", "no feasible solution exists", "solution is optimal", "solution is unbounded")

  dir <- c("<=", "==")[eq + 1]
  m <- ncol(mat)
  # Explicitly set bounds to (-Inf, +Inf) - the default is [0, +Inf)
  bounds <- list(lower=list(ind=1:m, val=rep(-Inf, m)),
                 upper=list(ind=1:m, val=rep(+Inf, m)))
  sol <- Rglpk::Rglpk_solve_LP(obj, mat, dir, rhs, max=max, bounds=bounds, control = list(canonicalize_status=FALSE))
  if (sol$status == GLP_OPT) {
    sol$optimum
  } else if (sol$status == GLP_UNBND) {
    if (max) { +Inf } else { -Inf }
  } else {
    stop(paste("LP solver:", status[sol$status]))
  }
}

# Returns a matrix with one row for each of the given pairs,
# and columns 'mean' and 'sd' describing their relative effect
rel.mle.ab <- function(data, model, pairs) {
  matrix(sapply(1:nrow(pairs), function(i) {
    sel1 <- data[['treatment']] == pairs[['t1']][i]
    sel2 <- data[['treatment']] == pairs[['t2']][i]
    columns <- ll.call("required.columns.ab", model)
    ll.call("mtc.rel.mle", model, as.matrix(data[sel1 | sel2, columns, drop=FALSE]))
  }), ncol=2, byrow=TRUE, dimnames=list(NULL, c('mean', 'sd')))
}

# Returns a matrix with one row for each of the given pairs,
# and columns 'mean' and 'sd' describing their relative effect
rel.mle.re <- function(data, pairs) {
  # the mean vector
  mu <- data[['diff']]
  mu[1] <- 0.0 # the baseline relative to itself

  # construct the covariance matrix
  se <- data[['std.err']]
  sigma <- matrix(se[1]^2, nrow=length(se), ncol=length(se))
  diag(sigma) <- se^2
  sigma[1,] <- 0
  sigma[,1] <- 0

  # construct the permutation matrix
  b <- sapply(1:nrow(pairs), function(i) {
    x <- rep(0, length(se))
    x[data[['treatment']] == pairs[['t1']][i]] <- -1
    x[data[['treatment']] == pairs[['t2']][i]] <- 1
    x
  })
  b <- matrix(b, nrow=length(se))

  mu <- t(b) %*% mu
  sigma <- t(b) %*% sigma %*% b
  rval <- cbind(mu, sqrt(diag(sigma)))
  colnames(rval) <- c('mean', 'sd')
  rval
}

# Guess the measurement scale based on differences observed in the data set
guess.scale <- function(model) {
  data.ab <- model[['network']][['data.ab']]
  max.ab <- 0
  if (!is.null(data.ab)) {
    max.ab <- max(sapply(unique(data.ab[['study']]), function(study) {
      pairs <- mtc.treatment.pairs(mtc.study.design(model[['network']], study))
      max(abs(rel.mle.ab(data.ab[data.ab[['study']] == study, , drop=TRUE], model, pairs)[,'mean']))
    }))
  }
  data.re <- model[['network']][['data.re']]
  max.re <- 0
  if (!is.null(data.re)) {
    max.re <- max(sapply(unique(data.re[['study']]), function(study) {
      pairs <- mtc.treatment.pairs(mtc.study.design(model[['network']], study))
      max(abs(rel.mle.re(data.re[data.re[['study']] == study, , drop=TRUE], pairs)[,'mean']))
    }))
  }

  max(max.ab, max.re)
}

# Test calculation of all-pairs MLE estimates (needed for priors, starting values)
context("rel.mle.[ab|re]")

test_that("a single pair returns a one-row matrix", {
  data <- data.frame(treatment=c("A", "B"), mean=c(1.0, 2.0), std.err=c(0.5/4, 0.5/4))
  model <- list("likelihood"="normal", "link"="identity")
  pairs <- data.frame(t1=data$treatment[1], t2=data$treatment[2])
  expected <- matrix(c('mean'=1.0, 'sd'=sqrt(2*0.125^2)), nrow=1, ncol=2)
  colnames(expected) <- c('mean', 'sd')
  expect_that(rel.mle.ab(data, model, pairs), equals(expected))
})

test_that("two pairs return a two-row matrix", {
  data <- data.frame(treatment=c("A", "B", "C"), mean=c(1.0, 2.0, 2.5), std.err=c(0.5/4, 0.5/4, 1.0/4), stringsAsFactors=T)
  model <- list("likelihood"="normal", "link"="identity")
  ts <- data$treatment
  pairs <- data.frame(t1=forcats::fct_c(ts[1], ts[1]), t2=forcats::fct_c(ts[2], ts[3]))
  expected <- matrix(c(1.0, sqrt(2*0.125^2), 1.5, sqrt(0.125^2 + 0.25^2)), ncol=2, byrow=TRUE)
  colnames(expected) <- c('mean', 'sd')
  expect_that(rel.mle.ab(data, model, pairs), equals(expected))
})

test_that("calculating pairs for relative effect data transforms the mvnorm", {
  data <- read.table(textConnection("
study  treatment  diff  std.err
s07    A          NA    0.50
s07    B          -2.3  0.72
s07    D          -0.9  0.69"), header=T, stringsAsFactors=T)
  ts <- data$treatment
  pairs <- data.frame(t1=forcats::fct_c(ts[3], ts[3]), t2=forcats::fct_c(ts[1], ts[2]))
  expected <- matrix(c(0.9, 0.69, -1.4, sqrt(0.72^2+0.69^2-2*0.50^2)), ncol=2, byrow=TRUE)
  colnames(expected) <- c('mean', 'sd')
  expect_that(rel.mle.re(data, pairs), equals(expected))
})

test_that("calculating pairs for relative effect data handles 1-pair case", {
  data <- read.table(textConnection("
study  treatment  diff  std.err
s07    A          NA    0.50
s07    B          -2.3  0.72
s07    D          -0.9  0.69"), header=T, stringsAsFactors=T)
  ts <- data$treatment
  pairs <- data.frame(t1=forcats::fct_c(ts[3], ts[3]), t2=forcats::fct_c(ts[1], ts[2]))
  expected <- matrix(c(0.9, 0.69, -1.4, sqrt(0.72^2+0.69^2-2*0.50^2)), ncol=2, byrow=TRUE)
  colnames(expected) <- c('mean', 'sd')
  expect_that(rel.mle.re(data, pairs), equals(expected))
})

test_that("calculating pairs for relative effect data handles missing treatments", {
  data <- read.table(textConnection("
study  treatment  diff  std.err
s07    A          NA    0.50
s07    B          -2.3  0.72
s07    D          -0.9  0.69
s08    C          NA    0.3"), header=T, stringsAsFactors=T)
  ts <- data$treatment
  pairs <- data.frame(t1=ts[3], t2=ts[2])
  expected <- matrix(c(-1.4, sqrt(0.72^2+0.69^2-2*0.50^2)), ncol=2, byrow=TRUE)
  colnames(expected) <- c('mean', 'sd')
  expect_that(rel.mle.re(data[data$study=="s07",], pairs), equals(expected))
})

test_that("guess.scale handles relative effect data", {
  data <- read.table(textConnection("
study  treatment  diff  std.err
s07    A          NA    0.50
s07    B          -2.3  0.72
s07    D          -0.9  0.69"), header=T, stringsAsFactors=T)
  network <- mtc.network(data.re=data)

  model <- list(
    network = network,
    likelihood = 'normal',
    link = 'identity')
  expect_that(guess.scale(model), equals(2.3))
})

test_that("guess.scale not confused by unrealized study levels", {
  network <- list(treatments=data.frame(id=as.factor(c("A", "B"))), data.ab = data.frame(
    study=factor(c("1", "1"), levels=c("1", "2")), treatment=as.factor(c("A", "B")), responders=c(1, 3), sampleSize=c(10, 10)))
  expect_that(guess.scale(list(network=network, likelihood='binom', link='logit')), equals(1.083687, tolerance=1e-6))

  network <- list(treatments=data.frame(id=as.factor(c("A", "B"))), data.re = data.frame(
    study=factor(c("1", "1"), levels=c("1", "2")), treatment=as.factor(c("A", "B")), diff=c(NA, 1), std.err=c(NA, 1)))
  expect_that(guess.scale(list(network=network, likelihood='binom', link='logit')), equals(1))
})

# Consistency model
mtc.model.consistency <- function(model) {
  style.tree <- function(tree) {
    tree <- set.edge.attribute(tree, 'arrow.mode', value=2)
    tree <- set.edge.attribute(tree, 'color', value='black')
    tree <- set.edge.attribute(tree, 'lty', value=1)
    tree
  }
  model[['tree']] <-
    style.tree(minimum.diameter.spanning.tree(mtc.network.graph(model[['network']])))

  model[['data']] <- mtc.model.data(model)
  model[['inits']] <- mtc.init(model)

  model[['code']] <- mtc.model.code(model, mtc.basic.parameters(model), consistency.relative.effect.matrix(model))

  monitors <- inits.to.monitors(model[['inits']][[1]])
  model[['monitors']] <- list(
    available=monitors,
    enabled=c(monitors[grep('^d\\.', monitors)], monitors[grep('^sd.d$', monitors)])
  )

  class(model) <- "mtc.model"

  model
}

mtc.model.name.consistency <- function(model) {
  "consistency"
}

func.param.matrix.consistency <- function(model, t1, t2) {
  tree.relative.effect(model[['tree']], t1, t2)
}

consistency.relative.effect.matrix <- function(model) {
  # Generate list of linear expressions
  params <- mtc.basic.parameters(model)
  tree <- model[['tree']]
  re <- tree.relative.effect(tree, V(tree)[1], t2=NULL)
  expr <- apply(re, 2, function(col) { paste(sapply(which(col != 0), function(i) {
    paste(if (col[i] == -1) "-" else "", params[i], sep="")
  }), collapse = " + ") })
  expr <- sapply(1:length(expr), function(i) { paste('d[1, ', i + 1, '] <- ', expr[i], sep='') })
  expr <- c('d[1, 1] <- 0', expr, 'for (i in 2:nt) {\n\tfor (j in 1:nt) {\n\t\td[i, j] <- d[1, j] - d[1, i]\n\t}\n}')
  paste(expr, collapse="\n")
}

# connect the MDST of disconnected components of the graph
connect.mds.forest <- function(g) {
  clu <- clusters(g)
  trees <- lapply(1:clu$no, function(i) {
    sg <- induced.subgraph(g, which(clu$membership == i))
    if (clu$csize[i] == 1) {
      list(root=V(sg)$name, edges=matrix(NA,nrow=0,ncol=2))
    } else {
      tree <- minimum.diameter.spanning.tree(sg)
      vnames <- V(tree)$name
      edges <- matrix(vnames[get.edges(tree, E(tree))], ncol=2)
      root <- vnames[degree(tree, mode="in")==0]
      list(root=root, edges=edges)
    }
  })

  root <- trees[[1]]$root
  h <- graph.empty()
  h <- h + vertex(V(g)$name)
  for (tree in trees) {
    h <- h + edges(t(tree$edges))
    if (tree$root != root) {
      h <- h + edge(root, tree$root)
    }
  }

  h
}


edge.length <- function(g, e) {
  w <- get.edge.attribute(g, 'weight', e)
  if (is.null(w)) 1 else w
}

minimum.diameter.spanning.tree <- function(graph) {
  l <- function(edge) { edge.length(graph, edge) }

  # Determine the center of the graph
  f <- absolute.one.center(graph)
  dc <- f['t'] - 0.5 * l(f['e'])
  v <- ends(graph, f['e'], names=FALSE)
  if (dc < 0 || (dc == 0 && v[1] < v[2])) {
    edgelist <- v
  } else {
    edgelist <- rev(v)
  }

  # For each vertex, if the center is on (v[1], v[2]), determine which of the shortest paths through v[1] or v[2] to the center is shorter
  d <- shortest.paths(graph, mode="all")
  t1 <- f['t']
  t2 <- l(f['e']) - t1
  v.rem <- V(graph)[!(1:length(V(graph)) %in% v)]
  pairs <- sapply(v.rem, function(u) {
    d2 <- c(t1 + d[v[1], u, drop=TRUE], t2 + d[v[2], u, drop=TRUE])
    if (d2[1] < d2[2] || (d2[1] == d2[2] && v[1] < v[2])) {
      c(v[1], u)
    } else {
      c(v[2], u)
    }
  })
  pairs <- as.matrix(pairs) # necessary if there is only one pair

  h <- graph.empty()
  h <- h + vertex(V(graph))
  V(h)$name <- V(graph)$name
  h <- h + edge(edgelist)
  if (ncol(pairs) > 0 && nrow(pairs) > 0) {
    for (i in 1:ncol(pairs)) {
      p <- get.shortest.paths(graph, pairs[1, i, drop=TRUE], pairs[2, i, drop=TRUE], mode="all")
      if (is.list(p) && "vpath" %in% names(p)) { p <- p$vpath[[1]] } else { p <- p[[1]] }
      if (length(p) == 2) {
        h <- h + edge(p) # bug in one-edge paths?
      } else {
        h <- h + path(p)
      }
    }
  }
  simplify(h)
}

# Find the absolute 1-center of an undirected graph.
#
# Let e = (u,v) be an edge of the graph, and l(e) the length of e.
# Let x(e) be a point along the edge, with t = t(x(e)) \in [0, l(e)] the distance of x(e) from u.
# Then, for any point x on G, d(v, x) is the length of the shortest path in G between the vertex v and point x.
# Define F(x) = \max_{v \in V} d(v, x), and x* = \arg\min_{x on G} F(x).
# x* is the absolute 1-center of G and F(x*) is the absolute 1-radius of G.
#
# Algorithm from <a href="http://www.jstor.org/pss/2100910">Kariv and Hakimi (1979), SIAM J Appl Math 37 (3): 513-538</a>.
#
# (Note: in the paper, the weight of an edge is referred to as its length l(e), and vertices can also have weights w(v).
# Only the vertex-unweighted algorithm is implemented. Edge-weighted graphs are supported, however.)
absolute.one.center <- function(g) {
  f <- sapply(E(g), function(edge) { local.center(g, edge) })
  v <- sapply(E(g), function(edge) { ends(g, edge, names=FALSE) })
  i <- order(f['r',], v[1,], v[2,])[1]
  f[, i, drop=TRUE]
}

# A few conventions:
#  - a point along an edge is represented by a pair x = (e, t) where e is the edge index and t is the
#  distance from the left vertex
#  - a center is a triple f = (e, t, r) where (e, t) is a point and r is the graph's radius around that
#  point
local.center <- function(graph, edge) {
  d <- shortest.paths(graph)

  # L(v): vertices ordered by distance from v
  L <- t(apply(d, 2, function(x) { order(-x, V(graph)) }))

  # l(e): length of edge e
  l <- function(edge) { edge.length(graph, edge) }

  # left and right vertices of an edge
  vr <- function(edge) { ends(graph, edge, names=FALSE)[1] }
  vs <- function(edge) { ends(graph, edge, names=FALSE)[2] }

  # de(x, v): distance between point x and vertex v
  de <- function(x, v) {
    min(
      x['t'] + d[vr(x['e']), v, drop=TRUE],
      l(x['e']) - x['t'] + d[vs(x['e']), v, drop=TRUE]
    )
  }

  # Given an edge e = (e0, e1) and vertices u and v, find the distance t* \in
  # (0, l(e)) from e0 where D_e(u, t*) = D_e(v, t*), and D_e(u, t*) and
  # D_e(v, t*) have opposite signs (if it exists).
  intersect <- function(e, u, v) {
    e0 <- vr(e)
    e1 <- vs(e)
    lu <- d[e0, u, drop=TRUE]
    ru <- d[e1, u, drop=TRUE]
    lv <- d[e0, v, drop=TRUE]
    rv <- d[e1, v, drop=TRUE]

    if (lu == lv || ru == rv) { # they coincide or intersect only at the edge
      NA
    } else if (lu > lv && ru > rv) { # u dominates v
      NA
    } else if (lu < lv && ru < rv) { # v dominates u
      NA
    } else {
      t1 <- 0.5 * (rv - lu + l(e));
      t2 <- 0.5 * (ru - lv + l(e));
      if (t1 + lu <= l(e) - t1 + ru) {
        t1
      } else {
        t2
      }
    }
  }

  # Step 1: treatment of t = 0 and t = l(e)
  step1 <- function() {
    xr <- c('e' = edge, 't' = 0)
    xs <- c('e' = edge, 't' = l(edge))
    dr <- de(xr, L[vr(edge), 1, drop=TRUE])
    ds <- de(xs, L[vs(edge), 1, drop=TRUE])
    f <- if (dr <= ds) c(xr, 'r' = dr) else c(xs, 'r' = ds)

    if (L[vr(edge), 1, drop=TRUE] == L[vs(edge), 1, drop=TRUE]) f
    else step3(f, L[vr(edge), 1, drop=TRUE], 1)
  }

  # Step 3: treatment of vertices v s.t. D_e(v, 0) = D_e(v_1, 0)
  # f: The suspected center
  # vm: Vertex to consider
  # i: Index of the last-treated vertex
  # Returns the local center
  step3 <- function(f, vm, i) {
    v <- L[vr(f['e']), i + 1, drop=TRUE]
    xr <- c(f['e'], 't' = 0)
    xs <- c(f['e'], 't' = l(f['e']))
    if (de(xs, v) != de(xs, vm)) {
      step4(f, vm, i + 1)
    } else if (de(xr, v) > de(xs, vm)) {
      step3(f, v, i + 1) # v_m <- v*
    } else {
      step3(f, vm, i + 1)
    }
  }

  step4 <- function(f, vm, i) {
    vbar <- vm
    if (i == nrow(d)) {
      step8(f, vbar)
    } else {
      vm <- L[vr(f['e']), i, drop=TRUE]
      step5(f, vm, vbar, i)
    }
  }

  # Step 5: find all vertices v s.t. D_e(v, 0) = D_e(v_i, 0) and find the corresponding v_m.
  step5 <- function(f, vm, vbar, i) {
    v <- L[vr(f['e']), i + 1, drop=TRUE]
    xr <- c(f['e'], 't' = 0)
    xs <- c(f['e'], 't' = l(f['e']))
    if (de(xr, v) != de(xr, vm)) {
      step6(f, vm, vbar, i + 1)
    } else if (de(xs, v) > de(xs, vm)) {
      step5(f, v, vbar, i + 1) # v_m <- v*
    } else {
      step5(f, vm, vbar, i + 1)
    }
  }

  # Step 6: treatment of the point (e, t_m).
  step6 <- function(f, vm, vbar, i) {
    tm <- intersect(f['e'], vm, vbar)

    if (is.na(tm)) {
      step7(f, vm, vbar, i)
    } else {
      xt <- c(f['e'], 't' = tm)
      ft <- c(xt, 'r' = de(xt, vm))
      if (ft['r'] < f['r']) {
        step7(ft, vm, vbar, i)
      } else {
        step7(f, vm, vbar, i)
      }
    }
  }

  # Step 7: proceed to the next vertex.
  step7 <- function(f, vm, vbar, i) {
    xs <- c(f['e'], 't' = l(f['e']))
    if (de(xs, vm) > de(xs, vbar)) {
      vbar <- vm
    }

    if (i == nrow(d)) {
      step8(f, vbar)
    } else {
      step5(f, L[vr(f['e']), i, drop=TRUE], vbar, i)
    }
  }

  step8 <- function(f, vbar) {
    vn <- L[vr(f['e']), nrow(d), drop=TRUE]
    tm <- intersect(f['e'], vn, vbar)

    if (is.na(tm)) {
      f
    } else {
      xt <- c(f['e'], 't'=tm)
      ft <- c(xt, 'r' = de(xt, vn))
      if (ft['r'] < f['r']) {
        ft
      } else {
        f
      }
    }
  }

  step1()
}

context("Minimum diameter spanning tree")

lc_collate <- Sys.getlocale("LC_COLLATE")
Sys.setlocale("LC_COLLATE", "C")

# Simple line graph with 5 vertices, 4 edges
test_that("absolute.one.center of a 5-vertex line graph is on the middle vertex", {
  n <- 5
  g <- graph.edgelist(matrix(c(1:(n-1), 2:n), ncol=2))
  expect_that(local.center(g, 1), equals(c('e' = 1, 't' = 1, 'r' = 3)))
  expect_that(local.center(g, 2), equals(c('e' = 2, 't' = 1, 'r' = 2)))
  expect_that(local.center(g, 3), equals(c('e' = 3, 't' = 0, 'r' = 2)))
  expect_that(local.center(g, 4), equals(c('e' = 4, 't' = 0, 'r' = 3)))
  expect_that(absolute.one.center(g), equals(c('e' = 2, 't' = 1, 'r' = 2)))
})

# With edge lengths (weights)
test_that("absolute.one.center takes edge weights into account", {
  n <- 5
  g <- graph.edgelist(matrix(c(1:(n-1), 2:n), ncol=2))
  g <- set.edge.attribute(g, 'weight', value=c(1, 1, 1, 0.5))
  expect_that(local.center(g, 2), equals(c('e' = 2, 't' = 0.75, 'r' = 1.75)))
  expect_that(local.center(g, 3), equals(c('e' = 3, 't' = 0, 'r' = 2)))
  expect_that(absolute.one.center(g), equals(c('e' = 2, 't' = 0.75, 'r' = 1.75)))
})

# Simple line graph with 6 vertices, 5 edges
test_that("absolute.one.center of 6-vertex line graph is on the middle of the middle edge", {
  n <- 6
  g <- graph.edgelist(matrix(c(1:(n-1), 2:n), ncol=2))
  expect_that(local.center(g, 1), equals(c('e' = 1, 't' = 1, 'r' = 4)))
  expect_that(local.center(g, 2), equals(c('e' = 2, 't' = 1, 'r' = 3)))
  expect_that(local.center(g, 3), equals(c('e' = 3, 't' = 0.5, 'r' = 2.5)))
  expect_that(local.center(g, 4), equals(c('e' = 4, 't' = 0, 'r' = 3)))
  expect_that(absolute.one.center(g), equals(c('e' = 3, 't' = 0.5, 'r' = 2.5)))
})

# Slightly less simple graph
test_that("absolute.one.center of a non-line graph is correct", {
  g <- graph.edgelist(rbind(c(1, 2), c(1, 3), c(1, 4), c(4, 5)))
  expect_that(local.center(g, 1), equals(c('e' = 1, 't' = 0, 'r' = 2)))
  expect_that(local.center(g, 2), equals(c('e' = 2, 't' = 0, 'r' = 2)))
  expect_that(local.center(g, 3), equals(c('e' = 3, 't' = 0.5, 'r' = 1.5)))
  expect_that(local.center(g, 4), equals(c('e' = 4, 't' = 0, 'r' = 2)))
  expect_that(absolute.one.center(g), equals(c('e' = 3, 't' = 0.5, 'r' = 1.5)))
})

# Graph with a cycle
test_that("absolute.once.center of a cyclic graph is correct", {
  g <- graph.edgelist(rbind(c(1, 2), c(1, 3), c(1, 4), c(1, 5), c(4, 5)))
  expect_that(local.center(g, 1), equals(c('e' = 1, 't' = 0, 'r' = 1)))
  expect_that(local.center(g, 2), equals(c('e' = 2, 't' = 0, 'r' = 1)))
  expect_that(local.center(g, 3), equals(c('e' = 3, 't' = 0, 'r' = 1)))
  expect_that(local.center(g, 4), equals(c('e' = 4, 't' = 0, 'r' = 1)))
  expect_that(local.center(g, 5), equals(c('e' = 5, 't' = 0, 'r' = 2)))
  expect_that(absolute.one.center(g), equals(c('e' = 1, 't' = 0, 'r' = 1)))
})

# Graph with only one edge
test_that("absolute.once.center of one-edge graph is correct", {
  g <- graph.edgelist(rbind(c(1, 2)))
  expect_that(local.center(g, 1), equals(c('e' = 1, 't' = 0.5, 'r' = 0.5)))
  expect_that(absolute.one.center(g), equals(c('e' = 1, 't' = 0.5, 'r' = 0.5)))
})

# Complex real network (luades-thrombolytic)
test_that("absolute.one.center of complex graph is correct", {
  v <- as.factor(c("ASPAC", "AtPA", "Ret", "SK", "SKtPA", "Ten", "tPA", "UK"))
  e <- list(
    c("ASPAC", "AtPA"),
    c("ASPAC", "tPA"),
    c("ASPAC", "SK"),
    c("AtPA", "Ten"),
    c("AtPA", "SKtPA"),
    c("AtPA", "Ret"),
    c("AtPA", "SK"),
    c("AtPA", "UK"),
    c("Ret", "SK"),
    c("SK", "UK"),
    c("SK", "SKtPA"),
    c("SK", "tPA"),
    c("tPA", "UK")
  )
  e <- do.call(rbind, lapply(e, function(x) { factor(x, levels=levels(v)) }))
  g <- graph.edgelist(e)
  V(g)$name <- levels(v)
  expect_that(absolute.one.center(g), equals(c('e' = 1, 't' = 0.5, 'r' = 1.5)))

  tree <- minimum.diameter.spanning.tree(g)
  expect_that(degree(tree, mode="out"), equals(c(
    'ASPAC'=3, 'AtPA'=4, 'Ret'=0, 'SK'=0, 'SKtPA'=0, 'Ten'=0, 'UK'=0, 'tPA'=0)))
  expect_that(degree(tree, mode="in"), equals(c(
    'ASPAC'=0, 'AtPA'=1, 'Ret'=1, 'SK'=1, 'SKtPA'=1, 'Ten'=1, 'UK'=1, 'tPA'=1)))
  expect_that(tree['AtPA', 'SK'], equals(0))
  expect_that(tree['ASPAC', 'AtPA'], equals(1))
})

Sys.setlocale("LC_COLLATE", lc_collate)

#' @include forest.R

relative.effect.table <- function(result, covariate=NA) {
  ts <- as.character(result[['model']][['network']][['treatments']][['id']])
  tbl <- array(NA, dim=c(length(ts), length(ts), 3), dimnames=list(ts, ts, c("2.5%", "50%", "97.5%")))
  comps <- combn(ts, 2)

  for (i in 1:ncol(comps)) {
    comp <- comps[,i]
    samples <- as.matrix(relative.effect(result, comp[1], comp[2], preserve.extra=FALSE, covariate=covariate)$samples)
    q <- quantile(samples, prob=c(0.025, 0.5, 0.975))
    tbl[comp[1], comp[2],] <- unname(q)
    q.inv <- c(-q[3], -q[2], -q[1])
    tbl[comp[2], comp[1],] <- unname(q.inv)
  }

  attr(tbl, "model") <- result[['model']]
  attr(tbl, "covariate") <- covariate
  class(tbl) <- "mtc.relative.effect.table"

  tbl
}

relative.effect.table.to.matrix <- function(x, formatNumber=formatC) {
  y <- apply(x, c(1,2), function(x) {
    if (all(!is.na(x))) {
      paste0(formatNumber(x[2]), " (", formatNumber(x[1]), ", ", formatNumber(x[3]), ")")
    } else {
      NA
    }
  })
  diag(y) <- rownames(x)
  y
}

as.data.frame.mtc.relative.effect.table <- function(x, ...) {
  as.data.frame(relative.effect.table.to.matrix(x, paste), stringsAsFactors=FALSE)
}

print.mtc.relative.effect.table <- function(x, ...) {
  scale.log <- if (ll.call('scale.log', attr(x, 'model'))) 'Log ' else ''
  scale.name <- ll.call('scale.name', attr(x, 'model'))
  y <- relative.effect.table.to.matrix(x)

  cat(paste0(scale.log, scale.name, " (95% CrI)\n\n"))
  write.table(format(y, justify="centre"), quote=FALSE, row.names=FALSE, col.names=FALSE)
}

forest.mtc.relative.effect.table <- function(x, t1=rownames(x)[1], use.description=FALSE, ...) {
  i1 <- which(rownames(x) == t1)
  stats <- x[i1, -i1,]

  model <- attr(x, 'model')
  network <- model[['network']]

  ts <- rownames(stats)
  if (use.description) {
    ts <- treatment.id.to.description(network, ts)
    t1 <- treatment.id.to.description(network, t1)
  }

  data <- data.frame(id=ts,
                     pe=stats[,2], ci.l=stats[,1], ci.u=stats[,3],
                     style="normal")

  blobbogram(data,
    columns=c(), column.labels=c(),
    id.label="",
    ci.label=paste(ll.call('scale.name', model), "(95% CrI)"),
    log.scale=ll.call('scale.log', model),
    center.label=paste("Compared with", t1),
    ...)
}

context("relative.effect.table")

test_that("it works for the smoking example", {
  smoking_result <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))
  smoking_table <- relative.effect.table(smoking_result)

  # Check dimnames
  expect_that(rownames(smoking_table), equals(c("A", "B", "C", "D")))
  expect_that(colnames(smoking_table), equals(c("A", "B", "C", "D")))

  # Check that the diagonal contains NA
  expect_that(diag(smoking_table[,,2]), is_equivalent_to(as.numeric(rep(NA, 4))))

  # Check that off-diagonal entries contain the quantiles
  q <- function(q1, q2, q3) { c("2.5%"=q1, "50%"=q2, "97.5%"=q3) }
  expect_that(smoking_table[1,2,], equals(q(-0.29846826342809, 0.490982134423406, 1.34066639613713)))
  expect_that(smoking_table[1,3,], equals(q(0.387798548149361, 0.827333271108623, 1.3530539185826)))
  expect_that(smoking_table[1,4,], equals(q(0.269236199821778, 1.09825953831406, 2.00604408009687)))
  expect_that(smoking_table[2,3,], equals(q(-0.480946561958643, 0.341121326321452, 1.17021895949004)))
  expect_that(smoking_table[2,4,], equals(q(-0.308278534656184, 0.604352628567083, 1.57902423190838)))
  expect_that(smoking_table[3,4,], equals(q(-0.532179016795455, 0.261896715203374, 1.11556777887809)))
  expect_that(smoking_table[2,1,], equals(q(-1.34066639613713, -0.490982134423406, 0.29846826342809)))

  expect_that(attr(smoking_table, "model"), equals(smoking_result[['model']]))
  expect_that(smoking_table, is_a("mtc.relative.effect.table"))
})

#' @include ll-helper.counts.R
#' @include likelihoods.R

# Arm-level effect estimate (given a one-row data frame)
# Returns mean, standard deviation.
mtc.arm.mle.binom.log <- function(data, k=0.5) {
  s <- unname(data['responders'] + k)
  f <- unname(data['sampleSize'] + 2 * k)
  c('mean'=log(s/f), 'sd'=sqrt(1/s - 1/f))
}

# Relative effect estimate (given a two-row data frame)
mtc.rel.mle.binom.log <- function(data, correction.force=TRUE, correction.type="constant", correction.magnitude=1) {
  correction <- correction.counts(data, correction.force, correction.type, correction.magnitude)

  e1 <- mtc.arm.mle.binom.log(data[1,], correction[1])
  e2 <- mtc.arm.mle.binom.log(data[2,], correction[2])
  c(e2['mean'] - e1['mean'], sqrt(e1['sd']^2 + e2['sd']^2))
}

mtc.code.likelihood.binom.log <- function(powerAdjust) {
  paste("log(p[i, k]) <- min($armLinearModel$, -1E-16)", likelihood.code.binom[powerAdjust + 1], sep="\n")
}

fitted.values.parameter.binom.log <- fitted.values.parameter.binom
deviance_fn.binom.log <- deviance_fn.binom

scale.log.binom.log <- function() { TRUE }
scale.name.binom.log <- function() { "Risk Ratio" }

# Initial values outside this range result in probability 0 or 1 for the
# binomial, which may lead to BUGS/JAGS rejecting the data
inits.info.binom.log <- function() {
  list(
    limits=c(-745, -1E-7),
    param='mu',
    transform=identity)
}

required.columns.ab.binom.log <- required.columns.counts
validate.data.binom.log <- validate.data.counts

study.baseline.priors.binom.log <- function() {
"for (i in studies.a) {
  mu[i] ~ dnorm(log(0.5), prior.prec) T(,0)
}
"
}

context("ll.binom.log")

test_that("mtc.arm.mle (default correction)", {
  expect_that(mtc.arm.mle.binom.log(c('responders'=0, 'sampleSize'=25)),
              equals(c('mean'=log(0.5/26), 'sd'=sqrt(1/0.5 - 1/26))))

  expect_that(mtc.arm.mle.binom.log(c('responders'=12, 'sampleSize'=25)),
              equals(c('mean'=log(12.5/26), 'sd'=sqrt(1/12.5 - 1/26))))

  expect_that(mtc.arm.mle.binom.log(c('responders'=25, 'sampleSize'=25)),
              equals(c('mean'=log(25.5/26), 'sd'=sqrt(1/25.5 - 1/26))))
})

test_that("mtc.arm.mle (no correction)", {
  expect_equal(mtc.arm.mle.binom.log(c('responders'=12, 'sampleSize'=25), k=0),
               c('mean'=log(12/25), 'sd'=sqrt(1/12 - 1/25)))
})

test_that("mtc.arm.mle (other correction)", {
  expect_equal(mtc.arm.mle.binom.log(c('responders'=0, 'sampleSize'=25), k=0.05),
               c('mean'=log(0.05/25.1), 'sd'=sqrt(1/0.05 - 1/25.1)))
})

test_that("mtc.rel.mle (forced default correction)", {
  expect_that(mtc.rel.mle.binom.log(rbind(c('responders'=0, 'sampleSize'=25), c('responders'=3, 'sampleSize'=25))),
              equals(c('mean'=log(7), 'sd'=sqrt(1/3.5 + 1/0.5 - 2/26))))

  expect_that(mtc.rel.mle.binom.log(rbind(c('responders'=0, 'sampleSize'=25), c('responders'=0, 'sampleSize'=25))),
              equals(c('mean'=0, 'sd'=sqrt(2/0.5 - 2/26))))
})

test_that("mtc.rel.mle (as-needed default correction)", {
  expect_equal(mtc.rel.mle.binom.log(rbind(c('responders'=1, 'sampleSize'=25), c('responders'=3, 'sampleSize'=24)), correction.force=FALSE),
               c('mean'=log((3/24)/(1/25)), 'sd'=sqrt(1/3 - 1/24 + 1/1 - 1/25)))

  expect_equal(mtc.rel.mle.binom.log(rbind(c('responders'=0, 'sampleSize'=25), c('responders'=3, 'sampleSize'=24)), correction.force=FALSE),
               c('mean'=log((3.5/25)/(0.5/26)), 'sd'=sqrt(1/3.5 - 1/25 + 1/0.5 - 1/26)))

  expect_equal(mtc.rel.mle.binom.log(rbind(c('responders'=1, 'sampleSize'=25), c('responders'=24, 'sampleSize'=24)), correction.force=FALSE),
               c('mean'=log((24.5/25)/(1.5/26)), 'sd'=sqrt(1/24.5 - 1/25 + 1/1.5 - 1/26)))
})

test_that("mtc.rel.mle (alternative magnitude correction)", {
  expect_equal(mtc.rel.mle.binom.log(rbind(c('responders'=0, 'sampleSize'=25), c('responders'=3, 'sampleSize'=25)), correction.magnitude=0.1),
               c('mean'=log((3.05/25.1)/(0.05/25.1)), 'sd'=sqrt(1/3.05 - 1/25.1 + 1/0.05 - 1/25.1)))
})

test_that("mtc.rel.mle (reciprocal correction)", {
  # no correction
  expect_equal(mtc.rel.mle.binom.log(rbind(c('responders'=1, 'sampleSize'=25), c('responders'=3, 'sampleSize'=24)), correction.type="reciprocal", correction.force=FALSE),
               c('mean'=log((3/24)/(1/25)), 'sd'=sqrt(1/3 - 1/24 + 1/1 - 1/25)))

  # 1:2 group ratio (R = 2), correction for the control is R/(R+1) = 2/3, for the treatment 1/(R+1) = 1/3
  expect_equal(mtc.rel.mle.binom.log(rbind(c('responders'=0, 'sampleSize'=50), c('responders'=3, 'sampleSize'=25)), correction.type="reciprocal"),
               c('mean'=log(((3+1/3)/(25+2/3))/((0+2/3)/(50+4/3))), 'sd'=sqrt(1/(3+1/3) + 1/(0+2/3) - 1/(25+2/3) - 1/(50+4/3))))

  # 1:4 group ratio (R = 4), correction for the control is R/(R+1) = 4/5, for the treatment 1/(R+1) = 1/5
  expect_equal(mtc.rel.mle.binom.log(rbind(c('responders'=0, 'sampleSize'=100), c('responders'=3, 'sampleSize'=25)), correction.type="reciprocal"),
               c('mean'=log((3.2/25.4)/(0.8/101.6)), 'sd'=sqrt(1/3.2 + 1/0.8 - 1/25.4 - 1/101.6)))

  # 1:4 group ratio (R = 4), correction for the control is 0.1 R/(R+1) = 0.4/5, for the treatment 0.1/(R+1) = 0.1/5
  expect_equal(mtc.rel.mle.binom.log(rbind(c('responders'=0, 'sampleSize'=100), c('responders'=3, 'sampleSize'=25)), correction.type="reciprocal", correction.magnitude=0.1),
               c('mean'=log((3.02/25.04)/(0.08/100.16)), 'sd'=sqrt(1/3.02 + 1/0.08 - 1/25.04 - 1/100.16)))
})

#' @include mtc.model.consistency.R
#' @include template.R

mtc.model.regression <- function(model, regressor) {
  style.tree <- function(tree) {
    tree <- set.edge.attribute(tree, 'arrow.mode', value=2)
    tree <- set.edge.attribute(tree, 'color', value='black')
    tree <- set.edge.attribute(tree, 'lty', value=1)
    tree
  }
  model[['tree']] <-
    style.tree(minimum.diameter.spanning.tree(mtc.network.graph(model[['network']])))

  if (is.vector(regressor)) {
    regressor <- as.list(regressor)
  }

  if (is.null(regressor[['variable']]) || is.null(regressor[['coefficient']]) ||
      (is.null(regressor[['control']]) && is.null(regressor[['classes']]))) {
    stop("Regressor specification incomplete")
  }

  if (!is.null(regressor[['control']]) && !is.null(regressor[['classes']])) {
    stop("Can not specify both 'control' and 'classes'")
  }

  by.class <- !is.null(regressor[['classes']])

  if (!by.class) {
    control <- as.treatment.factor(regressor[['control']], model[['network']])
    if (is.na(control)) {
      stop(paste0("Control treatment \"", regressor[['control']], "\" not found"))
    }
    regressor[['control']] <- control
  }

  if (!(regressor[['coefficient']] %in% c('shared', 'unrelated', 'exchangeable'))) {
    stop(paste0("Coefficient type \"", regressor[['coefficient']], "\" not supported"))
  }
  if (by.class && regressor[['coefficient']] == 'unrelated') {
    stop("Unrelated coefficients not supported for regression by class")
  }
  if (by.class && regressor[['coefficient']] == 'exchangeable') {
    stop("Exchangeable coefficients not yet implemented for regression by class")
  }

  if (by.class) {
    classes <- regressor[['classes']]
    stopifnot(is.list(classes))
    if (is.null(names(classes)) || anyDuplicated(names(classes))) {
      names(classes) <- 1:length(classes)
    }
    classes <- lapply(classes, as.treatment.factor, model[['network']])
    class.trts <- do.call(c, regressor[['classes']])
    all.trts <- model[['network']][['treatments']][['id']]
    stopifnot(!anyDuplicated(class.trts), setequal(class.trts, all.trts))
    trt.to.class <- regressionClassMap(classes)
  }

  studyData <- model[['network']][['studies']]
  if (is.null(studyData) || is.null(studyData[[regressor[['variable']]]])) {
    stop(paste0("Regressor variable \"", regressor[['variable']], "\" not found"))
  }

  # make sure the sort order of x is correct
  x <- studyData[[regressor[['variable']]]] 
  names(x) <- studyData[['study']]
  studies <- mtc.studies.list(model[['network']])[['values']]
  x <- x[studies]

  if (any(is.na(x))) {
    stop("NA values for the covariate are not supported")
  } else if (length(unique(x)) < 2) {
    stop("The covariate is degenerate (has < 2 unique values)")
  } else if (all(x %in% 0:1)) { # binary covariate
    model[['regressor']] <- c(regressor, type='binary', center=mean(x), scale=1)
  } else if (is.numeric(x)) { # continuous covariate
    model[['regressor']] <- c(regressor, type='continuous', center=mean(x), scale=2*sd(x))
  } else { # unsupported covariate
    stop("The covariate must be either binary (0/1) or numeric")
  }
  x <- (x - model[['regressor']][['center']]) / model[['regressor']][['scale']]
  model[['data']] <- mtc.model.data(model)
  model[['data']][['x']] <- x
  if (!by.class) {
    model[['data']][['reg.control']] <- control
  } else {
    model[['data']][['reg.classes']] <- trt.to.class
    model[['data']][['reg.nclasses']] <- length(classes)
  }
  model[['inits']] <- mtc.init(model)

  reg.prior.tpl <- '\n# Regression priors\nreg.prior.prec <- pow(om.scale, -2)\nfor (k in c(1:(reg.control-1), (reg.control+1):nt)) {\n  $regPrior$\n}\nbeta[reg.control] <- 0\n'
  priors <- list(
    'shared'=paste0(template.block.sub(reg.prior.tpl, 'regPrior', 'beta[k] <- B'), 'B ~ dt(0, reg.prior.prec, 1)\n'),
    'unrelated'=template.block.sub(reg.prior.tpl, 'regPrior', 'beta[k] ~ dt(0, reg.prior.prec, 1)'),
    'exchangeable'=paste0(template.block.sub(reg.prior.tpl, 'regPrior', 'beta[k] ~ dnorm(B, reg.tau)'), 'B ~ dt(0, reg.prior.prec, 1)\nreg.sd ~ dunif(0, om.scale)\nreg.tau <- pow(reg.sd, -2)'))

  priors.classes <- list(
    'shared'='\n# Regression priors\nreg.prior.prec <- pow(om.scale, -2)\nfor (k in 1:nt) {\n  beta[k] <- B[reg.classes[k]]\n}\nB[1] <- 0\nfor (k in 2:reg.nclasses) {\n  B[k] ~ dt(0, reg.prior.prec, 1)\n}')

  nt <- model[['data']][['nt']]
  reg.monitors <- c(regressionParams(regressor, nt, length(classes)), if(regressor[['coefficient']] == 'exchangeable') 'reg.sd')

  model[['code']] <- mtc.model.code(model, mtc.basic.parameters(model), consistency.relative.effect.matrix(model),
                                    linearModel='delta[i, k] + (beta[t[i, k]] - beta[t[i, 1]]) * x[i]',
                                    regressionPriors=if (by.class) priors.classes[[regressor[['coefficient']]]] else priors[[regressor[['coefficient']]]])

  monitors <- inits.to.monitors(model[['inits']][[1]])
  model[['monitors']] <- list(
    available=c(monitors, reg.monitors),
    enabled=c(monitors[grep('^d\\.', monitors)], monitors[grep('^sd.d$', monitors)], reg.monitors)
  )

  class(model) <- "mtc.model"

  model
}

mtc.model.name.regression <- function(model) {
  "meta-regression"
}

func.param.matrix.regression <- function(model, t1, t2) {
  tree.relative.effect(model[['tree']], t1, t2)
}

# Unrelated mean effects model
mtc.model.ume <- function(model) {
  network <- model[['network']]

  studies <- mtc.studies.list(network)
  na <- studies[['lengths']]
  studies <- studies[['values']]
  if (any(na > 2)) {
    warning("The Unrelated Mean Effects model does not handle multi-arm trials correctly.")
  }
  
  # these comparisons may contain duplicates due to the baselines of the RE data
  comparisons <- mtc.comparisons.baseline(network)
  # the basic parameters are the comparisons without duplicates
  basicParameters <- unique(apply(comparisons, 1, function(comparison) {
    t1 <- as.character(comparison['t1'])
    t2 <- as.character(comparison['t2'])
    if (t1 < t2) {
      data.frame('t1'=t1, 't2'=t2, stringsAsFactors=FALSE)
    } else {
      data.frame('t1'=t2, 't2'=t1, stringsAsFactors=FALSE)
    }
  }))
  basicParameters <- do.call(rbind, basicParameters)
  basicParameters[['t1']] <- as.treatment.factor(basicParameters[['t1']], network)
  basicParameters[['t2']] <- as.treatment.factor(basicParameters[['t2']], network)

  model[['graph']] <- graph.create(
    network[['treatments']][['id']],
    basicParameters,
    arrow.mode=2, color='black', lty=1)

  model[['data']] <- mtc.model.data(model)
  model[['data']][['nt']] <- NULL
  model[['inits']] <- mtc.init(model)

  model[['code']] <- mtc.model.code(model, mtc.basic.parameters(model), sparse.relative.effect.matrix(model, comparisons))

  monitors <- inits.to.monitors(model[['inits']][[1]])
  model[['monitors']] <- list(
    available=monitors,
    enabled=c(monitors[grep('^d\\.', monitors)], monitors[grep('^sd.d$', monitors)])
  )

  class(model) <- "mtc.model"

  model
}

mtc.model.name.ume <- function(model) {
  "unrelated mean effects"
}

func.param.matrix.ume <- function(model, t1, t2) {
  g <- model[['graph']]
  if((is.null(t2) || length(t2) == 0) && length(t1) == 1) {
    t2 <- V(g)[V(g) != as.numeric(t1)]
  }
  if(length(t1) > length(t2)) t2 <- rep(t2, length.out=length(t1))
  if(length(t2) > length(t1)) t1 <- rep(t1, length.out=length(t2))
  pairs <- matrix(c(t1, t2), ncol=2)
  paths <- apply(pairs, 1, function(rel) {
    edge_pos <- igraph::get.edge.ids(model[['graph']], rel)
    edge_neg <- igraph::get.edge.ids(model[['graph']], rev(rel))
    edge <- max(edge_pos, edge_neg)
    if (!edge) {
      stop(paste("The requested comparison ",
             V(g)[rel[2]]$name, " vs ", V(g)[rel[1]]$name,
             " is not present in the UME model."))
    }
    expr <- rep(0, length(igraph::E(g)))
    expr[edge] <- if (edge_pos) 1 else -1
    expr})

  # Ensure paths is a matrix, since apply() will simplify to a vector if
  # either ncol==1 or nrow==1
  paths <- matrix(as.numeric(paths), ncol=length(t1), nrow=length(E(g)))

  colnames(paths) <-  apply(pairs, 1, function(pair) {
    pair <- V(g)[pair]$name
    paste('d', pair[1], pair[2], sep='.')
  })
  paths
}

sparse.relative.effect.matrix <- function(model, comparisons) {
  ts <- model[['network']][['treatments']][['id']]
  nt <- length(ts)
  x <- sapply(1:nrow(comparisons), function(k) {
    i <- as.numeric(comparisons[k, 't1'])
    j <- as.numeric(comparisons[k, 't2'])
    if (model[['graph']][i, j, sparse=FALSE, drop=TRUE]) {
      paste("d[", i, ", ", j, "] <- d.", ts[i], ".", ts[j], sep="")
    } else {
      paste("d[", i, ", ", j, "] <- -d.", ts[j], ".", ts[i], sep="")
    }
  })
  paste(x, collapse="\n")
}

mtc.comparisons.baseline <- function(network) {
  baseline.pairs <- function(treatments) {
    n <- length(treatments)
    t1 <- rep(treatments[1], n - 1)
    t2 <- treatments[2:n]
    data.frame(t1=t1, t2=t2)
  }
  data <- mtc.merge.data(network)

  # Identify the unique "designs" (treatment combinations)
  design <- function(study) { mtc.study.design(network, study) }
  designs <- unique(lapply(unique(data[['study']]), design))

  # Generate all pair-wise comparisons from each "design"
  comparisons <- do.call(rbind, lapply(designs, baseline.pairs))

  # Ensure the output comparisons are unique and always in the same order
  comparisons <- unique(comparisons)
  comparisons <- comparisons[order(comparisons[['t1']], comparisons[['t2']]), , drop=FALSE]
  row.names(comparisons) <- NULL
  comparisons[['t1']] <- as.treatment.factor(comparisons[['t1']], network)
  comparisons[['t2']] <- as.treatment.factor(comparisons[['t2']], network)
  comparisons
}

all_pair_matrix <- function(m) {
  do.call(rbind, lapply(1:(m - 1), function(i) {
    do.call(rbind, lapply((i + 1):m, function(j) {
      c(i, j)
    }))
  }))
}

filter.network <- function(network, filter, filter.ab=filter, filter.re=filter) {
  data.ab <- if (!is.null(network[['data.ab']])) {
    network[['data.ab']][apply(network[['data.ab']], 1, filter.ab), , drop=FALSE]
  }
  data.re <- if (!is.null(network[['data.re']])) {
    network[['data.re']][apply(network[['data.re']], 1, filter.re), , drop=FALSE]
  }
  mtc.network(data=data.ab, data.re=data.re, treatments=network$treatments)
}

decompose.variance <- function(V) {
  na <- ncol(V)
  J <- matrix(1, ncol=na, nrow=na)
  # Pseudo-inverse Laplacian
  Lt <- -0.5 * (V - (1 / na) * (V %*% J + J %*% V) + (1 / na^2) * (J %*% V %*% J))
  # Laplacian
  L <- solve(Lt - J/na) + J/na

  prec <- -L
  diag(prec) <- Inf
  1/prec
}

# Decompose trials based on a consistency model and study.samples
decompose.trials <- function(result) {
  decompose.study <- function(samples, study) {
    na <- ncol(samples) + 1
    samples <- cbind(0, samples)
    mu <- sapply(1:na, function(i) {
      sapply(1:na, function(j) {
        mean(samples[ , i, drop=TRUE] - samples[ , j, drop=TRUE])
      })
    })
    # Effective variances
    V <- sapply(1:na, function(i) {
      sapply(1:na, function(j) {
        var(samples[ , i, drop=TRUE] - samples[ , j, drop=TRUE])
      })
    })

    var <- decompose.variance(V)
    se <- sqrt(var)
    if (any(is.nan(se))) {
      stop(paste(paste0("Decomposed variance ill-defined for ", study, ". Most likely the USE did not converge:"), paste(capture.output(print(var)), collapse="\n"), sep="\n"))
    }

    pairs <- all_pair_matrix(na)
    list(
      mu = apply(pairs, 1, function(p) { mu[p[1], p[2], drop=TRUE] }),
      se = apply(pairs, 1, function(p) { se[p[1], p[2], drop=TRUE] })
    )
  }

  study.samples <- as.matrix(result[['samples']])
  studies <- unique(mtc.merge.data(result[['model']][['network']])[['study']])

  decomposed <- lapply(1:length(studies), function(i) {
    study <- mtc.study.design(result[['model']][['network']], studies[i])
    study <- study[!is.na(study)]
    na <- length(study)
    colIndexes <- grep(paste("delta[", i, ",", sep=""), colnames(study.samples), fixed=TRUE)
    stopifnot(length(colIndexes) == (na - 1)) # A bug in WinBUGS caused this to happen -- repeated variables
    effects <- if (na > 2) {
      data <- decompose.study(
        study.samples[, colIndexes, drop=FALSE], studies[i])
      ts <- matrix(study[all_pair_matrix(na)], ncol=2)
      list(m=data[['mu']], e=data[['se']], t=ts)
    } else {
      samples <- study.samples[ , colIndexes, drop=FALSE]
      list(m=apply(samples, 2, mean), e=apply(samples, 2, sd), t=matrix(study, nrow=1))
    }

    # make the baseline treatment consistent (i.e. in the same order as the
    # basic parameters)
    for (k in 1:length(effects[['m']])) {
      t1 <- effects$t[k, 1]
      t2 <- effects$t[k, 2]
      if (t1 > t2) {
        effects$t[k, 1] <- t2
        effects$t[k, 2] <- t1
        effects$m[k] <- -effects$m[k]
      }
    }
    effects
  })

  # (mu1, prec1): posterior parameters
  mu1 <- lapply(decomposed, function(x) { x[['m']] })
  sigma1 <- lapply(decomposed, function(x) { x[['e']] })

  ts <- lapply(decomposed, function(x) { x[['t']] })

  studyNames <- mtc.studies.list(result[['model']][['network']])[['values']]
  names(ts) <- studyNames
  names(mu1) <- studyNames
  names(sigma1) <- studyNames

  list(t=ts, m=mu1, e=sigma1)
}

# decomposes the given network's multi-arm trials into
# a series of (approximately) equivalent two-arm trials
decompose.network <- function(network, result) {
  # find all multi-arm trials
  data <- mtc.merge.data(network)
  data[['study']] <- as.character(data[['study']])
  studies <- unique(data[['study']])
  studies <- studies[sapply(studies, function(study) { sum(data[['study']] == study) > 2 })]
  # check if all studies are multi-arm
  all.multiarm <- length(studies) == length(unique(data[['study']]))
  
  data <- decompose.trials(result)
  data.re <- do.call(rbind, lapply(studies, function(study) {
    do.call(rbind, lapply(1:length(data[['m']][[study]]), function(j) {
      ts <- data[['t']][[study]][j, , drop=TRUE]
      m <- data[['m']][[study]][j]
      e <- data[['e']][[study]][j]
      rbind(
        data.frame(study=paste(study, ts[1], ts[2], sep="__"), treatment=ts[1], diff=NA, std.err=NA, stringsAsFactors=FALSE),
        data.frame(study=paste(study, ts[1], ts[2], sep="__"), treatment=ts[2], diff=m, std.err=e, stringsAsFactors=FALSE)
      )
    }))
  }))
  
  if (all.multiarm) {
      mtc.network(data.ab = network[['data.ab']], data.re = data.re, treatments = network$treatments)
  } else {
      ta.network <- filter.network(network, function(row) { !(row['study'] %in% studies) })
      ta.data.re <- ta.network[['data.re']][ , c('study','treatment','diff','std.err')]
      mtc.network(data.ab=ta.network[['data.ab']], data.re=rbind(ta.data.re, data.re), treatments=network$treatments)
  }
}

mtc.anohe <- function(network, ...) {
  network <- fix.network(network)

  result.use <- mtc.model.run(network, type='use', ...)

  network.decomp <- decompose.network(network, result=result.use)
  result.ume <- mtc.model.run(network.decomp, type='ume', ...)

  result.cons <- mtc.model.run(network, type='consistency', ...)

  result <- list(result.cons=result.cons, result.ume=result.ume, result.use=result.use)
  class(result) <- "mtc.anohe"
  result
}

print.mtc.anohe <- function(x, ...) {
  cat("Analysis of heterogeneity (mtc.anohe) object\n")
  for (name in names(x)) {
    cat(paste("$", name, ": ", class(x[[name]]), "\n", sep=""))
  }
}

plot.mtc.anohe <- function(x, ask=dev.interactive(orNone=TRUE), ...) {
  cat("Analysis of heterogeneity -- convergence plots\n")

  cat("Unrelated Study Effects (USE) model:\n")
  plot(x[['result.use']], ask=ask, ...)
  cat("Unrelated Mean Effects (UME) model:\n")
  par(ask=ask)
  plot(x[['result.ume']], ask=ask, ...)
  cat("Consistency model:\n")
  par(ask=ask)
  plot(x[['result.cons']], ask=ask, ...)
}

i.squared <- function (mu, se, x, df.adj=-1) {
  stopifnot(is.numeric(mu))
  stopifnot(is.numeric(se))
  stopifnot(is.numeric(x))
  stopifnot(is.numeric(df.adj))
  stopifnot(length(mu) == length(se))
  stopifnot(length(mu) == length(x))
  dev <- (mu - x)^2 # squared deviance
  q <- sum(dev * 1/se^2) # Cochran's Q
  100 * max(0, (q - length(mu) - df.adj) / q) # I^2
}

summary.mtc.anohe <- function(object, ...) {
  result.use <- object[['result.use']]
  result.ume <- object[['result.ume']]
  result.cons <- object[['result.cons']]
  network <- result.cons[['model']][['network']]

  se <- decompose.trials(result.use)
  se[['s']] <- lapply(names(se[['e']]), function(s) { rep(s, length(se[['e']][[s]])) })
  se[['s']] <- do.call(c, se[['s']])
  se[['t']] <- do.call(rbind, se[['t']])
  se[['m']] <- do.call(c, se[['m']])
  se[['e']] <- do.call(c, se[['e']])
  studyEffects <- data.frame(study=se[['s']], t1=se[['t']][,1], t2=se[['t']][,2], pe=se[['m']], ci.l=se[['m']]-1.96*se[['e']], ci.u=se[['m']]+1.96*se[['e']], stringsAsFactors=FALSE)
  rownames(studyEffects) <- NULL

  ume.samples <- as.matrix(result.ume[['samples']])
  varNames <- colnames(ume.samples)
  varNames <- varNames[grep('^d\\.', varNames)]
  ume.samples <- ume.samples[,varNames,drop=FALSE]
  comps <- extract.comparisons(varNames)
  qs <- apply(ume.samples, 2, function(samples) { quantile(samples, c(0.025, 0.5, 0.975)) })
  pairEffects <- data.frame(t1=comps[,1], t2=comps[,2], pe=qs[2,], ci.l=qs[1,], ci.u=qs[3,], stringsAsFactors=FALSE)
  rownames(pairEffects) <- NULL

  cons.samples <- as.matrix(relative.effect(result.cons, t1=comps[,1], t2=comps[,2], preserve.extra=FALSE)[['samples']])
  qs <- apply(cons.samples, 2, function(samples) { quantile(samples, c(0.025, 0.5, 0.975)) })
  consEffects <- data.frame(t1=comps[,1], t2=comps[,2], pe=qs[2,], ci.l=qs[1,], ci.u=qs[3,], stringsAsFactors=FALSE)
  rownames(consEffects) <- NULL

  data <- studyEffects
  data[['t1']] <- as.character(data[['t1']])
  data[['t2']] <- as.character(data[['t2']])
  data[['p']] <- sapply(1:nrow(data), function(i) {
    row <- data[i, , drop=TRUE]
    pairEffects[['pe']][pairEffects[['t1']] == row[['t1']] & pairEffects[['t2']] == row[['t2']]]
  })
  data[['c']] <- sapply(1:nrow(data), function(i) {
    row <- studyEffects[i, , drop=TRUE]
    consEffects[['pe']][consEffects[['t1']] == row[['t1']] & consEffects[['t2']] == row[['t2']]]
  })
  data[['se']] <- (data[['ci.u']] - data[['ci.l']]) / 3.92

  pairEffects[['t1']] <- as.character(pairEffects[['t1']])
  pairEffects[['t2']] <- as.character(pairEffects[['t2']])

  indEffects <- data.frame(t1=pairEffects[['t1']], t2=pairEffects[['t2']], stringsAsFactors=FALSE)
  indEffects <- cbind(indEffects, t(apply(pairEffects, 1, function(row) {
    has.indirect <- has.indirect.evidence(network, row['t1'], row['t2'])
    if (has.indirect) {
      dir <- as.numeric(row[3:5])
      names(dir) <- c('pe', 'ci.l', 'ci.u') # sigh
      con <- consEffects[consEffects[['t1']] == row['t1'] & consEffects[['t2']] == row['t2'], 3:5, drop=]

      se.con <- (con['ci.u'] - con['ci.l']) / 3.92
      se.dir <- (dir['ci.u'] - dir['ci.l']) / 3.92
      pe.con <- con['pe']
      pe.dir <- dir['pe']

      # Back-calculate indirect estimate (see Dias et al. 2010)
      if (se.con < se.dir) {
        se.ind <- sqrt(1 / (1 / se.con^2 - 1 / se.dir^2))
        pe.ind <- (pe.con / se.con^2 - pe.dir / se.dir^2) * se.ind^2
        unlist(list('pe'=unname(pe.ind), 'se'=unname(se.ind)))
      } else {
        unlist(list('pe'=NA, 'se'=NA))
      }
    } else {
      unlist(list('pe'=NA, 'se'=NA))
    }
  })))

  i2.pair <- apply(pairEffects, 1, function(row) {
    data2 <- data[data[['t1']] == row['t1'] & data[['t2']] == row['t2'], , drop=FALSE]
    if (nrow(data2) > 1) {
      i.squared(data2[['pe']], data2[['se']], data2[['p']])
    } else {
      NA
    }
  })

  i2.cons <- apply(pairEffects, 1, function(row) {
    data2 <- data[data[['t1']] == row['t1'] & data[['t2']] == row['t2'], , drop=FALSE]
    ind <- indEffects[indEffects[['t1']] == row['t1'] & indEffects[['t2']] == row['t2'], 3:4, drop=]
    se.ind <- unname(ind['se'])
    pe.ind <- unname(ind['pe'])
    if (!is.na(se.ind)) {
      i.squared(unlist(c(data2[['pe']], pe.ind)), unlist(c(data2[['se']], se.ind)), c(data2[['c']], data2[['c']][1]))
    } else if (nrow(data2) > 1) {
      i.squared(data2[['pe']], data2[['se']], data2[['c']])
    } else {
      NA
    }
  })

  incons <- apply(pairEffects, 1, function(row) {
    dir <- as.numeric(row[3:5])
    names(dir) <- c('pe', 'ci.l', 'ci.u') # sigh
    se.dir <- (dir['ci.u'] - dir['ci.l']) / 3.92
    pe.dir <- dir['pe']

    ind <- indEffects[indEffects[['t1']] == row['t1'] & indEffects[['t2']] == row['t2'], 3:4, drop=]
    se.ind <- ind['se']
    pe.ind <- ind['pe']
    if (!is.na(se.ind)) {
      pe.inc <- (pe.dir - pe.ind)
      se.inc <- sqrt(se.dir^2 + se.ind^2)
      p.inc <- pnorm(as.numeric(pe.inc / se.inc))

      as.numeric(2 * min(p.inc, 1 - p.inc))
    } else {
      NA
    }
  })

  i.sq <- data.frame(t1=pairEffects[['t1']], t2=pairEffects[['t2']], i2.pair=i2.pair, i2.cons=i2.cons, incons.p=incons, stringsAsFactors=FALSE)
  total <- list(i2.pair=i.squared(data[['pe']], data[['se']], data[['p']], df.adj=-nrow(pairEffects)), i2.cons=i.squared(data[['pe']], data[['se']], data[['c']], df.adj=-nrow(network[['treatments']])+1))
  result <- list(studyEffects=studyEffects, pairEffects=pairEffects, consEffects=consEffects, indEffects=indEffects,
    isquared.comp=i.sq, isquared.glob=total, cons.model=result.cons[['model']])
  class(result) <- 'mtc.anohe.summary'
  result
}

print.mtc.anohe.summary <- function(x, ...) {
  cat("Analysis of heterogeneity\n")
  cat("=========================\n\n")
  cat("Per-comparison I-squared:\n")
  cat("-------------------------\n\n")
  print(x[['isquared.comp']])
  cat("\nGlobal I-squared:\n")
  cat("-------------------------\n\n")
  print(as.data.frame(x[['isquared.glob']]))
}

plot.mtc.anohe.summary <- function(x, ...) {
  stats <- x
  data <- list(id=character(), group=character(), i2=c(), pe=c(), ci.l=c(), ci.u=c(), style=factor(levels=c('normal','pooled')))
  group.labels <- character()

  studyEffects <- stats[['studyEffects']]
  pairEffects <- stats[['pairEffects']]
  consEffects <- stats[['consEffects']]
  indEffects <- stats[['indEffects']]
  isq <- stats[['isquared.comp']]

  appendEstimates <- function(data, rows) {
    if (!is.null(rows[['i2']]) && !is.na(rows[['i2']])) {
      data[['i2']] <- c(data[['i2']], paste(formatC(rows[['i2']], format="f", digits=1), "%", sep=""))
    } else {
      data[['i2']] <- c(data[['i2']], rep(NA, length(rows[['pe']])))
    }
    data[['pe']] <- c(data[['pe']], rows[['pe']])
    data[['ci.l']] <- c(data[['ci.l']], rows[['ci.l']])
    data[['ci.u']] <- c(data[['ci.u']], rows[['ci.u']])
    data
  }

  for (i in 1:nrow(pairEffects)) {
    t1 <- pairEffects[['t1']][i]
    t2 <- pairEffects[['t2']][i]

    param <- paste('d', t1, t2, sep='.')
    group.labels[param] <- paste(t2, 'vs', t1)

    # Study-level effects
    rows <- studyEffects[studyEffects[['t1']] == t1 & studyEffects[['t2']] == t2, , drop=FALSE]
    data[['id']] <- c(data[['id']], rows[['study']])
    data[['group']] <- c(data[['group']], rep(param, nrow(rows)))
    data[['style']] <- c(data[['style']], rep('normal', nrow(rows)))
    data <- appendEstimates(data, rows)

    isq.rows <- isq[isq[['t1']] == t1 & isq[['t2']] == t2, ,drop=FALSE]
    # Pair-wise pooled effect
    rows <- pairEffects[i, , drop=FALSE]
    rows[['i2']] <- isq.rows[, 'i2.pair']
    data[['id']] <- c(data[['id']], 'Pooled (pair-wise)')
    data[['group']] <- c(data[['group']], param)
    data[['style']] <- c(data[['style']], 'pooled')
    data <- appendEstimates(data, rows)

    # Indirect effect
    rows <- indEffects[indEffects[['t1']] == t1 & indEffects[['t2']] == t2, , drop=FALSE]
    data[['id']] <- c(data[['id']], 'Indirect (back-calculated)')
    data[['group']] <- c(data[['group']], param)
    data[['style']] <- c(data[['style']], 'indirect')
    rows[['ci.l']] <- rows[['pe']] - 1.96 * rows[['se']]
    rows[['ci.u']] <- rows[['pe']] + 1.96 * rows[['se']]
    data <- appendEstimates(data, rows)

    # Network pooled effect
    rows <- consEffects[consEffects[['t1']] == t1 & consEffects[['t2']] == t2, , drop=FALSE]
    rows[['i2']] <- isq.rows[, 'i2.cons']
    data[['id']] <- c(data[['id']], 'Pooled (network)')
    data[['group']] <- c(data[['group']], param)
    data[['style']] <- c(data[['style']], 'pooled')
    data <- appendEstimates(data, rows)
  }

  styles <- blobbogram.styles.default()
  my.styles <- data.frame(style='indirect', font.weight='plain', row.height=1, pe.style='circle', pe.scale=FALSE, lty=3)
  rownames(my.styles) <- my.styles[['style']]
  styles <- rbind(styles, my.styles)


  data <- as.data.frame(data)
  om.scale <- x[['cons.model']][['om.scale']]
  log.scale <- ll.call("scale.log", x[['cons.model']])
  default.xlim=c(nice.value(-om.scale, floor, log.scale), nice.value(om.scale, ceiling, log.scale))
  do.plot <- function(xlim=default.xlim, ...) {
    blobbogram(data, group.labels=group.labels,
      columns=c('i2'), column.labels=c('I^2'),
      ci.label=paste(ll.call("scale.name", x[['cons.model']]), "(95% CrI)"),
      log.scale=log.scale,
      styles=styles, xlim=xlim, ...)
  }
  do.plot(...)
}

test.regress <- function(network, likelihood, link, t1=NULL, t2=NULL) {
  n.trt <- nrow(network$treatments)
  devianceNames <- c("Dbar", "pD", "DIC", "data points", "dev.ab", "dev.re", "fit.ab", "fit.re", "lev.ab", "lev.re", "nd.ab", "nd.re", "fitted")

  # consistency model, random effects
  model <- mtc.model(network, likelihood=likelihood, link=link)
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))
  plot(result)
  forest(result)
  s <- summary(result)
  expect_true("sd.d" %in% rownames(s$summaries$quantiles))
  summary(relative.effect(result, t1=network$treatments$id[2]))
  expect_equal(dim(rank.probability(result)), c(n.trt, n.trt))
  expect_equal(names(result$deviance), devianceNames)
  expect_true(!all(is.na(unlist(result$deviance[c("Dbar", "pD", "DIC")]))))

  # consistency model, fixed effect
  model <- mtc.model(network, likelihood=likelihood, link=link, linearModel="fixed")
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))
  plot(result)
  forest(result)
  s <- summary(result)
  expect_false("sd.d" %in% rownames(s$summaries$quantiles))
  summary(relative.effect(result, t1=network$treatments$id[2]))
  expect_equal(dim(rank.probability(result)), c(n.trt, n.trt))
  expect_equal(names(result$deviance), devianceNames)
  expect_true(!all(is.na(unlist(result$deviance[c("Dbar", "pD", "DIC")]))))

  if (!is.null(t1) && !is.null(t2)) {
    # node-splitting, random effects
    model <- mtc.model(network, likelihood=likelihood, link=link, type="nodesplit", t1=t1, t2=t2)
    capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))
    plot(result)
    expect_error(forest(result))
    s <- summary(result)
    expect_true("sd.d" %in% rownames(s$summaries$quantiles))
    expect_true("d.direct" %in% rownames(s$summaries$quantiles))
    expect_true("d.indirect" %in% rownames(s$summaries$quantiles))
    expect_error(relative.effect(result, t1="C"))
    expect_error(rank.probability(result))
    expect_equal(names(result$deviance), devianceNames)
    expect_true(!all(is.na(unlist(result$deviance[c("Dbar", "pD", "DIC")]))))

    # node-splitting, fixed effect
    model <- mtc.model(network, likelihood=likelihood, link=link, type="nodesplit", t1=t2, t2=t1, linearModel="fixed")
    capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))
    plot(result)
    s <- summary(result)
    expect_false("sd.d" %in% rownames(s$summaries$quantiles))
    expect_true("d.direct" %in% rownames(s$summaries$quantiles))
    expect_true("d.indirect" %in% rownames(s$summaries$quantiles))
    expect_equal(names(result$deviance), devianceNames)
    expect_true(!all(is.na(unlist(result$deviance[c("Dbar", "pD", "DIC")]))))
  }

  # ume, random effects
  suppressWarnings(model <- mtc.model(network, likelihood=likelihood, link=link, type="ume"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))
  plot(result)
  s <- summary(result)
  expect_equal(names(result$deviance), devianceNames)
  expect_true(!all(is.na(unlist(result$deviance[c("Dbar", "pD", "DIC")]))))

  # ume, fixed effect
  suppressWarnings(model <- mtc.model(network, likelihood=likelihood, link=link, type="ume", linearModel="fixed"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))
  plot(result)
  s <- summary(result)
  expect_equal(names(result$deviance), devianceNames)
  expect_true(!all(is.na(unlist(result$deviance[c("Dbar", "pD", "DIC")]))))

  # anohe, random effects
  capture.output(anohe <- mtc.anohe(network, likelihood=likelihood, link=link, n.adapt=100, n.iter=500))
  capture.output(plot(anohe))
  anohe.s <- summary(anohe)
  plot(anohe.s)
  capture.output(print(anohe.s))
  expect_equal(names(result$deviance), devianceNames)
  expect_true(!all(is.na(unlist(result$deviance[c("Dbar", "pD", "DIC")]))))

  # anohe, fixed effect
  capture.output(anohe <- mtc.anohe(network, likelihood=likelihood, link=link, linearModel="fixed", n.adapt=100, n.iter=500))
  capture.output(plot(anohe))
  anohe.s <- summary(anohe)
  plot(anohe.s)
  capture.output(print(anohe.s))
  expect_equal(names(result$deviance), devianceNames)
  expect_true(!all(is.na(unlist(result$deviance[c("Dbar", "pD", "DIC")]))))
}

context("Regressions previously observed for ANOHE")

test_that("anohe-breaking.txt does not break summary.mtc.anohe", {
  network <- dget("../data/anohe-breaking.txt")
  capture.output(anohe <- mtc.anohe(network, n.adapt=200, n.iter=500))
  x <- summary(anohe)
  expect_true('studyEffects' %in% names(x))
})

test_that("Mixing up the order of treatments does not break summary.mtc.anohe", {
  network <- thrombolytic
  treatments <- network$treatments
  treatments$id <- factor(rev(as.character(treatments$id)), levels=rev(as.character(treatments$id)))
  network <- mtc.network(data=network$data.ab, treatments=treatments)
  capture.output(anohe <- mtc.anohe(network, n.adapt=200, n.iter=500))
  x <- summary(anohe)
  expect_true('studyEffects' %in% names(x))
})

#' Draw a single sample from a truncated normal distribution
#'
#' Generates random samples from a normal distribution truncated to the interval
#' `[a, b]`. This is a pure R implementation using inverse transform sampling
#' and does not depend on compiled code or external packages.
#'
#' For each observation, a uniform random value is drawn between the normal CDF
#' values at the lower and upper truncation limits, and then transformed back
#' using the normal quantile function.' 
rtruncnorm <- function(mean, sd, a, b) {
  stopifnot(
    length(a) == 1,
    length(b) == 1,
    length(mean) == 1,
    length(sd) == 1,
    is.numeric(a),
    is.numeric(b),
    is.numeric(mean),
    is.numeric(sd),
    sd > 0,
    a <= b
  )

  if (a == b) {
    a
  } else {
    # calculate the CDF values at the truncation bounds
    p_lo <- pnorm(a, mean, sd)
    p_up <- pnorm(b, mean, sd)
    # draw uniformly from the probability interval
    p <- runif(n = 1, min = p_lo, max = p_up)
    # transform from probability to value
    mean + sd * qnorm(p)
  }
}

test_that("rtruncnorm can sample across the mean", {
  x <- replicate(1000, rtruncnorm(mean = 0, sd = 1, a = -1, b = 1))
  expect_true(all(x >= -1))
  expect_true(all(x <= 1))
  expect_equal(mean(x), 0, tolerance = 0.1)
})

test_that("rtruncnorm can sample from a non-standard normal", {
  x <- replicate(1000, rtruncnorm(mean = 2, sd = 2, a = -2, b = 6))
  expect_true(all(x >= -2))
  expect_true(all(x <= 6))
  expect_equal(mean(x), 2, tolerance = 0.2)
})

test_that("rtruncnorm can sample the left tail", {
  x <- replicate(1000, rtruncnorm(mean = 0, sd = 1, a = -5, b = -3))
  expect_true(all(x >= -5))
  expect_true(all(x <= -3))
})

test_that("rtruncnorm can sample the right tail", {
  x <- replicate(1000, rtruncnorm(mean = 0, sd = 1, a = 4, b = 8))
  expect_true(all(x >= 4))
  expect_true(all(x <= 8))
})

regressionParams <- function(regressor, nt, nc=0) {
  if (!is.null(regressor[['control']])) {
    control <- as.numeric(regressor[['control']])
    betas <- paste0('beta[', (1:nt)[-control], ']')
    regression.parameters <- list('shared'='B',
                                  'unrelated'=betas,
                                  'exchangeable'=c(betas, 'B')) #, 'reg.sd'
    regression.parameters[[regressor[['coefficient']]]]
  } else { # by class
    paste0('B[', 2:nc, ']')
  }
}

regressionClassMap <- function(classes) {
  trt.to.class <- rep(NA, sum(sapply(classes, length)))
  for (i in 1:length(classes)) {
    trt.to.class[as.numeric(classes[[i]])] <- i
  }
  trt.to.class
}

#' @return true if t is a control treatment for the regression model
isRegressionControl <- function(model, t) {
  regressor <- model[['regressor']]

  t <- as.numeric(t)
  if (!is.null(regressor[['classes']])) {
    regressionClassMap(regressor[['classes']])[t] == 1
  } else {
    t == as.numeric(regressor[['control']])
  }
}

regressionAdjustMatrix <- function(t1, t2, regressor, nt) {
  stopifnot(is.factor(t1))
  nc <- length(regressor[['classes']])
  nparams <- length(regressionParams(regressor, nt, nc))
  pairs <- treatment.pairs(t1, t2, factor(1:nt, levels=1:nt, labels=levels(t1)))

  if (nc > 0) {
    pairs <- matrix(regressionClassMap(regressor[['classes']])[pairs], nrow=nrow(pairs))
    control <- 1
  } else {
    control <- as.numeric(regressor[['control']])
  }

  betaIndex <- function(i) {
    if (i > control) i - 1 else i
  }
  transform <- apply(pairs, 1, function(pair) {
    v <- rep(0, nparams)
    t1 <- pair[1]
    t2 <- pair[2]
    if (regressor[['coefficient']] == 'shared' && nc == 0) {
      if (t1 == control && t2 != control) {
        v[1] <- 1
      } else if (t1 != control && t2 == control) {
        v[1] <- -1
      }
    } else {
      if (t1 == control && t2 != control) {
        v[betaIndex(t2)] <- 1
      } else if (t1 != control && t2 == control) {
        v[betaIndex(t1)] <- -1
      } else if (t1 != control && t2 != control && t1 != t2) {
        v[betaIndex(t1)] <- -1
        v[betaIndex(t2)] <- 1
      }
    }
    v
  })
  matrix(transform, ncol=nrow(pairs))
}

context("regression")

test_that("regressionParams is correct", {
  expect_equal(regressionParams(list('coefficient'='shared', 'control'=1), 4), c('B'))
  expect_equal(regressionParams(list('coefficient'='shared', 'control'=3), 5), c('B'))

  expect_equal(regressionParams(list('coefficient'='unrelated', 'control'=1), 4), c('beta[2]', 'beta[3]', 'beta[4]'))
  expect_equal(regressionParams(list('coefficient'='unrelated', 'control'=3), 5), c('beta[1]', 'beta[2]', 'beta[4]', 'beta[5]'))
  expect_equal(regressionParams(list('coefficient'='unrelated', 'control'=4), 4), c('beta[1]', 'beta[2]', 'beta[3]'))

  expect_equal(regressionParams(list('coefficient'='exchangeable', 'control'=3), 4), c('beta[1]', 'beta[2]', 'beta[4]', 'B'))

  expect_equal(regressionParams(list('coefficient'='unrelated'), 8, nc=3), c('B[2]', 'B[3]'))
})

test_that("regressionAdjustMatrix is correct", {
  trt <- function(tx) factor(tx, levels=1:4)
  expect_equal(regressionAdjustMatrix(trt(c(1,1,2)), trt(c(1,2,3)), list('coefficient'='shared', 'control'=1), 4),
               cbind(0,1,0))
  expect_equal(regressionAdjustMatrix(trt(c(1,1,2)), trt(c(1,2,3)), list('coefficient'='shared', 'control'=2), 4),
               cbind(0,-1,1))

  expect_equal(regressionAdjustMatrix(trt(c(1,1,2)), trt(c(1,2,3)), list('coefficient'='unrelated', 'control'=1), 4),
               cbind(c(0,0,0), c(1,0,0), c(-1,1,0)))
  expect_equal(regressionAdjustMatrix(trt(c(1,1,2)), trt(c(1,2,3)), list('coefficient'='unrelated', 'control'=2), 4),
               cbind(c(0,0,0), c(-1,0,0), c(0,1,0)))
  expect_equal(regressionAdjustMatrix(trt(1), trt(3), list('coefficient'='unrelated', 'control'=3), 4),
               cbind(c(-1,0,0)))

  expect_equal(regressionAdjustMatrix(trt(c(1,1,2)), trt(c(1,2,3)), list('coefficient'='exchangeable', 'control'=1), 4),
               cbind(c(0,0,0,0), c(1,0,0,0), c(-1,1,0,0)))

  expect_equal(regressionAdjustMatrix(trt(c(1,1,2,2)), trt(c(1,2,3,4)), list('coefficient'='shared', 'classes'=list('C'=1, 'X'=c(2,3), 'Y'=4)), 4),
               cbind(c(0, 0), c(1,0), c(0, 0), c(-1, 1)))
})

test_that("Degenerate covariate is detected", {
  network <- mtc.network(
    data=data.frame(
      study=c('A', 'A', 'B', 'B'),
      treatment=c('X', 'Y', 'X', 'Y'),
      responders=3,
      sampleSize=10),
    studies=data.frame(
      study=c('A', 'B'),
      covariate=0
    )
  )
  expect_error(mtc.model(network, type='regression', regressor=list(variable='covariate', 'coefficient'='shared', 'control'='X')),
    "covariate is degenerate");
})

#' @include deviance.R

mtc.model.run <- function(network, type, ...) {
  runNames <- names(formals(mtc.run))
  runNames <- runNames[runNames != 'model']
  args <- list(...)

  # Call mtc.model with any arguments not intended for mtc.run
  modelArgs <- args[!(names(args) %in% runNames)]
  modelArgs$network <- network
  modelArgs$type <- type
  model <- do.call(mtc.model, modelArgs)

  # Call mtc.run with all arguments intended for mtc.run
  runArgs <- args[names(args) %in% runNames]
  runArgs$model <- model
  do.call(mtc.run, runArgs)
}

# If is.na(sampler), a sampler will be chosen based on availability, in this order:
# JAGS, BUGS. When the sampler is BUGS, BRugs or R2WinBUGS will be used.
mtc.run <- function(model, sampler=NA, n.adapt=5000, n.iter=20000, thin=1, n.burnin=0) {
  
  if (!is.na(sampler)) {
    if (sampler %in% c("JAGS", "rjags")) {
      warning("Setting the sampler is deprecated.")
    } else {
      stop("Setting the sampler is deprecated, only JAGS is supported.")
    }
  }

  result <- mtc.sample(model, n.adapt=n.adapt, n.iter=n.iter, thin=thin, n.burnin=n.burnin)

  result <- c(result, list(model = model))
  class(result) <- "mtc.result"
  result
}

mtc.build.syntaxModel <- function(model) {
  list(
    model = model[['code']],
    data = model[['data']],
    inits = model[['inits']],
    vars = if (!is.null(model[['monitors']][['enabled']])) model[['monitors']][['enabled']] else c(mtc.basic.parameters(model), "sd.d")
  )
}

mtc.sample <- function(model, n.adapt, n.iter, thin, n.burnin) {
  # generate JAGS model
  syntax <- mtc.build.syntaxModel(model)

  # compile & run JAGS model
  file.model <- tempfile()
  cat(paste(syntax[['model']], "\n", collapse=""), file=file.model)

  vars <- syntax[['vars']]
  if (model$dic) {
    dic.vars <- deviance_monitors(model)
    dic.vars <- dic.vars[!(dic.vars %in% vars)] # jags complains about redundant variables
    vars <- c(vars, dic.vars)
  }

  # Note: n.iter must be specified *excluding* the n.adapt
  jags <- rjags::jags.model(file.model, data=syntax[['data']],
    inits=syntax[['inits']],
    n.chains=model[['n.chain']],
    n.adapt=n.adapt)
  
  if (n.burnin > 0) update(jags, n.iter=n.burnin)
  
  samples <- rjags::coda.samples(jags, variable.names=vars,
                                 n.iter=n.iter, thin=thin)
  unlink(file.model)

  deviance.stats <- if (model[['dic']]) {
    apply(as.matrix(samples[, deviance_monitors(model), drop=FALSE]), 2, mean)
  }
  samples <- samples[, syntax[['vars']], drop=FALSE]

  # return
  list(samples=samples, deviance=computeDeviance(model, deviance.stats))
}

generate.summaries <- function(result) {
  samples <- result$samples
  data <- as.matrix(samples)
  data <- data[, grep("^d\\.", colnames(data))]
  if (result$model$type == "consistency") {
    list(
      effectiveSize = effectiveSize(samples),
      summary       = summary(samples),
      cov           = cov(as.matrix(data)),
      ranks         = rank.probability(result),
      dic           = result$deviance[c('Dbar', 'pD', 'DIC', 'data points')]
    )
  } else {
    list(
      effectiveSize = effectiveSize(samples),
      summary       = summary(samples),
      cov           = cov(as.matrix(data)),
      dic           = result$deviance[c('Dbar', 'pD', 'DIC', 'data points')]
    )
  }
}

formatError <- function(name, s1, s2, test) {
  str.s1 <- paste(capture.output(print(s1)), collapse="\n")
  str.s2 <- paste(capture.output(print(s2)), collapse="\n")
  str.test <- paste(capture.output(print(test)), collapse="\n")

  paste(name,
        "=========== Expected:       ===========", str.s1,
        "=========== Actual:         ===========", str.s2,
        "=========== Test statistic: ===========", str.test, sep="\n\n")
}

compare.summaries <- function(s1, s2) {
  stopifnot(names(s1$effectiveSize) == names(s2$effectiveSize))
  d.idx <- grep("^d\\.", names(s1$effectiveSize))
  sd.idx <- grep("^sd\\.", names(s1$effectiveSize))

  # Force statistics to be matrix
  if (is.vector(s1$summary$statistics)) s1$summary$statistics <- t(s1$summary$statistics)
  if (is.vector(s2$summary$statistics)) s2$summary$statistics <- t(s2$summary$statistics)

  # Test equality of means
  mu1 <- s1$summary$statistics[, 'Mean']
  mu2 <- s2$summary$statistics[, 'Mean']
  se1 <- s1$summary$statistics[, 'Time-series SE']
  se2 <- s2$summary$statistics[, 'Time-series SE']
  test <- pnorm(mu1 - mu2, 0, sqrt(se1^2 + se2^2))
  expect(all(test > 0.025), formatError("Means were not equal", mu1, mu2, test))

  # Test equality of variance
  sd1 <- s1$summary$statistics[d.idx, 'SD']
  sd2 <- s2$summary$statistics[d.idx, 'SD']
  en1 <- s1$effectiveSize[d.idx]
  en2 <- s2$effectiveSize[d.idx]
  test <- pf(sd1^2 / sd2^2, en1, en2)
  expect(all(test > 0.025), formatError("Variances were not equal", sd1^2, sd2^2, test))

  # TODO: multivariate test for equality of means
  # TODO: compare covariance matrices

  # TODO: compare quantiles for the standard deviation using http://www.jstor.org/stable/2673594

  # Test equality of rank probabilities
  if (!is.null(s1$ranks)) {
    thin <- s2$summary$thin
    n.adapt <- s2$summary$start - thin
    n.iter <- s2$summary$end - n.adapt
    test <- sapply(rownames(s2$ranks), function(alt) {
      n.sample <- min(s2$effectiveSize[d.idx])
      x <- round(s2$ranks[alt, ] * n.sample)
      p <- s1$ranks[alt, ]

      # Continuity correction because too many zero cells cause the test to fail
      x <- x + 1
      p <- (p + 1/n.sample) / (1 + nrow(s2$ranks)/n.sample)

      test <- chisq.test(x, p=p, rescale.p=TRUE, simulate.p.value=TRUE)
      c('statistic'=unname(test$statistic), 'p.value'=test$p.value)
    })
    expect(all(test['p.value',] > 0.025), formatError("Rank probabilities were not equal", s1$ranks, s2$ranks, test))
  }

  # Test equality of deviance statistics
  if (!is.null(s1$dic)) {
    n <- min(s1$effectiveSize, s2$effectiveSize)
    expect_equal(s1$dic[['data points']], s2$dic[['data points']])
    # deviance should follow an approximate Chi-squared distribution with variance 2*df
    se <- sqrt(2 * s1$dic[['data points']] / n)
    v1 <- unlist(s1$dic[c('Dbar', 'pD')])
    v2 <- unlist(s2$dic[c('Dbar', 'pD')])
    test <- pnorm(v1 - v2, 0, se)
    expect(all(test > 0.025), formatError("Model fit statistics were not equal", v1, v2, test))
  }
}

replicate.example <- function(name, network, type="consistency", linearModel="random", likelihood="binom", link="logit") {
  s1 <- dget(paste0("../data/", name, '.summaries.txt'))

  n.chain <- s1$summary$nchain
  thin <- s1$summary$thin
  n.adapt <- s1$summary$start - thin
  n.iter <- s1$summary$end - n.adapt

  model <- if (exists("powerAdjustMode")) {
    network$studies <- data.frame(study=mtc.studies.list(network)[['values']], x=1)
    mtc.model(network, type=type,
      likelihood=likelihood, link=link,
      linearModel=linearModel,
      n.chain=4,
      powerAdjust='x')
  } else {
    mtc.model(network, type=type,
      likelihood=likelihood, link=link,
      linearModel=linearModel,
      n.chain=4)
  }
  capture.output(
    result <- mtc.run(model, n.adapt=n.adapt, n.iter=n.iter, thin=thin)
  )
  s2 <- generate.summaries(result)
  list(s1=s1, s2=s2)
}

verify.example <- function(example, sampler) {
  x <- replicate.example(example, sampler)
  compare.summaries(x$s1, x$s2)
}

# Change-of-Baseline matrix
# old: vector of old arms, first element the baseline
# new: vector of new arms, first element the baseline
cob.matrix <- function(old, new) {
  stopifnot(length(old) >= length(new))
  n <- length(old) - 1
  m <- length(new) - 1
  b <- matrix(0, nrow=m, ncol=n)
  for (i in 1:m) {
    for (j in 1:n) {
      if (old[j + 1] == new[i + 1]) b[i, j] <- 1
      if (old[j + 1] == new[1]) b[i, j] <- -1
    }
  }
  b
}

nodesplit.rewrite.data.ab <- function(data, t1, t2) {
  if (is.null(data)) return(NULL);
  t1 <- as.character(t1)
  t2 <- as.character(t2)
  studies <- unique(data[['study']])
  per.study <- lapply(studies, function(study) {
    study.data <- data[data$study == study, ]
    study.data$study <- as.character(study.data$study)
    has.both <- all(c(t1, t2) %in% study.data$treatment)
    if (nrow(study.data) == 3 && has.both) {
      study.data <- study.data[study.data$treatment %in% c(t1, t2), ]
    } else if (nrow(study.data) > 3 && has.both) {
      sel <- study.data$treatment %in% c(t1, t2)
      study.data$study[sel] <- paste(study, '*', sep='')
      study.data$study[!sel] <- paste(study, '**', sep='')
    }
    study.data
  })
  data <- do.call(rbind, per.study)
  data$study <- as.factor(data$study)
  data
}

nodesplit.rewrite.studies <- function(network, t1, t2) {
  studies <- network$studies
  data <- mtc.merge.data(network)
  if (is.null(studies)) return(NULL);
  t1 <- as.character(t1)
  t2 <- as.character(t2)
  per.study <- lapply(studies[['study']], function(study) {
    study.data <- data[data$study == study, ]
    has.both <- all(c(t1, t2) %in% study.data[['treatment']])
    orig <- studies[studies[['study']] == study, ,drop=FALSE]
    if (nrow(study.data) > 3 && has.both) {
      rval <- rbind(orig, orig)
      rval[['study']] <- c(paste0(study, "*"), paste0(study, "**"))
      rval
    } else {
      orig
    }
  })
  studies <- as.data.frame(do.call(rbind, per.study))
  rownames(studies) <- NULL
  studies
}

nodesplit.rewrite.data.re <- function(data, t1, t2) {
  if (is.null(data)) return(NULL);
  t1 <- as.character(t1)
  t2 <- as.character(t2)
  cob <- function(study, data, ts) {
    old <- as.character(data$treatment)
    new <- c(ts, old[!(old %in% ts)])
    n <- length(old) - 1
    m <- length(ts)
    b <- cob.matrix(old, new)
    diff <- c(NA, b %*% data$diff[-1])[1:m]

    cov.m <- matrix(data$std.err[1], nrow=n, ncol=n)
    diag(cov.m) <- data$std.err[-1]
    cov.m <- b %*% cov.m %*% t(b)
    std.err <- c(cov.m[1, 2], diag(cov.m))[1:m]

    treatment <- factor(ts, levels=levels(data$treatment))
    data.frame(study=rep(study, m),
               treatment=treatment,
               diff=diff,
               std.err=std.err)
  }
  studies <- unique(data$study)
  per.study <- lapply(studies, function(study) {
    study.data <- data[data$study == study, ]
    study.data$study <- as.character(study.data$study)
    has.both <- all(c(t1, t2) %in% study.data$treatment)
    if (nrow(study.data) == 3 && has.both) {
      study.data <- cob(study, study.data, c(t1, t2))
    } else if (nrow(study.data) > 3 && has.both) {
      ts <- as.character(study.data$treatment)
      ts <- ts[!(ts %in% c(t1, t2))]
      study.data <- rbind(cob(paste(study, '*', sep=''), study.data, c(t1, t2)),
                          cob(paste(study, '**', sep=''), study.data, ts))
    }
    study.data[, c('study','treatment','diff','std.err')]
  })
  data <- do.call(rbind, per.study)
  data$study <- as.factor(data$study)
  data
}

mtc.nodesplit.comparisons <- function(network) {
  comparisons <- as.data.frame(mtc.comparisons(network))
  comparisons <- apply(comparisons, 1, function(comparison) {
    if (has.indirect.evidence(network, comparison['t1'], comparison['t2'])) comparison else NULL
  })
  if (is.list(comparisons)) {
    as.data.frame(do.call(rbind, comparisons), stringsAsFactors=FALSE)
  } else if (is.null(comparisons)) {
    data.frame(t1=character(), t2=character(), stringsAsFactors=FALSE)
  } else {
    as.data.frame(t(comparisons), stringsAsFactors=FALSE)
  }
}

mtc.nodesplit <- function(network, comparisons=mtc.nodesplit.comparisons(network), ...) {
  if (nrow(comparisons) == 0) {
    stop("There are no comparisons to assess for inconsistency. See https://doi.org/10.1002/jrsm.1167 for how this is determined.")
  }
  results <- apply(comparisons, 1, function(comparison) {
        mtc.model.run(network, type='nodesplit', t1=comparison['t1'], t2=comparison['t2'], ...)
  })
  params <- apply(comparisons, 1, function(comparison) {
        paste('d', comparison['t1'], comparison['t2'], sep='.')
  })
  names(results) <- params
  results$consistency <- mtc.model.run(network, type='consistency', ...)
  class(results) <- "mtc.nodesplit"
  results
}

print.mtc.nodesplit <- function(x, ...) {
  cat("Node-splitting analysis of inconsistency (mtc.nodesplit) object\n")
  for (name in names(x)) {
    cat(paste("$", name, ": ", class(x[[name]]), "\n", sep=""))
  }
}

plot.mtc.nodesplit <- function(x, ask=dev.interactive(orNone=TRUE), ...) {
  cat("Node-splitting -- convergence plots\n")
  for (name in names(x)) {
    cat(if (name == "consistency") "Consistency model:\n" else paste("Split-node ", name, ":\n", sep=""))
    if(name != names(x)[1]) par(ask=ask)
    plot(x[[name]], ask=ask, ...)
  }
}

summary.mtc.nodesplit <- function(object, ...) {
  params <- names(object)
  params <- params[params != 'consistency']
  p.value <- do.call(rbind, lapply(params, function(param) {
    samples <- object[[param]][['samples']]
    split <- object[[param]][['model']][['split']]
    samples.dir <- as.matrix(samples[ , 'd.direct', drop=FALSE])
    samples.ind <- as.matrix(samples[ , 'd.indirect', drop=FALSE])
    p <- sum(samples.dir > samples.ind) / length(samples.dir)
    data.frame(t1=split[1], t2=split[2], p=2 * min(p, 1 - p))
  }))
  dir.effect <- do.call(rbind, lapply(params, function(param) {
    samples <- object[[param]][['samples']]
    split <- object[[param]][['model']][['split']]
    samples.dir <- as.matrix(samples[ , 'd.direct', drop=FALSE])
    qs <- quantile(as.numeric(samples.dir), c(0.025, 0.5, 0.975))
    data.frame(t1=split[1], t2=split[2], pe=qs[2], ci.l=qs[1], ci.u=qs[3])
  }))
  ind.effect <- do.call(rbind, lapply(params, function(param) {
    samples <- object[[param]][['samples']]
    split <- object[[param]][['model']][['split']]
    samples.ind <- as.matrix(samples[ , 'd.indirect', drop=FALSE])
    qs <- quantile(as.numeric(samples.ind), c(0.025, 0.5, 0.975))
    data.frame(t1=split[1], t2=split[2], pe=qs[2], ci.l=qs[1], ci.u=qs[3])
  }))
  cons.effect <- do.call(rbind, lapply(params, function(param) {
    split <- object[[param]][['model']][['split']]
    samples <- relative.effect(object[['consistency']], t1=split[1], t2=split[2], preserve.extra=FALSE)$samples
    qs <- quantile(as.matrix(samples), c(0.025, 0.5, 0.975))
    data.frame(t1=split[1], t2=split[2], pe=qs[2], ci.l=qs[1], ci.u=qs[3])
  }))
  result <- list(dir.effect=dir.effect,
                 ind.effect=ind.effect,
                 cons.effect=cons.effect,
                 p.value=p.value,
                 cons.model=object[['consistency']][['model']])
  class(result) <- 'mtc.nodesplit.summary'
  result
}

print.mtc.nodesplit.summary <- function(x, ...) {
  cat("Node-splitting analysis of inconsistency\n")
  cat("========================================\n\n")
  data <- do.call(rbind, lapply(1:length(x[['p.value']][['t1']]), function(i) {
    data.frame(comparison=c(paste('d', x$p.value[i,'t1'], x$p.value[i,'t2'], sep='.'), '-> direct', '-> indirect', '-> network'),
               p.value=c(x$p.value[i, 'p'], NA, NA, NA),
               CrI=c(NA,
                     formatCI(x$dir.effect[i,c('pe', 'ci.l', 'ci.u')]),
                     formatCI(x$ind.effect[i,c('pe', 'ci.l', 'ci.u')]),
                     formatCI(x$cons.effect[i,c('pe', 'ci.l', 'ci.u')])))

  }))
  formatted <- as.matrix(format.data.frame(data))
  formatted[is.na(data)] <- NA
  print(formatted, na.print="", quote=FALSE, row.names=FALSE)
}

plot.mtc.nodesplit.summary <- function(x, ...) {
  data <- list(id=character(), group=character(), p=c(), pe=c(), ci.l=c(), ci.u=c(), style=factor(levels=c('normal','pooled')))
  t1 <- x[['p.value']][['t1']]
  t2 <- x[['p.value']][['t2']]
  params <- paste('d', t1, t2, sep='.')
  group.labels <- paste(t2, 'vs', t1)
  names(group.labels) <- params

  data <- do.call(rbind, lapply(1:length(params), function(i) {
    data.frame(id=c('direct', 'indirect', 'network'),
               group=rep(params[i], 3),
               style=rep('normal', 3),
               p=c(NA, x$p.value[i, 'p'], NA),
               pe=c(x$dir.effect[i,'pe'], x$ind.effect[i,'pe'], x$cons.effect[i,'pe']),
               ci.l=c(x$dir.effect[i,'ci.l'], x$ind.effect[i,'ci.l'], x$cons.effect[i,'ci.l']),
               ci.u=c(x$dir.effect[i,'ci.u'], x$ind.effect[i,'ci.u'], x$cons.effect[i,'ci.u']))
  }))

  blobbogram(data, group.labels=group.labels, columns=c('p'), column.labels='P-value',
    ci.label=paste(ll.call("scale.name", x[['cons.model']]), "(95% CrI)"),
    log.scale=ll.call("scale.log", x[['cons.model']]), ...)
}

context("node splitting")

test_that("change of baseline: simple permutations", {
  expect_that(cob.matrix(old=c('X', 'A'), new=c('X', 'A')), equals(diag(1)))
  expect_that(cob.matrix(old=c('X', 'A', 'B'), new=c('X', 'A', 'B')), equals(diag(2)))
  expect_that(cob.matrix(old=c('X', 'A', 'B', 'C'), new=c('X', 'A', 'B', 'C')), equals(diag(3)))
  expect_that(cob.matrix(old=c('X', 'B', 'A'), new=c('X', 'A', 'B')),
              equals(rbind(c(0, 1), c(1, 0))))
  expect_that(cob.matrix(old=c('X', 'B', 'A', 'C'), new=c('X', 'A', 'B', 'C')),
              equals(rbind(c(0, 1, 0), c(1, 0, 0), c(0, 0, 1))))
})

test_that("change of baseline: change of baseline", {
  expect_that(cob.matrix(old=c('A', 'B'), new=c('B', 'A')), equals(diag(-1, nrow=1)))
  expect_that(cob.matrix(old=c('A', 'B', 'C', 'D'), new=c('B', 'A', 'C', 'D')),
              equals(rbind(c(-1, 0, 0), c(-1, 1, 0), c(-1, 0, 1))))
})

test_that("change of baseline: shorter new", {
  expect_that(cob.matrix(old=c('A', 'B', 'C', 'D'), new=c('B', 'A', 'C')),
              equals(rbind(c(-1, 0, 0), c(-1, 1, 0))))
})

test_that("rewrite AB: 2-arm trials untouched", {
  data.ab <- read.table(textConnection("
study treatment
s01   A
s01   B
s02   A
s02   C
s03   B
s03   C"), header=TRUE, stringsAsFactors=TRUE)
  expect_that(nodesplit.rewrite.data.ab(data.ab, "A", "B"), equals(data.ab))
})

test_that("rewrite AB: 3-arm trials arm removed", {
  data.ab <- read.table(textConnection("
study treatment
s01   A
s01   B
s02   A
s02   C
s03   A
s03   B
s03   C"), header=TRUE, stringsAsFactors=TRUE)
  expect_that(nodesplit.rewrite.data.ab(data.ab, "A", "B"), equals(data.ab[1:6,]))
})

test_that("rewrite AB: 4+-arm trials split", {
  data.ab <- read.table(textConnection("
study treatment
s01   A
s01   B
s01   C
s01   D"), header=TRUE, stringsAsFactors=TRUE)
  data.ab.rewrite <- read.table(textConnection("
study treatment
s01*  A
s01*  B
s01** C
s01** D"), header=TRUE, stringsAsFactors=TRUE)
  expect_that(nodesplit.rewrite.data.ab(data.ab, "A", "B"), equals(data.ab.rewrite))
})

test_that("rewrite RE: 2-arm trials untouched", {
  data.re <- read.table(textConnection("
study treatment diff std.err
s01   A         NA   NA
s01   B         1.0  0.4
s02   A         -1.5 0.3
s02   C         NA   0.2
s03   B         NA   NA
s03   C         0.8  0.4"), header=TRUE, stringsAsFactors=TRUE)
  expect_that(nodesplit.rewrite.data.re(data.re, "A", "B"), equals(data.re))
})

test_that("rewrite RE: 3-arm trials arm removed, baseline changed", {
  data.re <- read.table(textConnection("
study treatment diff std.err
s01   A         NA   0.2
s01   B         1.0  0.4
s01   C         -1.5 0.3
s02   B         NA   0.1
s02   A         -1.2 0.3
s02   C         0.8  0.4
s03   C         NA   0.1
s03   A         -1.2 0.3
s03   B         0.8  0.4"), header=TRUE, stringsAsFactors=TRUE)
  data.re.rewrite <- read.table(textConnection("
study treatment diff std.err
s01   A         NA   0.2
s01   B         1.0  0.4
s02   A         NA   0.2
s02   B         1.2  0.3
s03   A         NA   0.2
s03   B         2.0  0.5"), header=TRUE, stringsAsFactors=TRUE)
  levels(data.re.rewrite$treatment) <- c("A", "B", "C")
  expect_that(nodesplit.rewrite.data.re(data.re, "A", "B"), equals(data.re.rewrite))
})

test_that("rewrite RE: 4+-arm trials split, baseline changed", {
  data.re <- read.table(textConnection("
study treatment diff std.err
s01   B         NA   0.3
s01   A         0.7  0.6
s01   C         0.9  0.5
s01   D         0.5  0.6"), header=TRUE, stringsAsFactors=TRUE)
  data.re.rewrite <- read.table(textConnection("
study treatment diff std.err
s01*  A         NA   0.3
s01*  B         -0.7 0.6
s01** C         NA   0.2
s01** D         -0.4 0.5"), header=TRUE, stringsAsFactors=TRUE)
  expect_that(nodesplit.rewrite.data.re(data.re, "A", "B"), equals(data.re.rewrite))
})

test_that("rewrite studies", {
  data.re <- read.table(textConnection("
study treatment diff std.err
s01   B         NA   0.3
s01   A         0.7  0.6
s01   C         0.9  0.5
s01   D         0.5  0.6"), header=TRUE, stringsAsFactors=TRUE)
  data.ab <- read.table(textConnection("
study treatment mean std.err
s02   B         NA   0.3
s02   A         0.7  0.6
s03   A         0.9  0.5
s03   B         0.9  0.5
s03   D         0.5  0.6"), header=TRUE, stringsAsFactors=TRUE)
  studies <- read.table(textConnection("
study x
s01   1
s02   2
s03   3"), header=TRUE, stringsAsFactors=TRUE)
  studies.rewrite <- read.table(textConnection("
study x
s01*  1
s01** 1
s02   2
s03   3"), header=TRUE, stringsAsFactors=FALSE)
  network <- list(data.ab=data.ab, data.re=data.re, studies=studies, treatments=data.frame(id=as.factor(c("A","B","C","D"))))
  expect_that(nodesplit.rewrite.studies(network, "A", "B"), equals(studies.rewrite))
})

test_that("rewrite studies with trivial studies data frame", {
  data.re <- read.table(textConnection("
study treatment diff std.err
s01   B         NA   0.3
s01   A         0.7  0.6
s01   C         0.9  0.5
s01   D         0.5  0.6"), header=TRUE, stringsAsFactors=TRUE)
  data.ab <- read.table(textConnection("
study treatment mean std.err
s02   B         NA   0.3
s02   A         0.7  0.6
s03   A         0.9  0.5
s03   B         0.9  0.5
s03   D         0.5  0.6"), header=TRUE, stringsAsFactors=TRUE)
  studies <- read.table(textConnection("
study
s01
s02
s03"), header=TRUE, stringsAsFactors=TRUE)
  studies.rewrite <- read.table(textConnection("
study
s01*
s01**
s02
s03"), header=TRUE, stringsAsFactors=FALSE)
  network <- list(data.ab=data.ab, data.re=data.re, studies=studies, treatments=data.frame(id=as.factor(c("A","B","C","D"))))
  expect_that(nodesplit.rewrite.studies(network, "A", "B"), equals(studies.rewrite))
})

## Regression test for issue #10
test_that("non-lexicographical treatment order works correctly", {
  data.ab <- read.table(textConnection("
study treatment mean std.err
1     1         1    0.5
1     10        1    0.5
2     1         1    0.5
2     11        1    0.5
3     2         1    0.5
3     10        1    0.5
4     10        1    0.5
4     11        1    0.5
"), header=TRUE, stringsAsFactors=FALSE)
  treatments <- read.table(textConnection("
id description
1  A
2  B
10 C
11 D
"), header=TRUE, stringsAsFactors=FALSE)
  network <- mtc.network(data.ab=data.ab, treatments=treatments)
  mtc.model(network, type="nodesplit", t1=10, t2=11)
  succeed()
})

## Regression test for issue #22
test_that("study names do not mess up nodesplit with RE data", {
  data <- read.csv("../data/ns-complex.csv")
  network <- mtc.network(data.re=data)
  expect_that(mtc.nodesplit.comparisons(network), equals(data.frame(t1=c("B", "D"), t2=c("D", "H"), stringsAsFactors=FALSE)))
  data$study <- data$Study
  data$Study <- NULL
  network <- mtc.network(data.re=data)
  expect_that(mtc.nodesplit.comparisons(network), equals(data.frame(t1=c("B", "D"), t2=c("D", "H"), stringsAsFactors=FALSE)))
})

## Regression test for issue #25
test_that("mixing AB and RE data will not duplicate comparisons", {
  data.ab <- data.frame(study=c('1', '1', '2', '2', '4', '4'), treatment=c('A', 'B', 'A', 'C', 'B', 'C'))
  data.re <- data.frame(study=c('3', '3', '3'), treatment=c('C', 'A', 'B'), diff=c(NA, 1, 1), std.err=c(0.5, 1, 1))
  network <- mtc.network(data.ab=data.ab, data.re=data.re)
  expect_that(mtc.nodesplit.comparisons(network), equals(data.frame(t1=c("A", "A", "B"), t2=c("B", "C", "C"), stringsAsFactors=FALSE)))
})

#' @include stopIfNotConsistent.R

rank.probability <- function(result, preferredDirection=1, covariate=NA) {
  stopIfNotConsistent(result, 'rank.probability')

  stopifnot(preferredDirection %in% c(1, -1))

  treatments <- sort(unique(as.vector(extract.comparisons(colnames(result[['samples']][[1]])))))

  n.alt <- length(treatments)

  # count ranks given a matrix d of relative effects (treatments as rows)
  rank.count <- function(d) {
    .Call(gemtc_rank_count, d)
  }

  d <- relative.effect(result, treatments[1], treatments, covariate=covariate, preserve.extra=FALSE)[['samples']]
  ranks <- rank.count(t(as.matrix(d)))
  colnames(ranks) <- treatments

  data <- result[['samples']]
  n.iter <- nchain(data) * (end(data) - start(data) + thin(data)) / thin(data)

  result <- t(ranks / n.iter)
  if (identical(preferredDirection, -1)) {
    result <- result[,ncol(result):1]
  }
  class(result) <- "mtc.rank.probability"
  attr(result, "direction") <- preferredDirection
  result
}

sucra <- function(ranks) {
  apply(ranks, 1, function(p) {
    a <- length(p)
    sum(cumsum(p[-a]))/(a-1)
  })
}

rank.quantiles <- function(ranks, probs=c("2.5%"=0.025, "50%"=0.5, "97.5%"=0.975)) {
  sapply(probs, function(x) {
    apply(ranks, 1, function(p) {
      which(cumsum(p) >= x)[1]
    })
  })
}

print.mtc.rank.probability <- function(x, ...) {
  cat(paste("Rank probability; preferred direction = ", attr(x, "direction"), "\n", sep=""))
  attr(x, "direction") <- NULL
  print(unclass(x), ...)
}

plot.mtc.rank.probability <- function(x, ...) { 
  barplot(t(x), ...)
}

standardize.treatments <- function(treatments) {
  treatments[['id']] <- factor(treatments[['id']], levels=treatments[['id']])
  treatments[['description']] <- as.character(treatments[['description']])
  rownames(treatments) <- as.character(treatments[['id']])
  treatments[order(treatments[['id']]), , drop=FALSE]
}

treatment.id.to.description <- function(network, ids) {
  all.ts <- network[['treatments']][['description']]
  names(all.ts) <- as.character(network[['treatments']][['id']])
  all.ts[ids]
}

standardize.data <- function(data, treatment.levels, re.order=FALSE) {
  data[['study']] <- as.factor(data[['study']])
  data[['treatment']] <- factor(as.character(data[['treatment']]), levels=treatment.levels)
  if (re.order) {
    na <- sapply(data[['study']], function(study) { sum(data[['study']] == study) })
    bl <- !is.na(data[['diff']])
    data <- data[order(na, data[['study']], bl, data[['treatment']]), , drop=FALSE]
  } else {
    data <- data[order(data[['study']], data[['treatment']]), , drop=FALSE]
  }
  if (nrow(data) > 0) {
    rownames(data) <- seq(1:nrow(data))
  }
  data
}

remove.onearm <- function(data, warn=FALSE) {
  # Remove 1-arm studies
  sel <- as.logical(sapply(data[,'study'], function(study) {
    sum(data[,'study'] == study) > 1
  }))

  if (warn && !all(sel)) {
    warning(paste("Removing", sum(!sel), "one-arm studies:",
                  paste(data[!sel,'study'], collapse=", ")))
  }
  data[sel, , drop=FALSE]
}

check.duplicated.treatments <- function(network, warn=FALSE) {
  duplicates <- FALSE
  for (study in mtc.studies.list(network)[['values']]) {
    design <- rle(sort(as.vector(mtc.study.design(network, study))))
    dup.v <- design[['values']][design[['lengths']] > 1]
    dup.l <- design[['lengths']][design[['lengths']] > 1]
    if (any(design[['lengths']] > 1)) {
      duplicates <- TRUE
      if (warn) warning(paste(dup.v, "occurs", dup.l, "times in", study))
    }
  }
  duplicates
}

mtc.network <- function(
  data.ab=data, treatments=NULL, description="Network",
  data.re=NULL, studies=NULL, data=NULL
) {
  if (is.null(data.ab) && is.null(data.re)) {
    stop("Either `data.ab' or `data.re' (or both) must be specified")
  }
  # standardize the data
  if (!is.null(data.ab)) {
    data.ab <- remove.onearm(as.data.frame(data.ab), warn=TRUE)
    mtc.validate.data.ab(data.ab)
  }
  if (!is.null(data.re)) {
    data.re <- remove.onearm(as.data.frame(data.re), warn=TRUE)
    mtc.validate.data.re(data.re)
  }

  # standardize the treatments
  if (is.null(treatments)) {
    data.treatments <- vector(mode="character")
    if (!is.null(data.ab)) {
      data.treatments <- c(data.treatments, as.character(data.ab[['treatment']]))
    }
    if (!is.null(data.re)) {
      data.treatments <- c(data.treatments, as.character(data.re[['treatment']]))
    }
    treatments <- sort(unique(data.treatments))
  }
  if (is.list(treatments) && !is.data.frame(treatments)) {
    treatments <- as.data.frame(do.call(rbind, treatments))
  }
  if (is.character(treatments) || is.factor(treatments)) {
    treatments <- data.frame(id=treatments, description=treatments)
  }
  treatments <- standardize.treatments(treatments)

  # standardize the study data
  if (!is.null(studies)) {
    if (!is.data.frame(studies)) {
      stop("studies must be a data frame")
    }
  }

  network <- list(
    description=description,
    treatments=treatments)

  if (!is.null(data.ab) && nrow(data.ab) > 0) {
    network <- c(network, list(data.ab=standardize.data(data.ab, levels(treatments[['id']]))))
  }
  if (!is.null(data.re) && nrow(data.re) > 0) {
    network <- c(network, list(data.re=standardize.data(data.re, levels(treatments[['id']]), re.order=TRUE)))
  }

  if (!is.null(studies) && nrow(studies) > 0) {
    network <- c(network, list(studies=studies[order(studies[['study']]), ,drop=FALSE]))
  }

  mtc.network.validate(network)

  class(network) <- "mtc.network"
  network
}

fix.network <- function(network) {
  # move data into data.ab for legacy networks
  if (is.null(network[['data.ab']]) && !is.null(network[['data']])) {
    network[['data.ab']] <- network[['data']]
    network[['data']] <- NULL
  }
  network
}

mtc.validate.data.ab <- function(data) {
  # No longer validating arm-based data at this stage
  # This is now defined at the likelihood/link level
}

mtc.validate.data.re <- function(data) {
  # These checks are still essential, because they check more then just the presence / absence of
  # columns.

  columns <- colnames(data)
  reColumns <- c('diff', 'std.err')
  if (!all(reColumns %in% columns)) {
    stop(paste('data.re must contain columns: ', paste(reColumns, collapse=', ')))
  }

  baselineCount <- sapply(unique(data[['study']]), function(study) { sum(is.na(data[['diff']][data[['study']] == study])) })
  if (!all(baselineCount == 1)) {
    stop('Each study in data.re must have a unique baseline arm (diff=NA)')
  }
  if (!all(!is.na(data[['std.err']][!is.na(data[['diff']])]))) {
    stop('All non-baseline arms in data.re must have std.err specified')
  }

  studies <- unique(data[['study']])
  ma <- sapply(studies, function(study) { sum(data[['study']] == study) > 2 })
  ma <- studies[ma]
  ma.studies <- data[['study']] %in% ma
  nobaseline <- is.na(data[['std.err']][ma.studies])
  if (!all(!nobaseline)) {
    stop(paste('All multi-arm trials (> 2 arms) must have the std.err of the baseline specified.',
               'Constraint violated by:',
               paste(data[['study']][ma.studies][nobaseline], collapse=", ")))
  }

  se <- data[['std.err']][!is.na(data[['std.err']])]
  if (!all(se > 0.0)) {
    stop(paste('In data.re, std.err must be either positive or NA'))
  }

  if (!all(sapply(ma, function(study) {
    ref <- data[['study']] == study & is.na(data[['diff']])
    rel <- data[['study']] == study & !is.na(data[['diff']])
    all(data[['std.err']][ref] < data[['std.err']][rel])
  }))) {
    stop("In data.re, std.err of the reference arm must < std.err of the relative effects")
  }
    
}

mtc.network.validate <- function(network) {
  # Check that there is some data
  stopifnot(nrow(network[['treatments']]) > 0)
  stopifnot(nrow(network[['data.ab']]) > 0 || nrow(network[['data.re']]) > 0)

  # Check that the treatments are correctly cross-referenced and have valid names
  all.treatments <- forcats::fct_c(
    if (is.null(network[['data.ab']])) factor() else network[['data.ab']][['treatment']],
    if (is.null(network[['data.re']])) factor() else network[['data.re']][['treatment']])
  stopifnot(all(all.treatments %in% network[['treatments']][['id']]))
  # stopifnot(all(network[['treatments']][['id']] %in% all.treatments)) -- disabled for node-splitting
  invalidId <- grep('^[a-zA-Z0-9_]*$', network[['treatments']][['id']], invert=TRUE)
  if (length(invalidId) > 0) {
    stop(paste0("\n", paste0(' Treatment name "',
      network[['treatments']][['id']][invalidId], '" invalid.', collapse="\n"),
      '\nTreatment names may only contain letters, digits, and underscore (_).'))
  }

  # Check that studies are not duplicated between [['data.ab']] and [['data.re']]
  if (!is.null(network[['data.ab']]) && !is.null(network[['data.re']])) {
    dup.study <- intersect(unique(network[['data.ab']][['study']]), unique(network[['data.re']][['study']]))
    if (length(dup.study) > 0) {
      stop(paste('Studies', paste(dup.study, collapse=", "), 'occur in both data and data.re'))
    }
  }

  # Check that the data frame has a sensible combination of columns
  if (!is.null(network[['data.ab']])) {
    mtc.validate.data.ab(network[['data.ab']])
  }

  # Check data.re is well formed
  if (!is.null(network[['data.re']])) {
    mtc.validate.data.re(network[['data.re']])
  }

  # If studies are given, they must match the treatments in the data tables
  if (!is.null(network[['studies']])) {
    data.studies <- c(as.character(network[['data.ab']][['study']]), as.character(network[['data.re']][['study']]))
    if (!setequal(data.studies, as.character(network[['studies']][['study']]))) {
      stop(paste('The studies data frame must match the studies in data.ab and data.re'))
    }
  }

  check.duplicated.treatments(network)
}

as.treatment.factor <- function(x, network) {
  v <- network[['treatments']][['id']]
  if (is.numeric(x)) {
    factor(x, levels=1:nlevels(v), labels=levels(v))
  } else if (is.factor(x) || is.character(x)) {
    x <- as.character(x)
    factor(x, levels=levels(v))
  }
}

mtc.merge.data <- function(network) {
  data.frame(
    study=c(
      as.character(network[['data.ab']][['study']]),
      as.character(network[['data.re']][['study']])),
    treatment=as.treatment.factor(c(
      network[['data.ab']][['treatment']],
      network[['data.re']][['treatment']]), network),
    stringsAsFactors=FALSE)
}

mtc.studies.list <- function(network) {
  rle(as.character(mtc.merge.data(network)[['study']]))
}

mtc.study.design <- function(network, study) {
  data <- mtc.merge.data(network)
  data[['treatment']][data[['study']] == study]
}

coerce.factor <- function(x, prototype) {
  factor(x, levels=1:nlevels(prototype), labels=levels(prototype))
}

# See nodesplit-auto draft for definition
has.indirect.evidence <- function(network, t1, t2) {
  has.both <- sapply(mtc.studies.list(network)[['values']], function(study) {
    all(c(t1, t2) %in% mtc.study.design(network, study))
  })

  data <- mtc.merge.data(network)
  not.both <- !has.both[data[['study']]]
  data <- data[not.both, , drop=FALSE]

  if (nrow(data) > 0) {
    n <- mtc.network(data)
    g <- mtc.network.graph(n)
    all(c(t1, t2) %in% V(g)$name) && as.logical(is.finite(shortest.paths(as.undirected(g), t1, t2)))
  } else {
    FALSE
  }
}

mtc.treatment.pairs <- function(treatments) {
  n <- length(treatments)
  t1 <- do.call(forcats::fct_c, lapply(1:(n-1), function(i) { rep(treatments[i], n - i) }))
  t2 <- do.call(forcats::fct_c, lapply(1:(n-1), function(i) { treatments[(i+1):n] }))
  data.frame(t1=t1, t2=t2)
}

## Get all direct comparisons with the number of studies
## measuring the direct comparison. This is like mtc.comparisons
## but in addition the numbers are appended as an additional column.
mtc.nr.comparisons <- function(network) {
  m <- mtc.study.treatment.matrix(network)
  comp <- mtc.comparisons(network)
  cm <- as.matrix(comp)
  nr <- aaply(cm, 1, function(co) {
    sum(rowSums(m[,co, drop=FALSE]) == 2)
  })
  cbind(comp, nr)
}

# Get all comparisons with direct evidence from the data set.
# Returns a (sorted) data frame with two columns (t1 and t2).
mtc.comparisons <- function(network) {
  ## Generate all pair-wise comparisons from each "design"
  data <- mtc.merge.data(network)
  ## Identify the unique "designs" (treatment combinations)
  design <- function(study) { mtc.study.design(network, study) }
  designs <- unique(lapply(unique(data[['study']]), design))
  comparisons <- do.call(rbind, lapply(designs, mtc.treatment.pairs))
  ## Make sure we include each comparison in only one direction
  swp <- as.character(comparisons[['t1']]) > as.character(comparisons[['t2']])
  tmp <- comparisons[['t1']]
  comparisons[['t1']][swp] <- comparisons[['t2']][swp]
  comparisons[['t2']][swp] <- tmp[swp]

  ## Ensure the output comparisons are always in the same order
  comparisons <- comparisons[order(comparisons[['t1']], comparisons[['t2']]), ,drop=FALSE]
  comparisons[['t1']] <- as.treatment.factor(comparisons[['t1']], network)
  comparisons[['t2']] <- as.treatment.factor(comparisons[['t2']], network)
  ## Ensure the output comparisons are unique
  comparisons <- unique(comparisons)
  row.names(comparisons) <- NULL
  comparisons
}

edges.create <- function(e, ...) {
  ed <- t(matrix(c(e[['t1']], e[['t2']]), ncol=2))
  edges(as.vector(ed), weight=e[['nr']], ...)
}

graph.create <- function(v, e, ...) {
  g <- graph.empty()
  g <- g + vertex(levels(v))
  g <- g + edges.create(e, ...)
  g
}

mtc.network.graph <- function(network, include.nr.comparisons=FALSE) {
    comp <- if (include.nr.comparisons) mtc.nr.comparisons else mtc.comparisons
    comparisons <- comp(network)
    treatments <-   network[['treatments']][['id']]
    graph.create(treatments, comparisons, arrow.mode=0)
}

## mtc.network class methods
print.mtc.network <- function(x, ...) {
  x <- fix.network(x)
  cat("MTC dataset: ", x[['description']], "\n", sep="")
  if (!is.null(x[['data.ab']])) {
    cat('Arm-level data: \n')
    print(x[['data.ab']])
  }
  if (!is.null(x[['data.re']])) {
    cat('Relative effect data: \n')
    print(x[['data.re']])
  }
  if (!is.null(x[['studies']])) {
    cat('Study-level data: \n')
    print(x[['studies']])
  }
}

mtc.study.treatment.matrix <- function(network) {
  object <- fix.network(network)
  data <- mtc.merge.data(object)
  studies <- unique(data[['study']])
  m <- laply(object[['treatments']][['id']], function(treatment) {
    laply(studies, function(study) {
      any(data[['study']] == study & data[['treatment']] == treatment)
    }, .drop=TRUE)
  }, .drop=FALSE)
  m <- t(m)
  colnames(m) <- object[['treatments']][['id']]
  m
}

summary.mtc.network <- function(object, ...) {
  object <- fix.network(object)
  m <- mtc.study.treatment.matrix(object)
  x <- as.factor(apply(m, 1, sum))
  levels(x) <- sapply(levels(x), function(y) { paste(y, "arm", sep="-") })
  list("Description"=paste("MTC dataset: ", object[['description']], sep=""),
       "Studies per treatment"=apply(m, 2, sum),
       "Number of n-arm studies"=summary(x),
       "Studies per treatment comparison"=mtc.nr.comparisons(object)
       )
}

plot.mtc.network <- function(x, layout=igraph::layout.circle, dynamic.edge.width=TRUE, use.description=FALSE, ...) {
  x <- fix.network(x)
  g <- mtc.network.graph(x, TRUE)
  labels <- if (use.description)
                treatment.id.to.description(x, x[['treatments']][['id']])
            else
                as.character(x[['treatments']][['id']])

  if (dynamic.edge.width) {
    igraph::plot.igraph(g, layout=layout, edge.width=E(g)$weight, vertex.label=labels, ...)
  } else {
    igraph::plot.igraph(g, layout=layout, vertex.label=labels, ...)
  }
}

context("mtc.network(data.re)")

data.re <- read.table(textConnection("
study  treatment  diff  std.err
s01    A          NA    NA
s01    C          -0.31 0.67
s02    A          NA    NA
s02    B          -1.7  0.38
s03    C          NA    NA
s03    D          -0.35 0.44
s04    C          NA    NA
s04    D          0.55  0.55
s05    D          NA    NA
s05    E          -0.3  0.27
s06    D          NA    NA
s06    E          -0.3  0.32
s07    A          NA    0.50
s07    B          -2.3  0.72
s07    D          -0.9  0.69"), header=TRUE, stringsAsFactors=FALSE)

data.re.factors <- read.table(textConnection("
study  treatment  diff  std.err
s01    A          NA    NA
s01    C          -0.31 0.67
s02    A          NA    NA
s02    B          -1.7  0.38
s03    C          NA    NA
s03    D          -0.35 0.44
s04    C          NA    NA
s04    D          0.55  0.55
s05    D          NA    NA
s05    E          -0.3  0.27
s06    D          NA    NA
s06    E          -0.3  0.32
s07    A          NA    0.50
s07    B          -2.3  0.72
s07    D          -0.9  0.69"), header=TRUE, stringsAsFactors=TRUE)


test_that("either data.ab or data.re must be specified", {
  expect_error(mtc.network())
})

test_that("data.ab can be unspecified if data.re is given", {
  mtc.network(data.re=data.re)
  succeed()
})

test_that("data.re is properly stored", {
  network <- mtc.network(data.re=data.re)
  expect_that(network$data.re$study, equals(as.factor(data.re$study)))
  expect_that(network$data.re$treatment, equals(as.factor(data.re$treatment)))
  expect_that(network$data.re$diff, equals(data.re$diff))
  expect_that(network$data.re$std.err, equals(data.re$std.err))
})

test_that("data.re is properly stored if specified with factors", {
  network <- mtc.network(data.re=data.re.factors)
  expect_that(network$data.re$study, equals(data.re.factors$study))
  expect_that(network$data.re$treatment, equals(data.re.factors$treatment))
  expect_that(network$data.re$diff, equals(data.re$diff))
  expect_that(network$data.re$std.err, equals(data.re$std.err))
})

data.ab <- read.table(textConnection("
  study  treatment
  s08    A
  s08    C
  s09    A
  s09    D
  s09    F"), header=TRUE, stringsAsFactors=FALSE)

test_that("treatments for data.ab and data.re are merged", {
  network <- mtc.network(data.ab=data.ab, data.re=data.re)
  expect_that(levels(network$treatments$id), equals(c("A", "B", "C", "D", "E", "F")))
  expect_that(as.character(network$data.ab$treatment), equals(as.character(data.ab$treatment)))
  expect_that(as.character(network$data.re$treatment), equals(as.character(data.re$treatment)))
})

test_that("merged list of arms is correct", {
  network <- mtc.network(data.ab=data.ab, data.re=data.re)
  expect_that(as.character(mtc.merge.data(network)$study), equals(c(as.character(data.ab$study), as.character(data.re$study))))
  expect_that(as.character(mtc.merge.data(network)$treatment), equals(c(as.character(data.ab$treatment), as.character(data.re$treatment))))
})

test_that("duplicate studies raise an error", {
  data <- data.ab
  data$study <- c('s07', 's07', 's09', 's09', 's09')
  expect_error(mtc.network(data.ab=data, data.re=data.re))
})

test_that("data.re column names are checked", {
  expect_error(mtc.network(data.re=data.frame(study=c("s01", "s01"), treatment=c("A", "B"))))
  expect_error(mtc.network(data.re=data.frame(study=c("s01", "s01"), treatment=c("A", "B"), diff=c(NA, NA))))
  expect_error(mtc.network(data.re=data.frame(study=c("s01", "s01"), treatment=c("A", "B"), std.err=c(NA, NA))))
})

test_that("data.re checks that every study has a baseline (diff=NA) arm", {
  expect_error(mtc.network(data.re=data.frame(study=c("s01", "s01"), treatment=c('A', 'B'), diff=c(1, 2), std.err=c(0.5, 0.5))))
})

test_that("data.re checks that non-baseline arms have std.err specified", {
  mtc.network(data.re=data.frame(study=c("s01", "s01"), treatment=c('A', 'B'), diff=c(NA, 2), std.err=c(NA, 1)))
  expect_error(mtc.network(data.re=data.frame(study=c("s01", "s01"), treatment=c('A', 'B'), diff=c(NA, 2), std.err=c(NA, NA))))
})

test_that("data.re checks that multi-arm trials must have std.err specified for all arms", {
  expect_error(mtc.network(data.re=data.frame(study=c("s01", "s01", "s01"), treatment=c('A', 'B', 'C'), diff=c(NA, 2, 1), std.err=c(NA, 1, 1))))
})

test_that("data.re checks that std.err is positive", {
  expect_error(mtc.network(data.re=data.frame(study=c("s01", "s01"), treatment=c('A', 'B'), diff=c(NA, 1), std.err=c(NA, -1))), "std.err must be either positive or NA")
})

test_that("data.re checks the standard error in the reference arm < the standard error of the relative effects", {
  expect_error(mtc.network(data.re=data.frame(study=c("s01", "s01", "s01"), treatment=c('A', 'B', 'C'), diff=c(NA, 2, 1), std.err=c(2, 1, 1))), "std.err of the reference arm must < std.err of the relative effects")
})

test_that("data.re is ordered by number of arms", {
  data <- read.table(textConnection("
study  treatment  diff  std.err
s01    A          NA    0.43
s01    C          -0.31 0.67
s01    D          -0.31 0.67
s02    A          NA    NA
s02    B          -1.7  0.38"), header=TRUE)
  expected <- read.table(textConnection("
study  treatment  diff  std.err
s02    A          NA    NA
s02    B          -1.7  0.38
s01    A          NA    0.43
s01    C          -0.31 0.67
s01    D          -0.31 0.67"), header=TRUE, stringsAsFactors=TRUE)
  expect_that(mtc.network(data.re=data)$data.re, equals(expected))
})

test_that("data.re has the baseline arm of every study first", {
  data <- read.table(textConnection("
study  treatment  diff  std.err
s01    C          -0.31 0.67
s01    A          -0.31 0.67
s01    D          NA    0.43
s02    A          NA    NA
s02    B          -1.7  0.38"), header=TRUE)
  expected <- read.table(textConnection("
study  treatment  diff  std.err
s02    A          NA    NA
s02    B          -1.7  0.38
s01    D          NA    0.43
s01    A          -0.31 0.67
s01    C          -0.31 0.67"), header=TRUE, stringsAsFactors=TRUE)
  expect_that(mtc.network(data.re=data)$data.re, equals(expected))
})

context("Run models for all data sets and call the appropriate summaries / plots")

test_that("cipriani-efficacy", {
  # TODO: test node-splitting comparisons

  test.regress(depression, likelihood="binom", link="logit", t1="escitalopram", t2="sertraline")
})

test_that("luades-smoking", {
  # node-splitting comparisons
  expect_that(mtc.nodesplit.comparisons(smoking), equals(
    data.frame(t1=c("A", "A", "A", "B", "B", "C"), t2=c("B", "C", "D", "C", "D", "D"), stringsAsFactors=FALSE)))

  test.regress(smoking, likelihood="binom", link="logit", t1="B", t2="D")
})

test_that("luades-thrombolytic", {
  # node-splitting comparisons
  expect_that(mtc.nodesplit.comparisons(thrombolytic), equals(
    data.frame(t1=c("ASPAC", "ASPAC", "ASPAC", "AtPA", "AtPA", "AtPA", "AtPA", "Ret", "SK", "SK", "tPA"),
               t2=c("AtPA", "SK", "tPA", "Ret", "SK", "SKtPA", "UK", "SK", "tPA", "UK", "UK"), stringsAsFactors=FALSE)))

  test.regress(thrombolytic, likelihood="binom", link="logit", t1="SK", t2="UK")
})

test_that("tsd2-1 (event data, pair-wise)", {
  expect_that(nrow(mtc.nodesplit.comparisons(blocker)), equals(0))
  test.regress(blocker, likelihood="binom", link="logit")
})


test_that("tsd2-2 (fat survival, rate data)", {
  expect_that(nrow(mtc.nodesplit.comparisons(dietfat)), equals(0))
  test.regress(dietfat, likelihood="poisson", link="log")
})

test_that("tsd2-3 (diabetes, rate data)", {
  data.ab <- dget("../data/studyrow/tsd2-3.out.txt")
  network <- mtc.network(data.ab=data.ab)
  
  comparisons <- mtc.comparisons(network)
  comparisons$t1 <- as.character(comparisons$t1)
  comparisons$t2 <- as.character(comparisons$t2)
  expect_that(mtc.nodesplit.comparisons(network), equals(comparisons))

  test.regress(network, likelihood="binom", link="cloglog", t1="ARB", t2="BetaB")
})

test_that("parkinson example", {
  expect_that(mtc.nodesplit.comparisons(parkinson), equals(
    data.frame(t1=c("A", "A", "B", "C"),
               t2=c("C", "D", "D", "D"), stringsAsFactors=FALSE)))

  test.regress(parkinson, likelihood="normal", link="identity", t1="B", t2="D")
})

test_that("tsd2-5 (parkinson AB data)", {
  data.ab <- dget("../data/studyrow/tsd2-5.out.txt")
  network <- mtc.network(data.ab=data.ab)

  expect_that(mtc.nodesplit.comparisons(network), equals(
    data.frame(t1=c("1", "1", "2", "3"),
               t2=c("3", "4", "4", "4"), stringsAsFactors=FALSE)))

  test.regress(network, likelihood="normal", link="identity", t1="3", t2="4")
})

test_that("tsd2-7 (parkinson RE data)", {
  expect_that(mtc.nodesplit.comparisons(parkinson_diff), equals(
    data.frame(t1=c("A", "A", "B", "C"),
               t2=c("C", "D", "D", "D"), stringsAsFactors=FALSE)))

  test.regress(parkinson_diff, likelihood="normal", link="identity", t1="D", t2="A")
})

test_that("tsd2-8 (parkinson mixed data)", {
  expect_that(mtc.nodesplit.comparisons(parkinson_shared), equals(
    data.frame(t1=c("A", "A", "B", "C"),
               t2=c("C", "D", "D", "D"), stringsAsFactors=FALSE)))
  
  test.regress(parkinson_shared, likelihood="normal", link="identity", t1="B", t2="D")
})

test_that("ns-complex (numbered studies)", {
  data.re <- read.table("../data/ns-complex.csv", header=TRUE, sep=",")
  data.re$Study <- NULL
  network <- mtc.network(data.re=data.re)

  # Node-split comparisons are tested elsewhere

  test.regress(network, likelihood="normal", link="identity", t1="B", t2="D")
})

test_that("ns-complex (named studies)", {
  data.re <- read.table("../data/ns-complex.csv", header=TRUE, sep=",")
  data.re$study <- data.re$Study
  data.re$Study <- NULL
  network <- mtc.network(data.re=data.re)

  # Node-split comparisons are tested elsewhere

  test.regress(network, likelihood="normal", link="identity", t1="H", t2="D")
})

test_that("certolizumab regression (binomial data, continuous covariate)", {
  model <- mtc.model(certolizumab, type="regression", linearModel="fixed", 
                     regressor=list(control="Placebo", coefficient="unrelated", variable="diseaseDuration"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(certolizumab, type="regression", linearModel="fixed", 
                     regressor=list(control="Placebo", coefficient="shared", variable="diseaseDuration"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(certolizumab, type="regression", linearModel="fixed", 
                     regressor=list(control="Placebo", coefficient="exchangeable", variable="diseaseDuration"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(certolizumab, type="regression", linearModel="random", 
                     regressor=list(control="Placebo", coefficient="unrelated", variable="diseaseDuration"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(certolizumab, type="regression", linearModel="random", 
                     regressor=list(control="Placebo", coefficient="shared", variable="diseaseDuration"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(certolizumab, type="regression", linearModel="random", 
                     regressor=list(control="Placebo", coefficient="exchangeable", variable="diseaseDuration"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  expect_true(TRUE)
})

test_that("atrial fibrillation regression (classes, binomial data, continuous covariate)", {
  classes <- list("control"=c('01'),
                  "anti-coagulant"=c('02','03','04','09'),
                  "anti-platelet"=c('05','06','07','08','10','11','12','16','17'),
                  "mixed"=c('13','14','15'))

  model <- mtc.model(atrialFibrillation, type="regression", linearModel="fixed", 
                     regressor=list(classes=classes, coefficient="shared", variable="stroke"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(atrialFibrillation, type="regression", linearModel="random", 
                     regressor=list(classes=classes, coefficient="shared", variable="stroke"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  expect_true(TRUE)
})

test_that("heart failure prevention (binomial data, binary covariate)", {
  model <- mtc.model(hfPrevention, type="regression", linearModel="fixed", 
                     regressor=list(control="control", coefficient="unrelated", variable="secondary"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(hfPrevention, type="regression", linearModel="fixed", 
                     regressor=list(control="control", coefficient="shared", variable="secondary"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(hfPrevention, type="regression", linearModel="fixed", 
                     regressor=list(control="control", coefficient="exchangeable", variable="secondary"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(hfPrevention, type="regression", linearModel="random", 
                     regressor=list(control="control", coefficient="unrelated", variable="secondary"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(hfPrevention, type="regression", linearModel="random", 
                     regressor=list(control="control", coefficient="shared", variable="secondary"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  model <- mtc.model(hfPrevention, type="regression", linearModel="random", 
                     regressor=list(control="control", coefficient="exchangeable", variable="secondary"))
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  expect_true(TRUE)
})

test_that("node-splitting with down-weighting", {
  studies <- unique(smoking$data.ab$study)
  weight <- (0:(length(studies)-1))/length(studies)
  network <- mtc.network(smoking$data.ab, studies=data.frame(study=studies, weight=weight, stringsAsFactors=FALSE))
  model <- mtc.model(network, type="nodesplit", t1="A", t2="B", powerAdjust="weight")
  capture.output(result <- mtc.run(model, n.adapt=100, n.iter=500))

  expect_true(TRUE)
})

context("mtc.model.ume")

test_that("mtc.comparisons.baseline identical to mtc.comparisons for two-arm trials", {
  data <- read.table(textConnection('
    study  treatment  responders  sampleSize
        s01    A          3           10
        s01    B          5           9
        s02    B          10          40
        s02    C          10          38'), header=T)
  network <- mtc.network(data)
  expect_that(mtc.comparisons.baseline(network), equals(mtc.comparisons(network)))
})

test_that("mtc.comparisons.baseline only includes baseline comparisons for multi-arm trials", {
  data <- read.table(textConnection('
    study  treatment  responders  sampleSize
        s01    A          3           10
        s01    B          5           9
        s01    C          10          40'), header=T)
  network <- mtc.network(data)
  expected <- data.frame(
    t1=as.treatment.factor(c('A', 'A'), network),
    t2=as.treatment.factor(c('B', 'C'), network))
  expect_that(mtc.comparisons.baseline(network), equals(expected))
})

test_that("mtc.comparisons.baseline respects baseline in data.re", {
  data <- read.table(textConnection('
    study  treatment  diff  std.err
        s01    A          2.0   0.5
        s01    B          NA    0.3
        s01    C          1.5   0.6'), header=T)
  network <- mtc.network(data.re=data)
  expected <- data.frame(
    t1=as.treatment.factor(c('B', 'B'), network),
    t2=as.treatment.factor(c('A', 'C'), network))
  expect_that(mtc.comparisons.baseline(network), equals(expected))
})

test_that("mtc.model.ume warns about mutli-arm trials", {
  data <- read.table(textConnection('
    study  treatment  diff  std.err
        s01    A          2.0   0.5
        s01    B          NA    0.3
        s01    C          1.5   0.6'), header=T)
  network <- mtc.network(data.re=data)
    expect_warning(mtc.model(network, type='ume', likelihood='normal', link='identity'), "multi-arm trials")
})

test_that("Vertices agree between mtc.network.graph and ume model$graph", {
    network <- thrombolytic
    suppressWarnings(model <- mtc.model(network, type='ume'))
    graph <- mtc.network.graph(network)
    expect_that(V(model$graph)$name, equals(V(graph)$name))
    expect_that(V(mtc.model.graph(model))$name, equals(V(graph)$name))
})

test_that("Edges are consistent for ume model$graph", {
  data <- read.table(textConnection('
    study  treatment  diff  std.err
        s01    A          2.0   0.5
        s01    B          NA    0.3
        s01    C          1.5   0.6'), header=T)
  network <- mtc.network(data.re=data)
  suppressWarnings(model <- mtc.model(network, type='ume', likelihood='normal', link='identity'))

  expect_that(length(E(model$graph)), equals(2))
  expect_that(model$graph['A', 'B'], equals(1))
  expect_that(model$graph['B', 'C'], equals(1))
})

## Regression test for #26
test_that("RE data will not introduce duplicate basic parameters", {
  data.ab <- data.frame(
    study=c('1', '1', '2', '2', '3', '3'),
    treatment=c('A', 'B', 'A', 'C', 'B', 'C'),
    mean=rep(1,6), std.err=rep(0.5,6))
  data.re <- data.re <- data.frame(study=c('4', '4'), treatment=c('C', 'A'), diff=c(NA, 1), std.err=c(0.3, 0.5))
  network <- mtc.network(data.ab=data.ab, data.re=data.re)

  model <- mtc.model(network, type='ume', likelihood='normal', link='identity')

  expect_that(length(E(model$graph)), equals(3))
  expect_that(model$graph['A', 'B'], equals(1))
  expect_that(model$graph['A', 'C'], equals(1))
  expect_that(model$graph['B', 'C'], equals(1))

  # check that the relative effects matrix has the correct entries
  expect_that(grep("d\\[1, 2\\] <- d.A.B", model$code), equals(1))
  expect_that(grep("d\\[1, 3\\] <- d.A.C", model$code), equals(1))
  expect_that(grep("d\\[2, 3\\] <- d.B.C", model$code), equals(1))
  expect_that(grep("d\\[3, 1\\] <- -d.A.C", model$code), equals(1))
})

test_that("func.param.matrix was implemented correctly", {
  model <- list(
    'type'='ume',
    'graph'=igraph::make_graph(c('A','B','A','C','B','C','B','D')))

  expect_equal(matrix(c(1,0,0,0), nrow=4, dimnames=list(NULL, 'd.A.B')),
               mtc.model.call('func.param.matrix', model, t1='A', t2='B'))
  expect_equal(matrix(c(1,0,0,0,0,1,0,0), nrow=4, dimnames=list(NULL, c('d.A.B', 'd.A.C'))),
               mtc.model.call('func.param.matrix', model, t1='A', t2=c('B', 'C')))
  expect_equal(matrix(c(0,0,0,-1), nrow=4, dimnames=list(NULL, c('d.D.B'))),
               mtc.model.call('func.param.matrix', model, t1='D', t2='B'))
  expect_error(mtc.model.call('func.param.matrix', model, t1='A', t2='D'))
})

context("mtc.study.treatment.matrix")

test_that('mtc.study.treatment.matrix with 1 study', {
  data <- data.frame(cbind(c('s02', 's02'), c('A', 'B')))
  colnames(data) <- c('study', 'treatment')
  network <- mtc.network(data)
  m <- mtc.study.treatment.matrix(network)
  expect_equal(1, nrow(m))
})

test_that('mtc.study.treatment.matrix with 2 studies', {
  data <- data.frame(cbind(c('s02', 's02'), c('A', 'B')))
  data <- rbind(data, data.frame(cbind(c('s01', 's01'), c('A', 'B'))))
  
  colnames(data) <- c('study', 'treatment')
  network <- mtc.network(data)
  m <- mtc.study.treatment.matrix(network)
  expect_equal(2, nrow(m))
})

       

context("mtc.model.use")

test_that("the model generates correctly", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
1     A         10.5 0.18
1     B         15.3 0.17
2     B         15.7 0.12
2     C         18.3 0.15
3     B         13.1 0.19
3     C         14.2 0.20'), header=T)
  network <- mtc.network(data.ab)
  model <- mtc.model(network, type='use')
  
  expect_equal(3, length(model$inits[[1]]$mu))
  expect_equal(3, length(model$inits[[2]]$mu))
  expect_equal(3, length(model$inits[[3]]$mu))
  expect_equal(3, length(model$inits[[4]]$mu))
})

context("mtc.model with duplicated arms")

test_that("mtc.model refuses duplicated arms", {
  network <- mtc.network(read.table(textConnection("
    study   treatment  mean std.dev sampleSize
    s01     A          2.0  0.5     20
    s01     B          1.8  0.5     20
    s01     B          1.5  0.5     20"), header=T))
  expect_warning(expect_error(mtc.model(network)))
})

context("[validate] Modified from NICE TSD4 smoking example")

test_that("The summaries match", {
  network <- mtc.network(data.ab=read.table('../data/smoking-ume.data.txt', header=TRUE))
  result <- replicate.example("smoking-ume", network, likelihood="binom", link="logit", type="ume")
  compare.summaries(result$s1, result$s2)
})

context("[validate] NICE TSD2 program 3b")

test_that("The summaries match", {
  network <- mtc.network(data.ab=read.table('../data/diabetes-surv.data.txt', header=TRUE))
  result <- replicate.example("diabetes-surv.fe", network, likelihood="binom", link="cloglog", linearModel="fixed")
  compare.summaries(result$s1, result$s2)
})

context("mtc.nr.comparisons")

test_that('mtc.nr.comparisons with a single study', {
  data <- data.frame(cbind(c('s02', 's02'), c('A', 'B')))
  colnames(data) <- c('study', 'treatment')
  network <- mtc.network(data)
  comparisons <- mtc.nr.comparisons(network)
  expect_identical(as.matrix(data.frame(t1='A', t2='B', nr=1)), as.matrix(comparisons))
})

test_that('mtc.nr.comparisons with 2 studies', {
  data <- data.frame(cbind(c('s02', 's02'), c('A', 'B')))
  data <- rbind(data, data.frame(cbind(c('s01', 's01'), c('A', 'B'))))  
  colnames(data) <- c('study', 'treatment')
  network <- mtc.network(data)
  comparisons <- mtc.nr.comparisons(network)
  expect_identical(as.matrix(data.frame(t1='A', t2='B', nr=2)), as.matrix(comparisons))
})

context("[validate] Risk ratio example")

# Example from http://www.statsdirect.com/help/default.htm#meta_analysis/relative_risk.htms
# Original source https://doi.org/10.1016/0895-4356(91)90261-7

# Results of the Bayesian FE model closely match the frequentist RR of 0.913608 (0.8657, 0.964168)

test_that("The summaries match", {
  network <- mtc.network(data.ab=read.table("../data/riskratio.data.txt", header=TRUE))
  result <- replicate.example("riskratio", network, likelihood="binom", link="log", linearModel="fixed")
  compare.summaries(result$s1, result$s2)
})

context("mtc.model.data");

test_that("mtc.model.data runs on simple data", {
  data <- read.table(textConnection('
study treatment responders sampleSize
1     A         3          10
1     B         4          12'), header=T, stringsAsFactors=T)
  network <- mtc.network(data)
  model <- list(network=network, likelihood='binom', link='logit', om.scale=2.5)
  model <- mtc.model.data(model)
  expect_that(model$r, equals(t(c(3, 4))))
  expect_that(model$n, equals(t(c(10, 12))))
  expect_that(model$t, equals(t(as.numeric(network$data$treatment))))
})

test_that("mtc.model.data complains about NAs in data.ab", {
  data <- read.table(textConnection('
study treatment responders sampleSize
1     A         3          10
1     B         NA         12'), header=T)
  network <- mtc.network(data)
  model <- list(network=network, likelihood='binom', link='logit', om.scale=2.5)
  expect_error(mtc.model.data(model), 'data.ab contains NAs in column "responders"')

  data <- read.table(textConnection('
study treatment responders sampleSize
1     A         3          10
1     B         4          NA'), header=T)
  network <- mtc.network(data)
  model <- list(network=network, likelihood='binom', link='logit', om.scale=2.5)
  expect_error(mtc.model.data(model), 'data.ab contains NAs in column "sampleSize"')
})

test_that("mtc.model.data includes correct fields for data.ab, binom.logit", {
  data.ab <- read.table(textConnection('
study treatment responders sampleSize exposure
1     A         3          10         20
1     B         4          12         24'), header=T)
  network <- mtc.network(data.ab)
  model <- list(network=network, likelihood='binom', link='logit', om.scale=2.5)
  data <- mtc.model.data(model)
  expect_that(sort(names(data)), equals(
    sort(c('studies.a', 'studies', 'nt', 'om.scale', 're.prior.sd', 'na', 't', 'r', 'n'))
  ))
})

test_that("mtc.model.data includes correct fields for data.ab, binom.cloglog", {
  data.ab <- read.table(textConnection('
study treatment responders sampleSize exposure
1     A         3          10         20
1     B         4          12         24'), header=T)
  network <- mtc.network(data.ab)
  model <- list(network=network, likelihood='binom', link='cloglog', om.scale=2.5)
  data <- mtc.model.data(model)
  expect_that(sort(names(data)), equals(
    sort(c(
      'studies.a', 'studies', 'nt', 'om.scale', 're.prior.sd', 'na', 't', 'r', 'n'))
  ))
})

test_that("mtc.model.data includes correct fields for data.ab, poisson.log", {
  data.ab <- read.table(textConnection('
study treatment responders sampleSize exposure
1     A         3          10         20
1     B         4          12         24'), header=T)
  network <- mtc.network(data.ab)
  model <- list(network=network, likelihood='poisson', link='log', om.scale=2.5)
  data <- mtc.model.data(model)
  expect_that(sort(names(data)), equals(
    sort(c(
      'studies.a', 'studies', 'nt', 'om.scale', 're.prior.sd', 'na', 't', 'r', 'E'))
  ))
})

test_that("mtc.model.data includes correct fields for data.ab, normal.identity", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
1     A         3.4  0.21
1     B         4.0  0.19'), header=T)
  network <- mtc.network(data.ab)
  model <- list(network=network, likelihood='normal', link='identity', om.scale=2.5)
  data <- mtc.model.data(model)
  expect_that(sort(names(data)), equals(
    sort(c(
      'studies.a', 'studies', 'nt', 'om.scale', 're.prior.sd', 'na', 't', 'm', 'e'))
  ))
})

test_that("mtc.model.data includes correct fields for data.re", {
  data.re <- read.table(textConnection('
study treatment diff std.err
1     A         NA   0.21
1     B         4.0  0.19'), header=T)
  network <- mtc.network(data.re=data.re)
  model <- list(network=network, likelihood='normal', link='identity', om.scale=2.5)
  data <- mtc.model.data(model)
  expect_that(sort(names(data)), equals(
    sort(c(
      'studies.r2', 'studies', 'nt', 'om.scale', 're.prior.sd', 'na', 't', 'm', 'e'))
  ))
})

test_that("mtc.model.data includes correct fields for data.ab, normal.identity + data.re", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
1     A         3.4  0.21
1     B         4.0  0.19'), header=T)
  data.re <- read.table(textConnection('
study treatment diff std.err
2     A         NA   0.21
2     B         4.0  0.19'), header=T)
  network <- mtc.network(data.ab, data.re=data.re)
  model <- list(network=network, likelihood='normal', link='identity', om.scale=2.5)
  data <- mtc.model.data(model)
  expect_that(sort(names(data)), equals(
    sort(c(
      'studies.a', 'studies.r2', 'studies', 'nt', 'om.scale', 're.prior.sd', 'na', 't', 'm', 'e'))
  ))
})

test_that("mtc.model.data includes correct fields for data.ab, binom.logit + data.re", {
  data.ab <- read.table(textConnection('
study treatment responders sampleSize
1     A         3          10
1     B         4          12'), header=T)
  data.re <- read.table(textConnection('
study treatment diff std.err
2     A         NA   0.19
2     B         4.0  0.21
2     C         2.5  0.22'), header=T)
  network <- mtc.network(data.ab, data.re=data.re)
  model <- list(network=network, likelihood='binom', link='logit', om.scale=2.5)
  data <- mtc.model.data(model)
  expect_that(sort(names(data)), equals(
    sort(c(
      'studies.a', 'studies.rm', 'studies', 'nt', 'om.scale', 're.prior.sd', 'na', 't', 'r', 'n', 'm', 'e'))
  ))
})

test_that("mtc.model.data omits studies with alpha=0 (powerAdjust)", {
  data.ab <- read.table(textConnection('
study treatment responders sampleSize
1     A         3          10
1     B         4          12
1     C         8          20
1     D         7          21
2     B         3          11
2     C         5          13'), header=T)
  data.re <- read.table(textConnection('
study treatment diff std.err
3     A         NA   0.21
3     B         4.0  0.19'), header=T)
  studies <- data.frame('study'=c('1','2','3'), 'x'=c(1,0,0), stringsAsFactors=FALSE)
  network <- mtc.network(data.ab, data.re=data.re, studies=studies)
  model <- list(network=network, likelihood='binom', link='logit', om.scale=2.5, powerAdjust='x')
  data <- mtc.model.data(model)
  expect_equal(data[['studies.a']], 1)
  expect_equal(data[['studies.r2']], NULL)
  expect_equal(data[['studies.rm']], NULL)
  expect_equal(data[['studies']], 1)
})

test_that("mtc.model.data has correct values (1)", {
  data.ab <- read.table(textConnection('
study treatment responders sampleSize
1     A         3          10
1     B         4          12
1     C         8          20
1     D         7          21
2     B         3          11
2     C         5          13'), header=T)
  data.re <- read.table(textConnection('
study treatment diff std.err
3     A         NA   0.21
3     B         4.0  0.19'), header=T)
  network <- mtc.network(data.ab, data.re=data.re)
  model <- list(network=network, likelihood='binom', link='logit', om.scale=2.5)
  data <- mtc.model.data(model)
  expect_equal(data[['om.scale']], model$om.scale)
  expect_equal(data[['studies.a']], 1:2)
  expect_equal(data[['studies.r2']], 3:3)
  expect_equal(data[['studies.rm']], NULL)
  expect_equal(data[['studies']], 1:3)
  expect_equal(data[['nt']], 4)
  expect_equal(data[['na']], c(4,2,2))
  expect_that(data[['t']], equals(rbind(
    c(  1,  2,  3,  4),
    c(  2,  3, NA, NA),
    c(  1,  2, NA, NA)
  )))
  expect_that(data[['r']], equals(rbind(
    c(  3,  4,  8,  7),
    c(  3,  5, NA, NA),
    c( NA, NA, NA, NA)
  )))
  expect_that(data[['n']], equals(rbind(
    c( 10, 12, 20, 21),
    c( 11, 13, NA, NA),
    c( NA, NA, NA, NA)
  )))
  expect_that(data[['m']], equals(rbind(
    c( NA,  NA, NA, NA),
    c( NA,  NA, NA, NA),
    c( NA, 4.0, NA, NA)
  )))
  expect_that(data[['e']], equals(rbind(
    c(   NA,   NA, NA, NA),
    c(   NA,   NA, NA, NA),
    c( 0.21, 0.19, NA, NA)
  )))
})

test_that("mtc.model.data has correct values (2)", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
1     A         10.5 0.18
1     B         15.3 0.17
2     B         15.7 0.12
2     C         18.3 0.15
3     B         13.1 0.19
3     C         14.2 0.20'), header=T)
  data.re <- read.table(textConnection('
study treatment diff std.err
4     A         NA   0.15
4     B         3.1  0.22
4     C         4.2  0.24
5     A         2.9  0.28
5     B         NA   NA
6     A         0.2  0.23
6     B         0.8  0.27
6     C         NA   0.16
7     A         NA   0.18
7     B         4.3  0.21
8     B         NA   NA
8     C         4.0  0.19'), header=T)
  network <- mtc.network(data.ab, data.re=data.re)
  model <- list(network=network, likelihood='normal', link='identity', om.scale=2.5)
  data <- mtc.model.data(model)
  expect_equal(data[['om.scale']], model$om.scale)
  expect_equal(data[['studies.a']], 1:3)
  expect_equal(data[['studies.r2']], 4:6)
  expect_equal(data[['studies.rm']], 7:8)
  expect_equal(data[['studies']], 1:8)
  expect_equal(data[['nt']], 3)
  expect_equal(data[['na']], c(2,2,2,2,2,2,3,3))
  expect_that(data[['t']], equals(rbind(
    c(  1,  2, NA), # 1
    c(  2,  3, NA), # 2
    c(  2,  3, NA), # 3
    c(  2,  1, NA), # 5
    c(  1,  2, NA), # 7
    c(  2,  3, NA), # 8
    c(  1,  2,  3), # 4
    c(  3,  1,  2)  # 6
  )))
  expect_that(data[['m']], equals(rbind(
    c( 10.5, 15.3,   NA), # 1
    c( 15.7, 18.3,   NA), # 2
    c( 13.1, 14.2,   NA), # 3
    c(   NA,  2.9,   NA), # 5
    c(   NA,  4.3,   NA), # 7
    c(   NA,  4.0,   NA), # 8
    c(   NA,  3.1,  4.2), # 4
    c(   NA,  0.2,  0.8)  # 6
  )))
  expect_that(data[['e']], equals(rbind(
    c( 0.18, 0.17,   NA), # 1
    c( 0.12, 0.15,   NA), # 2
    c( 0.19, 0.20,   NA), # 3
    c(   NA, 0.28,   NA), # 5
    c( 0.18, 0.21,   NA), # 7
    c(   NA, 0.19,   NA), # 8
    c( 0.15, 0.22, 0.24), # 4
    c( 0.16, 0.23, 0.27)  # 6
  )))
})

test_that("not specifying likelihood / link generates warnings", {
  data.re <- read.table(textConnection("
  study  treatment  diff  std.err
  s01    A          NA    NA
  s01    C          -0.31 0.67
  s02    A          NA    NA
  s02    B          -1.7  0.38
  s03    C          NA    NA
  s03    D          -0.35 0.44
  s04    C          NA    NA
  s04    D          0.55  0.55
  s05    D          NA    NA
  s05    E          -0.3  0.27
  s06    D          NA    NA
  s06    E          -0.3  0.32
  s07    A          NA    0.50
  s07    B          -2.3  0.72
  s07    D          -0.9  0.69"), header=T)

  network <- mtc.network(data.re=data.re)

  expect_warning(model <- mtc.model(network, likelihood='normal'), 'Link can not be inferred. Defaulting to identity.')
  expect_that(model$link, equals('identity'))
  expect_warning(model <- mtc.model(network, link='identity'), 'Likelihood can not be inferred. Defaulting to normal.')
  expect_that(model$likelihood, equals('normal'))

  expect_error(mtc.model(network, link='logit', likelihood='normal'))
})

context("has.indirect.evidence")

test_that("single pair-wise comparison gives FALSE", {
  data <- read.table(textConnection('study treatment
s01 A
s01 B'), header=T)
  network <- mtc.network(data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(FALSE))
})

test_that("comparison in triangle gives TRUE", {
  data <- read.table(textConnection('study treatment
s01 A
s01 B
s02 B
s02 C
s03 C
s03 A'), header=T)
  network <- mtc.network(data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(TRUE))

  data <- read.table(textConnection('study treatment
s01 A
s01 B
s01 C
s02 B
s02 C
s03 C
s03 A'), header=T)
  network <- mtc.network(data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(TRUE))
})

test_that("comparison in three-arm trial gives FALSE", {
  data <- read.table(textConnection('study treatment
s01 A
s01 B
s01 C'), header=T)
  network <- mtc.network(data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(FALSE))

  data <- read.table(textConnection('study treatment
s01 A
s01 B
s01 C
s02 A
s02 C'), header=T)
  network <- mtc.network(data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(FALSE))
})

test_that("four-arm trials are handled correctly", {
  data <- read.table(textConnection('study treatment
s01 A
s01 B
s01 C
s01 D'), header=T)
  network <- mtc.network(data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(FALSE))

  data <- read.table(textConnection('study treatment
s01 A
s01 B
s01 C
s01 D
s02 B
s02 C'), header=T)
  network <- mtc.network(data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(FALSE))

  data <- read.table(textConnection('study treatment
s01 A
s01 B
s01 C
s01 D
s02 B
s02 C
s03 A
s03 D'), header=T)
  network <- mtc.network(data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(FALSE)) # used to be TRUE pre-0.6
})

test_that("data.re is incorporated", {
  data <- read.table(textConnection('study treatment diff std.err
s01 A NA 0.5
s01 B 1.0 0.8
s01 C -1.0 0.8'), header=T)
  network <- mtc.network(data.re=data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(FALSE))

  data <- read.table(textConnection('study treatment diff std.err
s01 A NA 0.5
s01 B 1.0 0.8
s01 C -1.0 0.8
s02 A NA NA
s02 C 0.1 0.8
s03 B NA NA
s03 C -1.8 0.8'), header=T)
  network <- mtc.network(data.re=data)
  expect_that(has.indirect.evidence(network, 'A', 'B'), equals(TRUE))
})

context("[validate] NICE TSD2 program 3a")

test_that("The summaries match", {
  network <- mtc.network(data.ab=read.table('../data/diabetes-surv.data.txt', header=TRUE))
  result <- replicate.example("diabetes-surv", network, likelihood="binom", link="cloglog")
  compare.summaries(result$s1, result$s2)
})

template.block.sub <- function(template, var, val) {
  while (length(grep(paste0('$', var, '$'), template, fixed=TRUE)) > 0) {
    matches <- regexec(paste("([ \t]*)[^\\$\n]*\\$", var, "\\$",sep=""), template)[[1]]
    start <- matches[2]
    len <- attr(matches, "match.length")[2]
    indent <- substr(template, start, start + len - 1)
    val <- paste(strsplit(val, "\n")[[1]], collapse=paste("\n", indent, sep=""))
    template <- sub(paste0('$', var, '$'), val, template, fixed=TRUE)
  }
  template
}

read.template <- function(file.name) {
    fileName <- system.file(file.name, package='gemtc')
    readChar(fileName, file.info(fileName)[['size']])
}

context("template")

test_that("template.block.sub for a single line keeps indent", {
  template <- "model {\n\t$model$\n}\n"
  expect_that(template.block.sub(template, 'model', 'foo'), equals("model {\n\tfoo\n}\n"))
})

test_that("template.block.sub for multi-line keeps indent", {
  template <- "model {\n\t$model$\n}\n"
  expect_that(template.block.sub(template, 'model', 'foo\nbar'), equals("model {\n\tfoo\n\tbar\n}\n"))
})

test_that("template.block.sub keeps other stuff on the same line", {
  template <- "model {\n\tbaz($model$)\n}\n"
  expect_that(template.block.sub(template, 'model', 'foo\nbar'), equals("model {\n\tbaz(foo\n\tbar)\n}\n"))
})

test_that("template.block.sub keeps other stuff with white space on the same line", {
  template <- "model {\n\tbaz baz($model$)\n}\n"
  expect_that(template.block.sub(template, 'model', 'foo\nbar'), equals("model {\n\tbaz baz(foo\n\tbar)\n}\n"))
})

#' @include stopIfNotConsistent.R
#' @include regression.R

treatment.pairs <- function(t1, t2, ts) {
  stopifnot(is.factor(ts))
  std <- function(tx) {
    if (is.numeric(tx)) factor(tx, levels=1:nlevels(ts), labels=levels(ts))
    else if (is.character(tx)) factor(tx, levels=levels(ts))
    else tx
  }
  t1 <- std(t1)
  t2 <- std(t2)
  if((is.null(t2) || length(t2) == 0) && length(t1) == 1) {
    t2 <- ts[ts != t1]
  }
  if(length(t1) > length(t2)) t2 <- rep(t2, length.out=length(t1))
  if(length(t2) > length(t1)) t1 <- rep(t1, length.out=length(t2))
  matrix(c(as.numeric(t1), as.numeric(t2)), ncol=2)
}

# basic parameters are rows, derived parameters (i.e. the relative effects) are columns
tree.relative.effect <- function(g, t1, t2) {
  ts <- factor(V(g)$name, levels=V(g)$name)
  pairs <- treatment.pairs(t1, t2, ts)
  paths <- apply(pairs, 1, function(rel) {
    p <- suppressWarnings(get.shortest.paths(g, rel[1], rel[2], mode='all'))
    if (is.list(p) && "vpath" %in% names(p)) { p <- p$vpath }
    p <- unlist(p)
    if (length(p) == 0) {
      stop(paste("The requested comparison ",
             V(g)[rel[2]]$name, " vs ", V(g)[rel[1]]$name,
             " can not be expressed as a linear combination of the ",
             "available parameters", sep=""))
    }
    p <- matrix(c(p[1:length(p)-1], p[-1]), ncol=2)

    sapply(E(g), function(e) {
      v <- ends(g, e, names=FALSE)
      if (sum(p[,1] == v[1] & p[,2] == v[2])) 1
      else if (sum(p[,1] == v[2] & p[,2] == v[1])) -1
      else 0
    })
  })

  # Ensure paths is a matrix, since apply() will simplify to a vector if
  # either ncol==1 or nrow==1
  paths <- matrix(as.numeric(paths), nrow=length(E(g)))

  colnames(paths) <- apply(pairs, 1, function(pair) {
    pair <- V(g)[pair]$name
    paste('d', pair[1], pair[2], sep='.')
  })
  paths
}

# Given a vector of parameters names (d.A.B, sd.d, etc.), extract all basic
# parameters and generate a two-column matrix in which each row is a comparison
extract.comparisons <- function(parameters) {
  x <- parameters[grep("^d\\.", parameters)]
  matrix(unlist(strsplit(x, "\\.")), ncol=3, byrow=TRUE)[ , -1, drop=FALSE]
}

spanning.tree.mtc.result <- function(result) {
  network <- result[['model']][['network']]
  pairs <- extract.comparisons(colnames(result[['samples']][[1]]))
  pairs <- matrix(pairs, ncol=2)
  parameters <- data.frame(
    t1=as.treatment.factor(pairs[,1], network),
    t2=as.treatment.factor(pairs[,2], network)
  )
  graph.create(network[['treatments']][['id']], parameters, arrow.mode=2, color="black", lty=1)
}

relative.effect <- function(result, t1, t2 = c(), preserve.extra=TRUE, covariate=NA) {
  stopIfNotConsistent(result, 'relative.effect')

  # Build relative effect transformation matrix
  network <- result[['model']][['network']]
  g <- spanning.tree.mtc.result(result)
  if (is.character(t1)) {
    t1 <- as.treatment.factor(t1, network)
  }
  if (is.character(t2)) {
    t2 <- as.treatment.factor(t2, network)
  }
  effects <- tree.relative.effect(g, t1, t2)

  parnames <- colnames(result[['samples']][[1]])
  parameters <- grep("^d\\.", parnames)

  if (result[['model']][['type']] == 'regression' && !is.na(covariate)) {
    model <- result[['model']]
    regressor <- model[['regressor']]

    if (regressor[['type']] == 'binary' && !(covariate %in% 0:1)) {
      stop("Invalid value for binary covariate")
    }

    nt <- nrow(model[['network']][['treatments']])
    nc <- length(model[['regressor']][['classes']])
    regression.parameters <- sapply(regressionParams(regressor, nt, nc), function(p) { which(parnames == p) })
    parameters <- c(parameters, regression.parameters)

    transform <- regressionAdjustMatrix(t1, t2, regressor, nt)
    transform <- transform * (covariate - regressor[['center']]) / regressor[['scale']]
    effects <- rbind(effects, transform)
  }

  # Apply tranformation to each chain
  samples <- as.mcmc.list(lapply(result[['samples']], function(chain) {
    samples <- chain[,parameters] %*% effects
    if (preserve.extra) {
      samples <- cbind(samples, chain[,-parameters,drop=FALSE])
    }
    mcmc(samples, start=start(chain), end=end(chain), thin=thin(chain))
  }))
  effects <- list(
    samples=samples,
    model=result[['model']],
    sampler=result[['sampler']],
    covariate=covariate)

  class(effects) <- "mtc.result"
  effects
}

context("relative.effect and rank.probability")

test_that("relative.effect outputs the correct parameters", {
  result <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))

  expect_that(colnames(result$samples[[1]]), equals(c("d.A.B", "d.A.C", "d.A.D", "sd.d")))

  out <- relative.effect(result, "A", preserve.extra=TRUE)
  expect_that(colnames(out$samples[[1]]), equals(c("d.A.B", "d.A.C", "d.A.D", "sd.d"))) #1

  out <- relative.effect(result, "A", preserve.extra=FALSE)
  expect_that(colnames(out$samples[[1]]), equals(c("d.A.B", "d.A.C", "d.A.D"))) #2

  out <- relative.effect(result, "B")
  expect_that(colnames(out$samples[[1]]), equals(c("d.B.A", "d.B.C", "d.B.D", "sd.d"))) #3

  out <- relative.effect(result, "B", "C")
  expect_that(colnames(out$samples[[1]]), equals(c("d.B.C", "sd.d")))

  out <- relative.effect(result, "B", c("A", "B", "C"))
  expect_that(colnames(out$samples[[1]]), equals(c("d.B.A", "d.B.B", "d.B.C", "sd.d")))

  out <- relative.effect(result, c("A", "B"), c("C"))
  expect_that(colnames(out$samples[[1]]), equals(c("d.A.C", "d.B.C", "sd.d")))
})

test_that("relative.effect generates the expected statistics", {
  result <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))
  stats <- summary(relative.effect(result, "B"))$summaries

  expected <- textConnection('
           Mean     SD  Naive.SE Time-series.SE
  d.B.A -0.4965 0.4081 0.004563       0.004989
  d.B.C  0.3394 0.4144 0.004634       0.004859
  d.B.D  0.6123 0.4789 0.005354       0.005297
  sd.d   0.8465 0.1913 0.002139       0.002965
  ')
  expected <- as.matrix(read.table(expected, header=TRUE))
  colnames(expected)[3] <- "Naive SE"
  colnames(expected)[4] <- "Time-series SE"
  expect_that(stats$statistics, equals(expected, tolerance=0.0001, scale=1))
})

test_that("tree.relative.effect handles a simple tree", {
  g <- graph.edgelist(t(matrix(c("A", "B", "A", "C", "A", "D"), nrow=2)))

  expected <- do.call(cbind, list(
    d.B.A = c(-1, 0, 0),
    d.B.C = c(-1, 1, 0),
    d.B.D = c(-1, 0, 1))
  )

  expect_that(tree.relative.effect(g, t1=2, t2=c()), equals(expected))
})

test_that("tree.relative.effect handles a more complex tree", {
  network <- thrombolytic
  tree <- minimum.diameter.spanning.tree(mtc.network.graph(network))

  expected <- do.call(cbind, list(
      d.tPa.Ten = c(
        1, 0, -1, # +ASPAC.AtPA -ASPAC.tPA
        0, 0, 1, 0) # +AtPA.Ten
      ))
  expect_that(tree.relative.effect(tree, t1="tPA", t2="Ten"), is_equivalent_to(expected))
})

test_that("tree.relative.effect handles two-treatment case", {
  g <- graph.edgelist(matrix(c("A", "B"), ncol=2))
  expected <- matrix(-1, dimnames=list(NULL, c("d.B.A")))
  expect_that(tree.relative.effect(g, t1=2, t2=c()), equals(expected))

  expected <- matrix(c(0, 1), dimnames=list(NULL, c("d.A.A", "d.A.B")), ncol=2)
  expect_that(tree.relative.effect(g, t1=1, t2=c(1, 2)), equals(expected))
})

test_that("relative.effect can be applied recursively", {
  result <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))
  result <- relative.effect(result, "C")
  stats <- summary(relative.effect(result, "B"))$summaries

  expected <- textConnection('
           Mean     SD  Naive.SE Time-series.SE
  d.B.A -0.4965 0.4081 0.004563       0.004989
  d.B.C  0.3394 0.4144 0.004634       0.004859
  d.B.D  0.6123 0.4789 0.005354       0.005297
  sd.d   0.8465 0.1913 0.002139       0.002965
  ')
  expected <- as.matrix(read.table(expected, header=TRUE))
  colnames(expected)[3] <- "Naive SE"
  colnames(expected)[4] <- "Time-series SE"
  expect_that(stats$statistics, equals(expected, tolerance=0.0001, scale=1))
})

test_that("rank.probability returns the right results", {
  expected <- c(0, 0.058, 0.22825, 0.71375, 0.0025, 0.176, 0.60075, 
                0.22075, 0.105125, 0.66175, 0.170625, 0.0625, 0.892375, 0.10425, 
                0.000375, 0.003)
  result <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))
  expect_equal(as.numeric(rank.probability(result)), expected)
})

test_that("rank.probability can be applied to a subset of parameters", {
  result <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))
  result <- relative.effect(result, "A", c("B", "C"))
  rank.probability(result)
  succeed()
})

test_that("relative.effect.tree throws an error if requested comparison is not connected", {
  g <- graph.edgelist(t(matrix(c("A", "B", "A", "C"), nrow=2)))
  g <- g + vertex("D")

  expect_error(tree.relative.effect(g, t1=1, t2=4))
})

test_that("spanning.tree.mtc.result handles two-treatment case", {
  result <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))
  result <- relative.effect(result, t1="A", t2="B")
  
  g <- graph.edgelist(matrix(c("A", "B"), ncol=2))
  g <- g + vertices(c("C", "D"))

  h <- spanning.tree.mtc.result(result)

  expect_that(V(h)$name, equals(V(g)$name))
  expect_that(h[,], equals(g[,]))
})

test_that("relative.effect is robust to missing sd.d", {
  result <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))

  out <- relative.effect(result, "A", preserve.extra=FALSE)
  expect_that(colnames(out$samples[[1]]), equals(c("d.A.B", "d.A.C", "d.A.D"))) #2

  out <- relative.effect(out, "A", preserve.extra=TRUE)
  expect_that(colnames(out$samples[[1]]), equals(c("d.A.B", "d.A.C", "d.A.D"))) #1
})

test_that("relative.effect is robust to leading columns", {
  result <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))
  leading <- matrix(rep(255,nrow(result[['samples']][[1]])), dimnames=list(NULL, "deviance"))
  for (i in 1:4) {
    result[['samples']][[i]] <- as.mcmc(cbind(leading, result[['samples']][[i]]))
  }

  stats <- summary(relative.effect(result, "B", preserve.extra=FALSE))$summaries

  expected <- textConnection('
           Mean     SD  Naive.SE Time-series.SE
  d.B.A -0.4965 0.4081 0.004563       0.004989
  d.B.C  0.3394 0.4144 0.004634       0.004859
  d.B.D  0.6123 0.4789 0.005354       0.005297
  ')
  expected <- as.matrix(read.table(expected, header=TRUE))
  colnames(expected)[3] <- "Naive SE"
  colnames(expected)[4] <- "Time-series SE"
  expect_that(stats$statistics, equals(expected, tolerance=0.0001, scale=1))
})

as.character.mtc.hy.prior <- function(x, ...) {
  type <- x[['type']]
  distr <- x[['distr']]
  args <- x[['args']]
  
  expr <- if (distr == "dhnorm") {
    paste0("dnorm", "(", paste(args, collapse=", "), ") T(0,)")
  } else {
    paste0(distr, "(", paste(args, collapse=", "), ")")
  }

  if (type == "std.dev") {
    paste0("sd.d ~ ", expr, "\ntau.d <- pow(sd.d, -2)")
  } else if (type == "var") {
    paste0("var.d ~ ", expr, "\nsd.d <- sqrt(var.d)\ntau.d <- 1 / var.d")
  } else {
    paste0("tau.d ~ ", expr, "\nsd.d <- sqrt(1 / tau.d)")
  }
}

mtc.hy.prior <- function(type, distr, ...) {
  stopifnot(class(type) == "character")
  stopifnot(length(type) == 1)
  stopifnot(type %in% c('std.dev', 'var', 'prec'))

  obj <- list(type=type, distr=distr, args=list(...))
  class(obj) <- "mtc.hy.prior"
  obj
}

hy.lor.outcomes <- c('mortality', 'semi-objective', 'subjective')
hy.lor.comparisons <- c('pharma-control', 'pharma-pharma', 'non-pharma')

hy.lor.mu <- matrix(
  c(-4.06, -3.02, -2.13, -4.27, -3.23, -2.34, -3.93, -2.89, -2.01),
  ncol=3, nrow=3,
  dimnames=list(hy.lor.outcomes, hy.lor.comparisons))

hy.lor.sigma <- matrix(
  c(1.45, 1.85, 1.58, 1.48, 1.88, 1.62, 1.51, 1.91, 1.64),
  ncol=3, nrow=3,
  dimnames=list(hy.lor.outcomes, hy.lor.comparisons))

mtc.hy.empirical.lor <- function(outcome.type, comparison.type) {
  stopifnot(outcome.type %in% hy.lor.outcomes)
  stopifnot(comparison.type %in% hy.lor.comparisons)
  mtc.hy.prior("var", "dlnorm",
    hy.lor.mu[outcome.type, comparison.type],
    signif(hy.lor.sigma[outcome.type, comparison.type]^-2, digits=3))
}

context("mtc.hy.prior")

test_that("the standard uniform prior on standard deviation is generated correctly", {
  expect_that(as.character(mtc.hy.prior("std.dev", "dunif", 0, "om.scale")), equals("sd.d ~ dunif(0, om.scale)\ntau.d <- pow(sd.d, -2)"))
  expect_that(as.character(mtc.hy.prior("std.dev", "dunif", 0, 5)), equals("sd.d ~ dunif(0, 5)\ntau.d <- pow(sd.d, -2)"))
  expect_that(as.character(mtc.hy.prior("std.dev", "dunif", 0.1, 2)), equals("sd.d ~ dunif(0.1, 2)\ntau.d <- pow(sd.d, -2)"))
})

test_that("other priors can be specified by name", {
  expect_that(as.character(mtc.hy.prior("std.dev", "dgamma", 0.01, 0.01)), equals("sd.d ~ dgamma(0.01, 0.01)\ntau.d <- pow(sd.d, -2)"))
  expect_that(as.character(mtc.hy.prior("std.dev", "dlnorm", -1.3, 0.49)), equals("sd.d ~ dlnorm(-1.3, 0.49)\ntau.d <- pow(sd.d, -2)"))
})

test_that("priors can have more or less than two parameters", {
  expect_that(as.character(mtc.hy.prior("std.dev", "dgamma", 0.01, 0.02, 0.03)), equals("sd.d ~ dgamma(0.01, 0.02, 0.03)\ntau.d <- pow(sd.d, -2)"))
  expect_that(as.character(mtc.hy.prior("std.dev", "dgamma", 0.01)), equals("sd.d ~ dgamma(0.01)\ntau.d <- pow(sd.d, -2)"))
})

test_that("the prior can be specified on the variance", {
  expect_that(as.character(mtc.hy.prior("var", "dlnorm", -1.3, 0.49)), equals("var.d ~ dlnorm(-1.3, 0.49)\nsd.d <- sqrt(var.d)\ntau.d <- 1 / var.d"))
})

test_that("the prior can be specified on the precision", {
  expect_that(as.character(mtc.hy.prior("prec", "dlnorm", -1.3, 0.49)), equals("tau.d ~ dlnorm(-1.3, 0.49)\nsd.d <- sqrt(1 / tau.d)"))
})

test_that("LOR empirical priors have correct values", {
  expect_that(as.character(mtc.hy.empirical.lor("mortality", "pharma-control")), equals("var.d ~ dlnorm(-4.06, 0.476)\nsd.d <- sqrt(var.d)\ntau.d <- 1 / var.d"))
  expect_that(as.character(mtc.hy.empirical.lor("mortality", "pharma-pharma")), equals("var.d ~ dlnorm(-4.27, 0.457)\nsd.d <- sqrt(var.d)\ntau.d <- 1 / var.d"))
  expect_that(as.character(mtc.hy.empirical.lor("subjective", "pharma-pharma")), equals("var.d ~ dlnorm(-2.34, 0.381)\nsd.d <- sqrt(var.d)\ntau.d <- 1 / var.d"))
})

#' @include template.R

mtc.model.code <- function(model, params, relEffectMatrix, template='gemtc.model.template.txt',
                           linearModel='delta[i, k]', regressionPriors='') {
  powerAdjust <- !is.null(model[['powerAdjust']]) && !is.na(model[['powerAdjust']])

  template <- read.template(template)

  if (length(model[['data']][['studies.a']]) > 0) {
    arm.code <- read.template('gemtc.armeffect.likelihood.txt')
    template <- template.block.sub(template, 'armeffect', arm.code)
    lik.code <- do.call(paste("mtc.code.likelihood", model[['likelihood']], model[['link']], sep="."), list(powerAdjust=powerAdjust))
    template <- template.block.sub(template, 'likelihood', lik.code)
  } else {
    template <- template.block.sub(template, 'armeffect', '## OMITTED')
  }

  if (length(model[['data']][['studies.r2']]) > 0) {
    rel.code <-
      if (powerAdjust) read.template('gemtc.releffect.likelihood.power.r2.txt')
      else read.template('gemtc.releffect.likelihood.r2.txt')
    template <- template.block.sub(template, 'releffect.r2', rel.code)
  } else {
    template <- template.block.sub(template, 'releffect.r2', '## OMITTED')
  }

  if (length(model[['data']][['studies.rm']]) > 0) {
    rel.code <-
      if (powerAdjust) read.template('gemtc.releffect.likelihood.power.rm.txt')
      else read.template('gemtc.releffect.likelihood.rm.txt')
    template <- template.block.sub(template, 'releffect.rm', rel.code)
  } else {
    template <- template.block.sub(template, 'releffect.rm', '## OMITTED')
  }

  template <- template.block.sub(template, 'armLinearModel', paste0('mu[i] + ', linearModel))
  template <- template.block.sub(template, 'relLinearModel', linearModel)

  hyModel <- if (model[['linearModel']] == "fixed") {
    read.template('gemtc.fixedeffect.txt')
  } else {
    read.template('gemtc.randomeffects.txt')
  }
  template <- template.block.sub(template, 'heterogeneityModel', hyModel)

  # substitute in heterogeneity prior
  template <- template.block.sub(template, 'hy.prior', as.character(model[['hy.prior']]))

  template <- template.block.sub(template, 'relativeEffectMatrix', relEffectMatrix)

  if (length(model[['data']][['studies.a']]) > 0) {
    sbPriors <- ll.call('study.baseline.priors', model)
    template <- template.block.sub(template, 'studyBaselinePriors', sbPriors)
  } else {
    template <- template.block.sub(template, 'studyBaselinePriors', '## OMITTED')
  }

  # Generate parameter priors
  priors <- paste(params, "~", "dnorm(0, prior.prec)", collapse="\n")
  template <- template.block.sub(template, 'relativeEffectPriors', priors)

  template <- template.block.sub(template, 'regressionPriors', regressionPriors)

  template
}

ll.call <- function(fnName, model, ...) {
  fn <- paste(fnName, model[['likelihood']], model[['link']], sep=".")
  do.call(fn, list(...))
}

ll.defined <- function(model) {
  fns <- c('mtc.arm.mle', 'mtc.rel.mle', 'mtc.code.likelihood',
    'fitted.values.parameter', 'deviance_fn',
    'scale.log', 'scale.name', 'inits.info',
    'required.columns.ab', 'validate.data',
    'study.baseline.priors')
  fns <- paste(fns, model[['likelihood']], model[['link']], sep=".")
  all(sapply(fns, function(fn) { exists(fn, mode='function') }))
}

context("likelihood/link");

# This file contains a single test for each implemented likelihood.
# Since ll.defined() checks whether all required methods are implemented,
# this is a simple guard against forgetting to add or correctly name new
# methods.

test_that("normal.identity is defined", {
  expect_true(ll.defined(list(likelihood='normal', link='identity')))
})

test_that("binom.logit is defined", {
  expect_true(ll.defined(list(likelihood='binom', link='logit')))
})

test_that("binom.log is defined", {
  expect_true(ll.defined(list(likelihood='binom', link='log')))
})

test_that("binom.cloglog is defined", {
  expect_true(ll.defined(list(likelihood='binom', link='cloglog')))
})

test_that("poisson.log is defined", {
  expect_true(ll.defined(list(likelihood='poisson', link='log')))
})

# Node-split model
mtc.model.nodesplit <- function(model, t1, t2) {
  # Make sure t1 and t2 are 'character' and t1 < t2
  network <- model[['network']]
  ts <- factor(c(t1, t2), levels=levels(network[['treatments']][['id']]))
  if (as.numeric(ts[1]) > as.numeric(ts[2])) {
    ts <- rev(ts)
  }
  t1 <- as.character(ts[1])
  t2 <- as.character(ts[2])

  stopifnot(has.indirect.evidence(network, t1, t2))

  # Rewrite the dataset
  data.ab <- nodesplit.rewrite.data.ab(network[['data.ab']], t1, t2)
  data.re <- nodesplit.rewrite.data.re(network[['data.re']], t1, t2)
  studies <- nodesplit.rewrite.studies(network, t1, t2)
  network <- mtc.network(data.ab=data.ab,
                         data.re=data.re,
                         treatments=model[['network']][['treatments']],
                         studies=studies)
  model[['network']] <- network

  # Construct indirect network
  has.both <- sapply(mtc.studies.list(network)$values,
         function(study) {
           all(c(t1, t2) %in% mtc.study.design(network, study))
         })
  network.indirect <- filter.network(network,
                                     function(row) {
                                       !has.both[row['study']]
                                     })

  # Generate tree for indirect evidence
  style.tree <- function(tree) {
    tree <- set.edge.attribute(tree, 'arrow.mode', value=2)
    tree <- set.edge.attribute(tree, 'color', value='black')
    tree <- set.edge.attribute(tree, 'lty', value=1)
    tree
  }
  tree.indirect <- connect.mds.forest(mtc.network.graph(network.indirect))
  model[['tree.indirect']] <- style.tree(tree.indirect)
  model[['graph']] <- model[['tree.indirect']] + edge(c(t1, t2))
  model[['graph']] <- set.edge.attribute(model[['graph']], 'arrow.mode', index=length(E(model[['graph']])), value=2)
  model[['graph']] <- set.edge.attribute(model[['graph']], 'color', index=length(E(model[['graph']])), value='black')
  model[['graph']] <- set.edge.attribute(model[['graph']], 'lty', index=length(E(model[['graph']])), value=2)

  model[['data']] <- mtc.model.data(model)
  model[['data']][['split']] <- as.numeric(ts)
  model[['split']] <- ts
  model[['inits']] <- mtc.init(model)

  model[['code']] <- mtc.model.code(model, mtc.basic.parameters(model), nodesplit.relative.effect.matrix(model, tree.indirect))

  monitors <- c(inits.to.monitors(model[['inits']][[1]]), 'd.direct', 'd.indirect')
  model[['monitors']] <- list(
    available=monitors,
    enabled=c(monitors[grep('^d\\.', monitors)], monitors[grep('^sd.d$', monitors)])
  )

  class(model) <- "mtc.model"

  model
}

mtc.model.name.nodesplit <- function(model) {
  paste("node-split (", model[['t1']], " / ", model[['t2']], ")", sep="")
}

func.param.matrix.nodesplit <- function(model, t1, t2) {
  base <- tree.relative.effect(model[['tree.indirect']], t1, t2)
  m <- rbind(base, rep(0, ncol(base)))
  param.pos <- paste("d", model[['split']][1], model[['split']][2], sep=".")
  param.neg <- paste("d", model[['split']][2], model[['split']][1], sep=".")
  if (param.pos %in% colnames(base)) {
    m[, param.pos] <- c(rep(0, nrow(base)), 1)
  }
  if (param.neg %in% colnames(base)) {
    m[, param.neg] <- c(rep(0, nrow(base)), -1)
  }
  m
}

nodesplit.relative.effect.matrix <- function(model, tree) {
  # Generate list of linear expressions
  params <- mtc.basic.parameters(model)
  re <- tree.relative.effect(tree, V(tree)[1], t2=NULL)
  expr <- apply(re, 2, function(col) { paste(sapply(which(col != 0), function(i) {
    paste(if (col[i] == -1) "-" else "", params[i], sep="")
  }), collapse = " + ") })
  expr <- sapply(1:length(expr), function(i) { paste('d1[', i + 1, '] <- ', expr[i], sep='') })
  expr <- c('d1[1] <- 0', expr, 'for (i in 1:nt) {\n\tfor (j in 1:nt) {\n\t\tis.split[i, j] <- equals(i, split[1]) * equals(j, split[2]) + equals(i, split[2]) * equals(j, split[1])\n\t\td[i, j] <- (1 - is.split[i, j]) * (d1[j] - d1[i]) + is.split[i, j] * (2 * equals(i, split[1]) - 1) * d.direct\n\t}\n}')
  expr <- c(expr, paste('d.direct <- d', model[['split']][1], model[['split']][2], sep='.'))
  expr <- c(expr, 'd.indirect <- d1[split[2]] - d1[split[1]]')
  paste(expr, collapse="\n")
}

#' @include forest.R
#' @include stopIfNotConsistent.R

## mtc.result class methods
print.mtc.result <- function(x, ...) {
  cat("MTC ", x[['model']][['type']], " results: ", x[['model']][['description']], sep="")
  print(x[['samples']])
}

summary.mtc.result <- function(object, ...) {
  scale.log <- if (ll.call('scale.log', object[['model']])) 'Log ' else ''
  scale.name <- ll.call('scale.name', object[['model']])
  rval <- list('measure'=paste0(scale.log, scale.name),
       'summaries'=summary(object[['samples']]),
       'DIC'=unlist(object[['deviance']][c('Dbar', 'pD', 'DIC', 'data points')]),
       'regressor'=object[['model']][['regressor']],
       'covariate'=object[['covariate']])
  class(rval) <- 'summary.mtc.result'
  rval
}

print.summary.mtc.result <- function(x, ...) {
  cat(paste("\nResults on the", x[['measure']], "scale\n"))
  print(x[['summaries']])
  if (!is.null(x[['DIC']])) {
    cat("-- Model fit (residual deviance):\n\n")
    dic <- x[['DIC']]
    print(dic[c('Dbar', 'pD', 'DIC')])
    cat(paste0("\n", dic['data points'], " data points, ratio ",
               format(dic['Dbar'] / dic['data points'], digits=4),
               ", I^2 = ", format(100 * max(0, min(1, (dic['Dbar'] - dic['data points'] + 1)/dic['Dbar'])), digits=1),
               "%\n"))
  }
  if (!is.null(x[['regressor']])) {
    cat("\n-- Regression settings:\n\n")
    r <- x[['regressor']]
    if (!is.null(x[['regressor']][['classes']])) {
      cat(paste0("Regression on \"", r[['variable']], "\", ", r[['coefficient']], " coefficients, by class\n"))
    } else {
      cat(paste0("Regression on \"", r[['variable']], "\", ", r[['coefficient']], " coefficients, \"", r[['control']], "\" as control\n"))
    }
    if (!is.null(x[['covariate']])) {
      cat(paste0("Values at ", r[['variable']], " = ", x[['covariate']], "\n"))
    } else {
      cat(paste0("Input standardized: x' = (", r[['variable']], " - ", format(r[['center']], digits=getOption("digits")), ") / ", format(r[['scale']], digits=getOption("digits")), "\n"))
      cat(paste0("Estimates at the centering value: ", r[['variable']], " = ", format(r[['center']], digits=getOption("digits")), "\n"))
    }
  }
  cat("\n")
}

plot.mtc.result <- function(x, ...) {
  plot(x[['samples']], ...)
}

forest.mtc.result <- function(x, use.description=FALSE, ...) {
  stopIfNotConsistent(x, "forest.mtc.result")

  varnames <- colnames(x[['samples']][[1]])
  samples <- as.matrix(x[['samples']][, grep("^d\\.", varnames)])
  stats <- t(apply(samples, 2, quantile, probs=c(0.025, 0.5, 0.975)))
  model <- x[['model']]
  network <- model[['network']]
  comps <- extract.comparisons(varnames)
  groups <- comps[,1]
  group.names <- unique(groups)
  group.labels <- paste("Compared with", if (use.description) treatment.id.to.description(network, group.names) else group.names)
  names(group.labels) <- group.names
  params <- list(...)

  data <- data.frame(
    id=if (use.description) treatment.id.to.description(network, comps[,2]) else comps[,2],
    pe=stats[,2], ci.l=stats[,1], ci.u=stats[,3],
    group=groups, style="normal")

  blobbogram(data,
    columns=c(), column.labels=c(),
    id.label="",
    ci.label=paste(ll.call('scale.name', model), "(95% CrI)"),
    log.scale=ll.call('scale.log', model),
    grouped=TRUE, group.labels=group.labels,
    ...)
}

as.mcmc.list.mtc.result <- function(x, ...) {
  x[['samples']]
}

arm.index.matrix <- function(network) {
  studies <- mtc.studies.list(network)
  all.studies <- inverse.rle(studies)
  n <- max(studies[['lengths']])
  t(sapply(studies[['values']], function(study) {
    v <- which(all.studies == study)
    length(v) <- n
    v
  }))
}

mtc.model.data <- function(model) {
  data.ab <- model[['network']][['data.ab']]
  data.re <- model[['network']][['data.re']]

  columns.ab <- ll.call('required.columns.ab', model)
  if (!(is.null(data.ab) || all(columns.ab %in% colnames(data.ab)))) {
    stop(paste(
      'likelihood =', model[['likelihood']],
      'link =', model[['link']], 'requires columns:', paste(columns.ab, collapse=', '), 'on data'))
  }
  columns.re <- c('m'='diff', 'e'='std.err')
  if (!(is.null(data.re) || all(columns.re %in% colnames(data.re)))) {
    stop(paste(
      'likelihood =', model[['likelihood']],
      'link =', model[['link']], 'requires columns:', paste(columns.ab, collapse=', '), 'on data.re'))
   }

  data <- data.frame(
    t=as.numeric(forcats::fct_c(
      if (is.null(data.ab)) factor() else data.ab[['treatment']],
      if (is.null(data.re)) factor() else data.re[['treatment']]))
  )
  nrow.ab <- if (!is.null(data.ab)) nrow(data.ab) else 0
  nrow.re <- if (!is.null(data.re)) nrow(data.re) else 0
  if (nrow.ab > 0) {
    for (column in names(columns.ab)) {
      data[[column]] <- NA
      coldata <- data.ab[[columns.ab[column]]]
      if (any(is.na(coldata))) {
        stop(paste('data.ab contains NAs in column "', columns.ab[column], '"', sep=""))
      }
      data[[column]][1:nrow.ab] <- data.ab[[columns.ab[column]]]
    }
  }
  if (nrow.re > 0) {
    for (column in names(columns.re)) {
      if (!(column %in% colnames(data))) {
        data[[column]] <- NA
      }
      data[[column]][(nrow.ab + 1):(nrow.ab + nrow.re)] <- data.re[[columns.re[column]]]
    }
    mtc.validate.data.re(data.re)
  }

  studies.ab <- rle(as.character(data.ab[['study']]))[['values']]
  studies.re <- rle(as.character(data.re[['study']]))[['values']]
  studies <- c(studies.ab, studies.re)
  study.arms <- c(as.character(data.ab[['study']]), as.character(data.re[['study']]))
  na <- sapply(studies, function(study) { sum(study.arms == study) })
  na.re <- if (length(studies.re) > 0 ) na[(length(studies.ab)+1):length(studies)] else c()
  s.mat <- arm.index.matrix(model[['network']])

  model.data <- lapply(data, function(column) { matrix(column[s.mat], nrow=nrow(s.mat)) })
  names(model.data) <- colnames(data)

  ns.a <- length(studies.ab)
  ns.r2 <- sum(na.re == 2)
  ns.rm <- sum(na.re > 2)
  model.data <- c(model.data, list(
    na = unname(na),
    nt = nrow(model[['network']][['treatments']]),
    om.scale = model[['om.scale']],
    re.prior.sd = model[['re.prior.sd']]))

  powerAdjust <- model[['powerAdjust']]
  if (!is.null(powerAdjust) && !is.na(powerAdjust)) {
    studyData <- model[['network']][['studies']]
    alpha <- studyData[[powerAdjust]]
    names(alpha) <- studyData[['study']]
    alpha <- alpha[as.character(studies)]
    model.data[['alpha']] <- alpha

    if (model[['likelihood']] != 'normal') {
      model.data[['zero']] <- matrix(0, ncol=max(na), nrow=length(studies.ab))
    }
  } else {
    alpha <- rep(1, length(studies))
  }

  studies.a <- if (ns.a > 0) 1:ns.a else numeric()
  studies.a <- studies.a[alpha[studies.a] > 0]
  studies.r2 <- if (ns.r2 > 0) ns.a + (1:ns.r2) else numeric()
  studies.r2 <- studies.r2[alpha[studies.r2] > 0]
  studies.rm <- if (ns.rm > 0) ns.a + ns.r2 + (1:ns.rm) else numeric()
  studies.rm <- studies.rm[alpha[studies.rm] > 0]
  # conditional assigns necessary until JAGS accepts zero-length vectors
  if (length(studies.a) > 0) {
    model.data[['studies.a']] <- studies.a
  }
  if (length(studies.r2) > 0) {
    model.data[['studies.r2']] <- studies.r2
  }
  if (length(studies.rm) > 0) {
    model.data[['studies.rm']] <- studies.rm
  }
  model.data[['studies']] <- c(studies.a, studies.r2, studies.rm)

  model.data
}

mtc.data.studyrow <- function(data,
    armVars=c('treatment'='t', 'responders'='r', 'sampleSize'='n'),
    nArmsVar='na', studyVars=c(),
    studyNames=1:nrow(data), treatmentNames=NA,
    patterns=c('%s..', '%s..%d.')) {

  studyNames <- as.character(studyNames)

  colsOrNA <- function(row, cols) {
    rval <- rep(NA, length(cols))
    sel <- cols %in% colnames(row)
    rval[sel] <- row[cols[sel]]
    rval
  }

  nArmsCol <- sprintf(patterns[1], nArmsVar)
  studyCols <- sprintf(patterns[1], studyVars)

  out <- do.call(rbind, lapply(1:nrow(data), function(i) {
    row <- data[i,]
    na <- row[nArmsCol]
    studyEntries <- row[studyCols]
    names(studyEntries) <- names(studyVars)
    do.call(rbind, lapply(1:unlist(na), function(j) {
      armCols <- sprintf(patterns[2], armVars, j)
      armEntries <- colsOrNA(row, armCols)
      names(armEntries) <- names(armVars)
      c(list('study'=i), studyEntries, armEntries)
    }))
  }))

  colNames <- colnames(out)
  out <- lapply(colnames(out), function(col) {
    unlist(out[,col])
  })
  names(out) <- colNames

  out[['study']] <- studyNames[out[['study']]]
  if (all(!is.na(treatmentNames))) {
    out[['treatment']] <- treatmentNames[out[['treatment']]]
  } else {
    out[['treatment']] <- as.character(out[['treatment']])
  }
  out[['stringsAsFactors']] <- FALSE
  do.call(data.frame, out)
}

context("mtc.data.studyrow")

studyrow_file <- function(name) {
  paste0("../data/studyrow/", name)
}

test_that("TSD2 example 1 (dichotomous data, binomial/logit)", {
  treatments <- c('Control', 'BetaB')
  data <- mtc.data.studyrow(read.table(studyrow_file('tsd2-1.data.txt'), header=TRUE), treatmentNames=treatments)
  expect_that(data, equals(dget(studyrow_file('tsd2-1.out.txt'))))
})

test_that("TSD2 example 2 (count data, poisson/log)", {
  treatments <- c('control', 'diet')
  studies <- c('DART', 'London Corn/Olive', 'London Low Fat', 'Minnesota Coronary', 'MRC Soya', 'Oslo Diet-Heart', 'STARS', 'Sydney Diet-Heart', 'Veterans Administration', 'Veterans Diet & Skin CA')
  data <- mtc.data.studyrow(
    read.table(studyrow_file('tsd2-2.data.txt'), header=TRUE),
    armVars=c('treatment'='t', 'responders'='r', 'exposure'='E'),
    treatmentNames=treatments, studyNames=studies)

  # Needs check.attributes=FALSE because locales may sort upper/lower case differently
  # This leads to differences in levels(data$studies)
  expect_that(data, equals(dget(studyrow_file('tsd2-2.out.txt')), check.attributes=FALSE))
})

test_that("TSD2 example 3 (count data, binomial/cloglog)", {
  studies <- c('MRC-E', 'EWPH', 'SHEP', 'HAPPHY', 'ALLHAT', 'INSIGHT', 'ANBP-2', 'ALPINE', 'FEVER', 'DREAM', 'HOPE', 'PEACE', 'CHARM', 'SCOPE', 'AASK', 'STOP-2', 'ASCOT', 'NORDIL', 'INVEST', 'CAPPP', 'LIFE', 'VALUE')
  treatments <- c('Diuretic', 'Placebo', 'BetaB', 'CCB', 'ACEi', 'ARB')
  data <- mtc.data.studyrow(
    read.table(studyrow_file('tsd2-3.data.txt'), header=TRUE),
    studyVars=c('time'='time'),
    studyNames=studies, treatmentNames=treatments)
  expect_that(data, equals(dget(studyrow_file('tsd2-3.out.txt'))))
})

## TSD2 example 4 (competing risks, multinomial/log) not supported

test_that("TSD2 example 5 (continuous data, normal/identity)", {
  data <- mtc.data.studyrow(
    read.table(studyrow_file('tsd2-5.data.txt'), header=TRUE),
    armVars=c('treatment'='t', 'mean'='y', 'std.err'='se'))
  expect_that(data, equals(dget(studyrow_file('tsd2-5.out.txt'))))
})

## TSD2 example 6 (categorical data, binomial/probit) not supported

test_that("TSD2 example 7 (relative effect data)", {
  data <- mtc.data.studyrow(
    read.table(studyrow_file('tsd2-7.data.txt'), header=TRUE),
    armVars=c('treatment'='t', 'diff'='y', 'std.err'='se'),
    studyVars=c('var'='V'))
  expect_that(data, equals(dget(studyrow_file('tsd2-7.out.txt'))))
})

test_that("TSD2 example 8 (mixed data)", {
  data1 <- mtc.data.studyrow(
    read.table(studyrow_file('tsd2-8.data1.txt'), header=TRUE),
    armVars=c('treatment'='t.a', 'mean'='y.a', 'std.err'='se.a'),
    nArmsVar='na.a')
  expect_that(data1, equals(dget(studyrow_file('tsd2-8.out1.txt'))))

  data2 <- mtc.data.studyrow(
    read.table(studyrow_file('tsd2-8.data2.txt'), header=TRUE),
    armVars=c('treatment'='t', 'diff'='y', 'std.err'='se'),
    nArmsVar='na',
    studyNames=4:7)
  expect_that(data2, equals(dget(studyrow_file('tsd2-8.out2.txt'))))
})

plotCovariateEffect <- function(result, t1, t2, xlim=NULL, ylim=NULL, ask=dev.interactive(orNone=TRUE)) {
  regressor <- result[['model']][['regressor']]
  if (is.null(xlim)) {
    if (regressor[['type']] == 'continuous') {
      studies <- result[['model']][['network']][['studies']]
      observed <- studies[, regressor[['variable']]]
      ctr <- regressor[['center']]
      xlim <- c(min(observed, na.rm=TRUE), max(observed, na.rm=TRUE))
      xvals <- seq(xlim[1], xlim[2], length.out=7)
    } else {
      xlim <- c(-0.5, 1.5)
      xvals <- c(0, 1)
    }
  } else {
    xvals <- seq(xlim[1], xlim[2], length.out=7)
  }

  pairs <- treatment.pairs(t1, t2, result[['model']][['network']][['treatments']][['id']])
  res <- lapply(xvals, function(xval) {
    re <- relative.effect(result, t1, t2, preserve.extra=FALSE, covariate=xval)
    samples <- as.matrix(re[['samples']])
    stats <- t(apply(samples, 2, quantile, probs=c(0.025, 0.5, 0.975)))
    comps <- extract.comparisons(rownames(stats))
    data.frame(t1=comps[,1], t2=comps[,2], median=stats[,"50%"], lower=stats[,"2.5%"], upper=stats[,"97.5%"], stringsAsFactors=FALSE)
  })

  if (is.null(ylim)) {
    ylim <- c(min(sapply(res, function(stats) { min(stats[['lower']]) })),
              max(sapply(res, function(stats) { max(stats[['upper']]) })))
  }

  first <- TRUE
  devAskNewPage(FALSE)
  for (pair in split(pairs, seq(nrow(pairs)))) {
    pair <- as.treatment.factor(pair, result[['model']][['network']])
    yvals <- sapply(res, function(stats) { stats[stats[['t1']] == pair[1] & stats[['t2']] == pair[2], c('median', 'lower', 'upper')] })
    if (regressor[['type']] == 'continuous') {
      plot(xvals, yvals['median', ], type='l', xlim=xlim, ylim=ylim, main="Treatment effect vs. covariate", xlab=regressor[["variable"]], ylab=paste("d", pair[1], pair[2], sep="."))
      lines(xvals, yvals['lower', ], lty=2)
      lines(xvals, yvals['upper', ], lty=2)
    } else {
      plot(xvals, yvals['median', ], type='p', xlim=xlim, ylim=ylim, main="Treatment effect vs. covariate", xlab=regressor[["variable"]], ylab=paste("d", pair[1], pair[2], sep="."), xaxp=c(0, 1, 1))
      segments(xvals, unlist(yvals['lower',]), xvals, unlist(yvals['upper',]))
      eps <- 0.01
      segments(xvals-eps, unlist(yvals['lower',]), xvals+eps, unlist(yvals['lower',]))
      segments(xvals-eps, unlist(yvals['upper',]), xvals+eps, unlist(yvals['upper',]))
    }
    if (first) devAskNewPage(ask)
    first <- FALSE
  }
}

#' @include ll-helper.counts.R
#' @include likelihoods.R

# Arm-level effect estimate (given a one-row data frame)
# Returns mean, standard deviation.
mtc.arm.mle.binom.cloglog <- function(data, k=0.5) {
  s <- data['responders'] + k
  n <- data['sampleSize'] + 2*k
  mu <- unname(log(-log(1 - s/n)))
  sigma <- unname(sqrt(1/n^2)/exp(mu))
  c('mean'=mu, 'sd'=min(1, sigma))
}

# Relative effect estimate (given a two-row data frame)
mtc.rel.mle.binom.cloglog <- function(data, correction.force=TRUE, correction.type="constant", correction.magnitude=1) {
  correction <- correction.counts(data, correction.force, correction.type, correction.magnitude)

  e1 <- mtc.arm.mle.binom.cloglog(data[1,], correction[1])
  e2 <- mtc.arm.mle.binom.cloglog(data[2,], correction[2])
  c(e2['mean'] - e1['mean'], sqrt(e1['sd']^2 + e2['sd']^2))
}

mtc.code.likelihood.binom.cloglog <- function(powerAdjust) {
  paste("cloglog(p[i, k]) <- $armLinearModel$", likelihood.code.binom[powerAdjust + 1], sep="\n")
}

fitted.values.parameter.binom.cloglog <- fitted.values.parameter.binom
deviance_fn.binom.cloglog <- deviance_fn.binom

scale.log.binom.cloglog <- function() { TRUE }
scale.name.binom.cloglog <- function() { "Hazard Ratio" }

# Initial values outside this range result in probability 0 or 1 for the
# binomial, which may lead to BUGS/JAGS rejecting the data
inits.info.binom.cloglog <- function() {
  list(
    limits=c(-37.4, 3.6),
    param='p.base',
    transform=function(theta) { 1 - exp(-exp(theta)) })
}

required.columns.ab.binom.cloglog <- required.columns.counts
validate.data.binom.cloglog <- validate.data.counts
study.baseline.priors.binom.cloglog <- function() {
"for (i in studies.a) {
  mu[i] <- cloglog(p.base[i])
  p.base[i] ~ dunif(0, 1)
}
"
}

#' @include likelihoods.R

# Arm-level effect estimate (given a one-row data frame)
# Returns mean, standard deviation.
mtc.arm.mle.poisson.log <- function(data, k=0.5) {
  r <- data['responders'] + k
  E <- data['exposure']
  mu <- as.numeric(log(r/E))
  sigma <- as.numeric(sqrt(1/E))
  c('mean'=mu, 'sd'=sigma)
}

# Relative effect estimate (given a two-row data frame)
mtc.rel.mle.poisson.log <- function(data, correction.force=TRUE, correction.type="constant", correction.magnitude=1) {
  correction.need <- data[1,"responders"] == 0 || data[2,"responders"] == 0

  groupRatio <- if (correction.type == "reciprocal") {
    data[1,'exposure'] / data[2,'exposure']
  } else {
    1
  }

  correction <- if (correction.force || correction.need) {
    correction.magnitude * c(groupRatio/(groupRatio+1), 1/(groupRatio+1))
  } else {
    c(0, 0)
  }

  e1 <- mtc.arm.mle.poisson.log(data[1,], correction[1])
  e2 <- mtc.arm.mle.poisson.log(data[2,], correction[2])
  c(e2['mean'] - e1['mean'], sqrt(e1['sd']^2 + e2['sd']^2))
}

mtc.code.likelihood.poisson.log <- function(powerAdjust) {
  paste("log(lambda[i, k]) <- $armLinearModel$",
        "theta[i, k] <- E[i, k] * lambda[i, k]",
        likelihood.code.poisson[powerAdjust + 1], sep="\n")
}

fitted.values.parameter.poisson.log <- fitted.values.parameter.poisson
deviance_fn.poisson.log <- deviance_fn.poisson

scale.log.poisson.log <- function() { TRUE }
scale.name.poisson.log <- function() { "Hazard Ratio" }

# Initial values outside this range result in rate 0 for the
# poisson, which may lead to BUGS/JAGS rejecting the data
scale.limit.inits.poisson.log <- function() { c(-745, +Inf) }
inits.info.poisson.log <- function() {
  list(
    limits=c(-745, +Inf),
    param='lambda.base',
    transform=exp)
}

required.columns.ab.poisson.log <- function() {
  c('r'='responders', 'E'='exposure')
}

validate.data.poisson.log <- function(data.ab) {
  stopifnot(all(data.ab[['responders']] >= 0))
  stopifnot(all(data.ab[['exposure']] >= 0))
}

study.baseline.priors.poisson.log <- function() {
"for (i in studies.a) {
  mu[i] <- log(lambda.base[i])
  lambda.base[i] ~ dgamma(0.001, 0.001)
}
"
}

context("ll.poisson.log")

test_that("mtc.arm.mle (default correction)", {
  expect_equal(mtc.arm.mle.poisson.log(c('responders'=0, 'exposure'=25)),
              c('mean'=log(0.5/25), 'sd'=sqrt(1/25)))

  expect_equal(mtc.arm.mle.poisson.log(c('responders'=12, 'exposure'=25)),
              c('mean'=log(12.5/25), 'sd'=sqrt(1/25)))

  expect_equal(mtc.arm.mle.poisson.log(c('responders'=25, 'exposure'=25)),
              c('mean'=log(25.5/25), 'sd'=sqrt(1/25)))
})

test_that("mtc.arm.mle (no correction)", {
  expect_equal(mtc.arm.mle.poisson.log(c('responders'=12, 'exposure'=25), k=0),
               c('mean'=log(12/25), 'sd'=sqrt(1/25)))
})

test_that("mtc.arm.mle (other correction)", {
  expect_equal(mtc.arm.mle.poisson.log(c('responders'=0, 'exposure'=25), k=0.05),
               c('mean'=log(0.05/25), 'sd'=sqrt(1/25)))
})

test_that("mtc.rel.mle (forced default correction)", {
  expect_that(mtc.rel.mle.poisson.log(rbind(c('responders'=0, 'exposure'=25), c('responders'=3, 'exposure'=25))),
              equals(c('mean'=log(7), 'sd'=sqrt(2/25))))

  expect_that(mtc.rel.mle.poisson.log(rbind(c('responders'=0, 'exposure'=25), c('responders'=0, 'exposure'=25))),
              equals(c('mean'=0, 'sd'=sqrt(2/25))))
})

test_that("mtc.rel.mle (as-needed default correction)", {
  expect_equal(mtc.rel.mle.poisson.log(rbind(c('responders'=1, 'exposure'=25), c('responders'=3, 'exposure'=24)), correction.force=FALSE),
               c('mean'=log((3/24)/(1/25)), 'sd'=sqrt(1/24 + 1/25)))

  expect_equal(mtc.rel.mle.poisson.log(rbind(c('responders'=0, 'exposure'=25), c('responders'=3, 'exposure'=24)), correction.force=FALSE),
               c('mean'=log((3.5/24)/(0.5/25)), 'sd'=sqrt(1/24 + 1/25)))

  expect_equal(mtc.rel.mle.poisson.log(rbind(c('responders'=1, 'exposure'=25), c('responders'=24, 'exposure'=24)), correction.force=FALSE),
               c('mean'=log((24/24)/(1/25)), 'sd'=sqrt(1/24 + 1/25)))
})

test_that("mtc.rel.mle (alternative magnitude correction)", {
  expect_equal(mtc.rel.mle.poisson.log(rbind(c('responders'=0, 'exposure'=25), c('responders'=3, 'exposure'=25)), correction.magnitude=0.1),
               c('mean'=log((3.05/25)/(0.05/25)), 'sd'=sqrt(2/25)))
})

test_that("mtc.rel.mle (reciprocal correction)", {
  # no correction
  expect_equal(mtc.rel.mle.poisson.log(rbind(c('responders'=1, 'exposure'=25), c('responders'=3, 'exposure'=24)), correction.type="reciprocal", correction.force=FALSE),
               c('mean'=log((3/24)/(1/25)), 'sd'=sqrt(1/24 + 1/25)))

  # 1:2 group ratio (R = 2), correction for the control is R/(R+1) = 2/3, for the treatment 1/(R+1) = 1/3
  expect_equal(mtc.rel.mle.poisson.log(rbind(c('responders'=0, 'exposure'=50), c('responders'=3, 'exposure'=25)), correction.type="reciprocal"),
               c('mean'=log(((3+1/3)/25)/((0+2/3)/50)), 'sd'=sqrt(1/25 + 1/50)))

  # 1:4 group ratio (R = 4), correction for the control is R/(R+1) = 4/5, for the treatment 1/(R+1) = 1/5
  expect_equal(mtc.rel.mle.poisson.log(rbind(c('responders'=0, 'exposure'=100), c('responders'=3, 'exposure'=25)), correction.type="reciprocal"),
               c('mean'=log((3.2/25)/(0.8/100)), 'sd'=sqrt(1/25 + 1/100)))

  # 1:4 group ratio (R = 4), correction for the control is 0.1 R/(R+1) = 0.4/5, for the treatment 0.1/(R+1) = 0.1/5
  expect_equal(mtc.rel.mle.poisson.log(rbind(c('responders'=0, 'exposure'=100), c('responders'=3, 'exposure'=25)), correction.type="reciprocal", correction.magnitude=0.1),
               c('mean'=log((3.02/25)/(0.08/100)), 'sd'=sqrt(1/25 + 1/100)))

  # 1:4 group ratio (R = 4), correction for the control is 0.1 R/(R+1) = 0.4/5, for the treatment 0.1/(R+1) = 0.1/5
  expect_equal(mtc.rel.mle.poisson.log(rbind(c('responders'=0, 'exposure'=100), c('responders'=0, 'exposure'=25)), correction.type="reciprocal", correction.magnitude=0.1),
               c('mean'=0, 'sd'=sqrt(1/25 + 1/100)))
})

#' @include solveLP.R
#' @include arrayize.R
#' @include rtruncnorm.R

#'Generate a list of arms with a likelihood contribution
#'@param baseline Include study baseline arms
likelihood.arm.list <- function(network, baseline=TRUE, includedStudies=NULL) {
  data <- mtc.merge.data(network)
  all.studies <- as.character(data[['study']])
  studies <- rle(all.studies)[['values']]
  studyIndices <- if (!is.null(includedStudies)) {
    which(includedStudies[studies])
  } else {
    1:length(studies)
  }
  arms <- do.call(rbind, lapply(studyIndices, function(i) {
    study <- studies[i]
    sel <- which(all.studies == study)
    ts <- as.character(data[['treatment']][sel])
    t1 <- rep(ts[1], length(sel) - 1)
    t2 <- ts[-1]
    idx <- 2:length(ts)
    n.ab <- if (is.null(network[['data.ab']])) 0 else nrow(network[['data.ab']])
    if (sel[1] <= n.ab && baseline) {
      t1 <- c(NA, t1)
      t2 <- c(NA, t2)
      idx <- c(1, idx)
    }
    data.frame(study=study, studyIndex=i, armIndex=idx, t1=t1, t2=t2, stringsAsFactors=FALSE)
  }))
  rownames(arms) <- NULL
  arms
}

powerAdjustIncludedStudies <- function(model) {
  data <- mtc.merge.data(model[['network']])
  all.studies <- as.character(data[['study']])
  studies <- rle(all.studies)[['values']]
  if (!is.null(model[['powerAdjust']]) && !is.na(model[['powerAdjust']])) {
    model[['data']][['alpha']] > 0
  } else {
    x <- c()
    x[studies] <- TRUE
    x
  }
}

# MLE estimates for the study baselines
mtc.init.mle.baseline <- function(model) {
  s.mat <- arm.index.matrix(model[['network']])
  data.ab <- model[['network']][['data.ab']]
  studies.ab <- rle(as.character(data.ab[['study']]))[['values']]
  studyIndices <- which(powerAdjustIncludedStudies(model)[studies.ab])
  rval <- if (length(studies.ab) > 0) {
    do.call(rbind, lapply(studyIndices, function(i) {
      study <- studies.ab[i]
      data.ab <- model[['network']][['data.ab']]
      treatment <- data.ab[['treatment']][s.mat[study, 1, drop=TRUE]]
      data <- data.ab[data.ab[['study']] == study & data.ab[['treatment']] == treatment, , drop=TRUE]
      data <- unlist(data[ll.call("required.columns.ab", model)])
      mle <- ll.call("mtc.arm.mle", model, data)
      data.frame(parameter=paste0("mu[", i, "]"), type="baseline", mean=mle['mean'], 'std.err'=mle['sd'], stringsAsFactors=FALSE)
    }))
  } else {
    data.frame(parameter=character(), type=character(), mean=numeric(), std.err=numeric())
  }
  rownames(rval) <- NULL
  rval
}

# MLE estimates for the study-level relative effects
mtc.init.mle.relative <- function(model) {
  data.ab <- model[['network']][['data.ab']]
  data.re <- model[['network']][['data.re']]

  columns <- ll.call("required.columns.ab", model)
  processArm <- function(study, studyIndex, armIndex, t1, t2) {
    if (!is.null(data.ab) && study %in% data.ab[['study']]) {
      data <- data.ab[data.ab[['study']] == study & (data.ab[['treatment']] == t1 | data.ab[['treatment']] == t2), columns, drop=FALSE]
      mle <- ll.call("mtc.rel.mle", model, as.matrix(data))
    } else { # data.re -- assumes baseline is unaltered
      data <- data.re[data.re[['study']] == study & data.re[['treatment']] == t2, , drop=TRUE]
      mle <- c('mean'=data[['diff']], 'sd'=data[['std.err']])
    }
    data.frame(parameter=paste0("delta[", studyIndex, ",", armIndex, "]"), type="relative", mean=mle['mean'], 'std.err'=mle['sd'], stringsAsFactors=FALSE)
  }

  arms <- likelihood.arm.list(model[['network']], baseline=FALSE, includedStudies=powerAdjustIncludedStudies(model))
  rval <- do.call(rbind, mapply(processArm, arms$study, arms$studyIndex, arms$armIndex, arms$t1, arms$t2, SIMPLIFY=FALSE))
  rownames(rval) <- NULL
  rval
}

mle.pooled.effect <- function(model, t1, t2, om.scale) {
  t1 <- as.treatment.factor(t1, model[['network']])
  t2 <- as.treatment.factor(t2, model[['network']])
  pair <- data.frame(t1=t1, t2=t2)

  calc <- function(data, fun) {
    sel1 <- data[['treatment']] == t1
    sel2 <- data[['treatment']] == t2
    studies <- intersect(unique(data[['study']][sel1]), unique(data[['study']][sel2]))

    study.mle <- sapply(studies, function(study) {
      fun(data[data[['study']] == study, , drop=FALSE])
    })

    if (!is.matrix(study.mle)) {
      study.mle <- matrix(study.mle, nrow=2)
    }
    rownames(study.mle) <- c('mean', 'sd')

    study.mle
  }

  study.mle <- NULL
  data.ab <- model[['network']][['data.ab']]
  if (!is.null(data.ab)) {
    study.mle <- calc(data.ab, function(data) {
      rel.mle.ab(data, model, pair)
    })
  }
  data.re <- model[['network']][['data.re']]
  if (!is.null(data.re)) {
    study.mle <- cbind(study.mle, calc(data.re, function(data) {
      rel.mle.re(data, pair)
    }))
  }
  meta <- 
    if (ncol(study.mle) != 0) {
      meta::metagen(unlist(study.mle['mean', ]), unlist(study.mle['sd', ]), method.tau="DL")
    } else {
      list('TE.random'=0, seTE.random=om.scale)
    }
  data.frame(parameter=paste('d', t1, t2, sep='.'), type='basic', mean=meta[['TE.random']], std.err=meta[['seTE.random']], stringsAsFactors=FALSE)
}

# MLE estimates for the basic parameters
# based on inverse-variance random effects meta-analysis (package meta)
mtc.init.mle.basic <- function(model) {
  graph <- if(!is.null(model[['tree']])) model[['tree']] else model[['graph']]
  if (!is.null(graph)) {
    params <- mtc.basic.parameters(model)
    rval <- do.call(rbind, lapply(E(graph), function(e) {
      v <- ends(graph, e, names=FALSE)
      mle.pooled.effect(model, V(graph)[v[1]]$name, V(graph)[v[2]]$name, model[['om.scale']])
    }))
    rownames(rval) <- NULL
    rval
  } else {
    NULL
  }
}

mtc.init.mle.regression <- function(model) {
  nc <- length(model[['regressor']][['classes']])
  params <- regressionParams(model[['regressor']], model[['data']][['nt']], nc)
  data.frame(parameter=params, type='coefficient',
             mean=0.0, std.err=model[['om.scale']],
             stringsAsFactors=FALSE)
}

# for regression, last element of array can be missing
# this leads to a JAGS error about dimension mismatch
mtc.init.fixRegressionDimensions <- function(model, x) {
  regressor <- model[['regressor']]
  if (!is.null(regressor[['control']]) && regressor[['coefficient']] %in% c('unrelated', 'exchangeable')) {
    length(x[['beta']]) <- nrow(model[['network']][['treatments']])
  }
  x
}

# Matrix representing the linear model level of the BHM
mtc.linearModel.matrix <- function(model, parameters, includedStudies=NULL) {
  basic <- mtc.basic.parameters(model)
  if (model[['type']] == 'regression') {
    regr <- regressionParams(model[['regressor']], model[['data']][['nt']], length(model[['regressor']][['classes']]))
  }
  processArm <- function(study, studyIndex, armIndex, t1, t2) {
    x <- rep(0, length(parameters))
    names(x) <- parameters
    x[parameters == paste0("mu[", studyIndex, "]")] <- 1
    if (!is.na(t1) && !is.na(t2)) {
      if (model[['linearModel']] == 'random') {
        x[parameters == paste0("delta[", studyIndex, ",", armIndex, "]")] <- 1
      } else {
        x[basic] <- as.vector(mtc.model.call('func.param.matrix', model, t1=t1, t2=t2))
      }
      if (model[['type']] == 'regression') {
        t1 <- as.treatment.factor(t1, model[['network']])
        t2 <- as.treatment.factor(t2, model[['network']])
        m <- regressionAdjustMatrix(t1, t2, model[['regressor']], model[['data']][['nt']])
        x[regr] <- m * model[['data']][['x']][studyIndex]
      }
    }
    x
  }

  # loop over arms that have a likelihood contribution, generate a row for each
  arms <- likelihood.arm.list(model[['network']], baseline=TRUE, includedStudies=includedStudies)
  rval <- t(mapply(processArm, arms$study, arms$studyIndex, arms$armIndex, arms$t1, arms$t2, USE.NAMES=FALSE))
  dimnames(rval) <- NULL
  rval
}

# Initial values for heterogeneity from prior
mtc.init.hy <- function(hy.prior, om.scale, n.chain) {
  fn <- hy.prior[['distr']]
  substr(fn, 1, 1) <- "r"
  args <- c(n.chain, hy.prior[['args']])
  args[args == 'om.scale'] <- om.scale
  if (grepl("norm$", fn)) { # for *norm, convert precision (JAGS) to sd (R)
    args[[3]] = sqrt(1/args[[3]])
  }
  values <- if (hy.prior[['distr']] == "dhnorm") { # special case dhnorm
    abs(rnorm(args[[1]], mean = args[[2]], sd = args[[3]]))
  } else {
    values <- do.call(fn, args)
  }
  if (hy.prior$type == "prec") {
    pmax(values, 1E-232) # prevent underflow in JAGS/BUGS (precision 0 is variance \infty)
  } else {
    values
  }
}

# Generate initial values for all relevant parameters
mtc.init <- function(model) {
  hy <- if (model[['linearModel']] == 'random') {
    mtc.init.hy(model[['hy.prior']], model[['om.scale']], model[['n.chain']])
  } else {
    c()
  }

  # Approximate MLE estimates of model parameters
  mle <- mtc.init.mle.baseline(model)
  if (model[['linearModel']] == 'random') {
    mle <- rbind(mle, mtc.init.mle.relative(model))
  }
  mle <- rbind(mle, mtc.init.mle.basic(model))

  if (model[['type']] == 'regression') {
    mle <- rbind(mle, mtc.init.mle.regression(model))
  }

  # Define parameter value constraints
  params <- mle[['parameter']]
  linearModel <- mtc.linearModel.matrix(model, params)
  limits <- ll.call("inits.info", model)[['limits']]

  constr.l <- list( # Ax >= L (-Ax <= -L)
    mat=-linearModel,
    rhs=rep(-limits[1], nrow(linearModel)),
    eq=rep(0, nrow(linearModel)))
  constr.u <- list( # Ax <= U
    mat=linearModel,
    rhs=rep(limits[2], nrow(linearModel)),
    eq=rep(0, nrow(linearModel)))

  constr <- if (all(is.finite(limits))) {
    list(mat=rbind(constr.l[['mat']], constr.u[['mat']]),
         rhs=c(constr.l[['rhs']], constr.u[['rhs']]),
         eq=c(constr.l[['eq']], constr.u[['eq']]))
  } else if (is.finite(limits[1])) {
    constr.l
  } else if (is.finite(limits[2])) {
    constr.u
  }

  # Generate a random permutation of the parameters
  randomParameterOrder <- function() {
    c(sample(params[mle[['type']] == 'coefficient']),
      sample(params[mle[['type']] == 'basic']),
      sample(params[mle[['type']] == 'relative']),
      sample(params[mle[['type']] == 'baseline']))
  }

  # Find min/max value for the given parameter under the constraints
  findLimit <- function(mat, rhs, eq, idx, max) {
    if (is.null(mat)) {
      if (max) { +Inf } else { -Inf }
    } else {
      obj <- rep(0, ncol(mat))
      obj[idx] <- 1
      solveLP(obj, mat, rhs, eq, max=max)
    }
  }

  # Generate initial values for each chain in turn
  inits <- lapply(1:model[['n.chain']], function(chain) {
    param.order <- randomParameterOrder()
    stopifnot(length(param.order) == length(params)) # sanity check

    # sample initial values for the linear model parameters
    x <- rep(NA, length(params))
    names(x) <- params
    for (param in param.order) {
      param.mle <- mle[params == param, ]
      param.limits <- c(findLimit(constr[['mat']], constr[['rhs']], constr[['eq']], params == param, FALSE),
                        findLimit(constr[['mat']], constr[['rhs']], constr[['eq']], params == param, TRUE))

      x[param] <- rtruncnorm(mean=param.mle[['mean']],
                             sd=model[['var.scale']] * param.mle[['std.err']],
                             a=param.limits[1], b=param.limits[2])

      where <- rep(0, length(params))
      where[params == param] <- 1
      if (!is.null(constr[['mat']]) && param.mle[['type']] != 'baseline') {
        constr[['mat']] <- rbind(constr[['mat']], where)
        constr[['rhs']] <- c(constr[['rhs']], unname(x[param]))
        constr[['eq']] <- c(constr[['eq']], TRUE)
      }
    }

    # add initial values for the heterogeneity
    if (model[['linearModel']] == 'random') {
      type <- model[['hy.prior']][['type']]
      if (type == 'std.dev') {
        x['sd.d'] <- hy[chain]
      } else if (type == 'var') {
        x['var.d'] <- hy[chain]
      } else if (type == 'prec') {
        x['tau.d'] <- hy[chain]
      } else {
        stop("Invalid heterogeneity prior type")
      }
    }

    if (model[['type']] == 'regression' && model[['regressor']][['coefficient']] == 'exchangeable') {
      x['reg.sd'] <- runif(1, 0, model[['om.scale']])
    }

    # convert flat representation to structured one
    x <- arrayize(x)

    # for regression, make sure the dimensions are correct
    # (if control is the last parameter in the array)
    if (model[['type']] == 'regression') {
      x <- mtc.init.fixRegressionDimensions(model, x)
    }

    # replace mu to whatever the study baseline prior is on
    if (!is.null(x[['mu']])) {
      mu <- x[['mu']]
      x[['mu']] <- NULL
      info <- ll.call('inits.info', model)
      x[[info[['param']]]] <- info[['transform']](mu)
    }

    x
  })
  
  inits
}

# All non-NA initial values correspond to a variable that can be monitored
inits.to.monitors <- function(inits) {
  unlist(lapply(names(inits), function(var) {
    struct <- inits[[var]]
    if (is.matrix(struct)) {
      lapply(1:(nrow(struct)*ncol(struct)), function(idx) {
        i <- (idx - 1) %/% ncol(struct) + 1
        j <- (idx - 1) %% ncol(struct) + 1
        if (!is.na(struct[i, j, drop=TRUE])) paste(var, "[", i, ",", j, "]", sep="")
      })
    } else if (length(struct) > 1) {
      lapply(1:length(struct), function(i) {
        if (!is.na(struct[i])) paste(var, "[", i, "]", sep="")
      })
    } else if (var == "var.d" || var == "tau.d") {
      c(var, "sd.d")
    } else {
      var
    }
  }))
}

context("mtc.model.inits")

test_that("mtc.init.mle.regression just returns 0 +/- om.scale", {
  data.ab <- read.table(textConnection('
study treatment responders sampleSize
Brown control 0 52
Brown statinA 1 94
CCAIT control 2 166
CCAIT statinA 2 165
Downs control 77 3301
Downs statinB 80 3304
EXCEL control 3 1663
EXCEL statinB 33 6582'), header=TRUE, stringsAsFactors=FALSE)

  network <- mtc.network(data.ab=data.ab)
  model <- list(network=network,
                type='regression',
                likelihood='binom',
                link='logit',
                om.scale=2.5,
                n.chain=4,
                var.scale=2.5,
                linearModel='random',
                tree=minimum.diameter.spanning.tree(mtc.network.graph(network)),
                hy.prior=mtc.hy.prior("std.dev", "dunif", 0, "om.scale"),
                data=list(x=c(1,1,0,0),nt=3),
                regressor=list('variable'='x', 'coefficient'='shared', 'control'=as.treatment.factor('control', network)))

  expected <- read.table(textConnection('
parameter type        mean std.err
B         coefficient 0.0  2.5'), header=TRUE, stringsAsFactors=FALSE)
  expect_equal(mtc.init.mle.regression(model), expected, tolerance=1E-5)

  expected <- read.table(textConnection('
parameter type        mean std.err
beta[2]   coefficient 0.0  2.5
beta[3]   coefficient 0.0  2.5'), header=TRUE, stringsAsFactors=FALSE)
  model[['regressor']][['coefficient']] <- 'unrelated'
  expect_equal(mtc.init.mle.regression(model), expected, tolerance=1E-5)

  expected <- read.table(textConnection('
parameter type        mean std.err
beta[2]   coefficient 0.0  2.5
beta[3]   coefficient 0.0  2.5
B         coefficient 0.0  2.5'), header=TRUE, stringsAsFactors=FALSE)
  model[['regressor']][['coefficient']] <- 'exchangeable'
  expect_equal(mtc.init.mle.regression(model), expected, tolerance=1E-5)

  expected <- read.table(textConnection('
parameter type        mean std.err
B[2]      coefficient 0.0  2.5'), header=TRUE, stringsAsFactors=FALSE)
  model[['regressor']][['coefficient']] <- 'unrelated'
  model[['regressor']][['classes']] <- list('control'=as.treatment.factor('control', network), 'statin'=as.treatment.factor(c('statinA', 'statinB'), network))
  model[['regressor']][['control']] <- NULL
  expect_equal(mtc.init.mle.regression(model), expected, tolerance=1E-5)
})

test_that("mtc.linearModel.matrix works correctly", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
1     A         10.5 0.18
1     B         15.3 0.17
2     B         15.7 0.12
2     C         18.3 0.15
3     B         13.1 0.19
3     C         14.2 0.20'), header=T)
  data.re <- read.table(textConnection('
study treatment diff std.err
4     A         NA   0.15
4     B         3.1  0.22
4     C         4.2  0.24'), header=T)
  network <- mtc.network(data.ab=data.ab, data.re=data.re)
  model <- list(network=network,
                type='consistency',
                likelihood='normal',
                link='identity',
                om.scale=2.5,
                n.chain=4,
                var.scale=2.5,
                linearModel='random',
                tree=minimum.diameter.spanning.tree(mtc.network.graph(network)),
                hy.prior=mtc.hy.prior("std.dev", "dunif", 0, "om.scale"))

  params.re <- c("mu[1]", "mu[2]", "mu[3]", "delta[1,2]", "delta[2,2]", "delta[3,2]", "delta[4,2]", "delta[4,3]", "d.A.B", "d.A.C")
  expected.re <- rbind(c(1, 0, 0, 0, 0, 0, 0, 0, 0, 0),
                       c(1, 0, 0, 1, 0, 0, 0, 0, 0, 0),
                       c(0, 1, 0, 0, 0, 0, 0, 0, 0, 0),
                       c(0, 1, 0, 0, 1, 0, 0, 0, 0, 0),
                       c(0, 0, 1, 0, 0, 0, 0, 0, 0, 0),
                       c(0, 0, 1, 0, 0, 1, 0, 0, 0, 0),
                       c(0, 0, 0, 0, 0, 0, 1, 0, 0, 0),
                       c(0, 0, 0, 0, 0, 0, 0, 1, 0, 0))
  expect_equal(mtc.linearModel.matrix(model, params.re), expected.re)

  params.fe <- c("mu[1]", "mu[2]", "mu[3]", "d.A.B", "d.A.C")
  expected.fe <- rbind(c(1, 0, 0,  0, 0),
                       c(1, 0, 0,  1, 0),
                       c(0, 1, 0,  0, 0),
                       c(0, 1, 0, -1, 1),
                       c(0, 0, 1,  0, 0),
                       c(0, 0, 1, -1, 1),
                       c(0, 0, 0,  1, 0),
                       c(0, 0, 0,  0, 1))
  model[['linearModel']] <- 'fixed'
  expect_equal(mtc.linearModel.matrix(model, params.fe), expected.fe)

  studies <- c("1"=TRUE, "2"=TRUE, "3"=FALSE, "4"=FALSE)
  params.omit <- c("mu[1]", "mu[2]", "delta[1,2]", "delta[2,2]", "d.A.B", "d.A.C")
  expected.omit <- rbind(c(1, 0, 0, 0, 0, 0),
                         c(1, 0, 1, 0, 0, 0),
                         c(0, 1, 0, 0, 0, 0),
                         c(0, 1, 0, 1, 0, 0))
  model[['linearModel']] <- 'random'
  expect_equal(mtc.linearModel.matrix(model, params.omit, includedStudies=studies), expected.omit)
})

test_that("mtc.linearModel.matrix works correctly for regression", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
1     A         10.5 0.18
1     B         15.3 0.17
2     B         15.7 0.12
2     C         18.3 0.15
3     B         13.1 0.19
3     C         14.2 0.20'), header=T)
  data.re <- read.table(textConnection('
study treatment diff std.err
4     A         NA   0.15
4     B         3.1  0.22
4     C         4.2  0.24'), header=T)

  network <- mtc.network(data.ab=data.ab, data.re=data.re)
  model <- list(network=network,
                type='regression',
                likelihood='normal',
                link='identity',
                om.scale=2.5,
                n.chain=4,
                var.scale=2.5,
                linearModel='random',
                data=list(x=c(1,0,0.5,0.3), nt=3),
                tree=minimum.diameter.spanning.tree(mtc.network.graph(network)),
                regressor=list('variable'='x', 'coefficient'='shared', 'control'=as.treatment.factor('A', network)),
                hy.prior=mtc.hy.prior("std.dev", "dunif", 0, "om.scale"))

  params.re <- c("mu[1]", "mu[2]", "mu[3]", "delta[1,2]", "delta[2,2]", "delta[3,2]", "delta[4,2]", "delta[4,3]", "d.A.B", "d.A.C", "B")
  expected.re <- rbind(c(1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
                       c(1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1),
                       c(0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0),
                       c(0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0),
                       c(0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0),
                       c(0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0),
                       c(0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0.3),
                       c(0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0.3))
  expect_equal(mtc.linearModel.matrix(model, params.re), expected.re)

  params.fe <- c("mu[1]", "mu[2]", "mu[3]", "d.A.B", "d.A.C", "B")
  expected.fe <- rbind(c(1, 0, 0,  0, 0, 0),
                       c(1, 0, 0,  1, 0, 1),
                       c(0, 1, 0,  0, 0, 0),
                       c(0, 1, 0, -1, 1, 0),
                       c(0, 0, 1,  0, 0, 0),
                       c(0, 0, 1, -1, 1, 0),
                       c(0, 0, 0,  1, 0, 0.3),
                       c(0, 0, 0,  0, 1, 0.3))
  model[['linearModel']] <- 'fixed'
  expect_equal(mtc.linearModel.matrix(model, params.fe), expected.fe)
})

test_that("likelihood.arm.list returns the correct arms", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
s1    A         10.5 0.18
s1    B         15.3 0.17
s2    B         15.7 0.12
s2    C         18.3 0.15
s2    D         18.3 0.15
s3    B         13.1 0.19
s3    C         14.2 0.20'), header=T)
  data.re <- read.table(textConnection('
study treatment diff std.err
s4    A         NA   0.15
s4    B         3.1  0.22
s4    C         4.2  0.24'), header=T)
  network <- mtc.network(data.ab=data.ab, data.re=data.re)

  expected1 <- read.table(textConnection('
study studyIndex armIndex t1 t2
s1    1          2        A  B
s2    2          2        B  C
s2    2          3        B  D
s3    3          2        B  C
s4    4          2        A  B
s4    4          3        A  C'), header=TRUE, stringsAsFactors=FALSE)
  expect_equal(likelihood.arm.list(network, baseline=FALSE), expected1)

  expected2 <- read.table(textConnection('
study studyIndex armIndex t1 t2
s1    1          1        NA NA
s1    1          2        A  B
s2    2          1        NA NA
s2    2          2        B  C
s2    2          3        B  D
s3    3          1        NA NA
s3    3          2        B  C
s4    4          2        A  B
s4    4          3        A  C'), header=TRUE, stringsAsFactors=FALSE)
  expect_equal(likelihood.arm.list(network, baseline=TRUE), expected2)

  expected3 <- read.table(textConnection('
study studyIndex armIndex t1 t2
s1    1          1        NA NA
s1    1          2        A  B
s3    3          1        NA NA
s3    3          2        B  C
s4    4          2        A  B
s4    4          3        A  C'), header=TRUE, stringsAsFactors=FALSE)
  expect_equal(likelihood.arm.list(network, baseline=TRUE, includedStudies=c("s1"=TRUE, "s2"=FALSE, "s3"=TRUE, "s4"=TRUE)), expected3)
})

test_that("mtc.model.inits has correct shape", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
1     A         10.5 0.18
1     B         15.3 0.17
2     B         15.7 0.12
2     C         18.3 0.15
3     B         13.1 0.19
3     C         14.2 0.20'), header=T)
  data.re <- read.table(textConnection('
study treatment diff std.err
4     A         NA   0.15
4     B         3.1  0.22
4     C         4.2  0.24
5     A         2.9  0.28
5     B         NA   NA
6     A         0.2  0.23
6     B         0.8  0.27
6     C         NA   0.16
7     A         NA   0.18
7     B         4.3  0.21
8     B         NA   NA
8     C         4.0  0.19'), header=T)
  network <- mtc.network(data.ab, data.re=data.re)
  model <- list(network=network,
                type='consistency',
                likelihood='normal',
                link='identity',
                om.scale=2.5,
                n.chain=4,
                var.scale=2.5,
                linearModel='random',
                tree=minimum.diameter.spanning.tree(mtc.network.graph(network)),
                hy.prior=mtc.hy.prior("std.dev", "dunif", 0, "om.scale"))
  inits <- mtc.init(model)
  whereNA <- rbind(
    c(NA, 1, NA), # 1
    c(NA, 1, NA), # 2
    c(NA, 1, NA), # 3
    c(NA, 1, NA), # 5
    c(NA, 1, NA), # 7
    c(NA, 1, NA), # 8
    c(NA, 1, 1),  # 4
    c(NA, 1, 1))  # 6

  expect_equal(is.na(inits[[1]]$delta), is.na(whereNA))
  expect_equal(is.na(inits[[2]]$delta), is.na(whereNA))
  expect_equal(is.na(inits[[3]]$delta), is.na(whereNA))
  expect_equal(is.na(inits[[4]]$delta), is.na(whereNA))
  expect_equal(length(inits[[1]]$mu), 3)
  expect_equal(length(inits[[2]]$mu), 3)
  expect_equal(length(inits[[3]]$mu), 3)
  expect_equal(length(inits[[4]]$mu), 3)
})

test_that("mtc.model.inits - regression parameters have the correct shape", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
1     A         10.5 0.18
1     B         15.3 0.17
2     B         15.7 0.12
2     C         18.3 0.15
3     B         13.1 0.19
3     C         14.2 0.20'), header=T)
  studies <- read.table(textConnection('
study x 
1     0
2     1
3     1'), header=T)
  network <- mtc.network(data.ab=data.ab, studies=studies)
  ts <- as.treatment.factor(c('A','B','C'), network)
  model <- list(network=network,
                type='regression',
                likelihood='normal',
                link='identity',
                om.scale=2.5,
                n.chain=4,
                var.scale=2.5,
                linearModel='fixed',
                tree=minimum.diameter.spanning.tree(mtc.network.graph(network)),
                hy.prior=mtc.hy.prior("std.dev", "dunif", 0, "om.scale"),
                data=list(x=c(0,1,1),nt=3),
                regressor=list('variable'='x', 'coefficient'='unrelated', 'control'=ts[1]))

  inits <- mtc.init(model)
  expect_equal(is.na(inits[[1]]$beta), c(TRUE, FALSE, FALSE))

  model$regressor$control <- ts[2]
  inits <- mtc.init(model)
  expect_equal(is.na(inits[[1]]$beta), c(FALSE, TRUE, FALSE))

  model$regressor$control <- ts[3]
  inits <- mtc.init(model)
  expect_equal(is.na(inits[[1]]$beta), c(FALSE, FALSE, TRUE))

  model$regressor$coefficient <- 'shared'
  inits <- mtc.init(model)
  expect_equal(inits[[1]]$beta, NULL)
  expect_equal(is.na(inits[[1]]$B), FALSE)

  model$regressor$coefficient <- 'exchangeable'
  inits <- mtc.init(model)
  expect_equal(is.na(inits[[1]]$beta), c(FALSE, FALSE, TRUE))
  expect_equal(is.na(inits[[1]]$B), FALSE)
  expect_equal(is.na(inits[[1]]$reg.sd), FALSE)

  model$regressor$coefficient <- 'shared'
  model$regressor$control <- NULL
  model$regressor$classes <- list('control'=ts[1], 'other'=ts[2:3])
  inits <- mtc.init(model)
  expect_equal(is.na(inits[[1]]$B), c(TRUE, FALSE))
})

test_that("mtc.model.inits has correct heterogeneity parameter", {
  data.ab <- read.table(textConnection('
study treatment mean std.err
1     A         10.5 0.18
1     B         15.3 0.17
2     B         15.7 0.12
2     C         18.3 0.15
3     B         13.1 0.19
3     C         14.2 0.20'), header=T)
  network <- mtc.network(data.ab)
  model <- list(network=network,
                type='consistency',
                likelihood='normal',
                link='identity',
                om.scale=0.1,
                n.chain=4,
                var.scale=2.5,
                linearModel='random',
                tree=minimum.diameter.spanning.tree(mtc.network.graph(network)))

  # standard deviation prior
  model[['hy.prior']] <- mtc.hy.prior("std.dev", "dunif", 0, "om.scale")
  inits <- mtc.init(model)
  expect_true("sd.d" %in% names(inits[[1]]))
  expect_false("var.d" %in% names(inits[[1]]))
  expect_false("tau.d" %in% names(inits[[1]]))
  expect_true(inits[[1]][['sd.d']] <= 0.1)
  expect_true(inits[[1]][['sd.d']] >= 0.0)

  # variance prior
  model[['hy.prior']] <- mtc.hy.prior("var", "dlnorm", -4, 2)
  inits <- mtc.init(model)
  expect_false("sd.d" %in% names(inits[[1]]))
  expect_true("var.d" %in% names(inits[[1]]))
  expect_false("tau.d" %in% names(inits[[1]]))
  expect_true(inits[[1]][['var.d']] >= 0.0)

  # precision prior
  model[['hy.prior']] <- mtc.hy.prior("prec", "dgamma", 0.01, 0.01)
  inits <- mtc.init(model)
  expect_false("sd.d" %in% names(inits[[1]]))
  expect_false("var.d" %in% names(inits[[1]]))
  expect_true("tau.d" %in% names(inits[[1]]))
  expect_true(inits[[1]][['tau.d']] >= 0.0)
})

context("inits.to.monitors")

test_that("inits.to.monitors maps scalars", {
  expect_that(inits.to.monitors(list(sd.d=3)), equals(c("sd.d")))
})

test_that("inits.to.monitors maps vectors", {
  expect_that(inits.to.monitors(list(mu=c(1, -1, 3))), equals(c("mu[1]", "mu[2]", "mu[3]")))
})

test_that("inits.to.monitors removes NAs from vectors", {
  expect_that(inits.to.monitors(list(mu=c(1, NA, 3))), equals(c("mu[1]", "mu[3]")))
})

test_that("inits.to.monitors maps matrices", {
  expect_that(inits.to.monitors(list(delta=matrix(c(5, 3, -1, 2, 4, 5), ncol=3, byrow=TRUE))),
    equals(c("delta[1,1]", "delta[1,2]", "delta[1,3]", "delta[2,1]", "delta[2,2]", "delta[2,3]")))
})

test_that("inits.to.monitors removes NAs from matrices", {
  expect_that(inits.to.monitors(list(delta=matrix(c(NA, 3, NA, NA, 4, 5), ncol=3, byrow=TRUE))),
    equals(c("delta[1,2]", "delta[2,2]", "delta[2,3]")))
})

test_that("inits.to.monitors adds sd.d if var.d is present", {
  expect_that(inits.to.monitors(list(var.d=2)),
    equals(c("var.d", "sd.d")))
})

test_that("inits.to.monitors adds sd.d if tau.d is present", {
  expect_that(inits.to.monitors(list(tau.d=2)),
    equals(c("tau.d", "sd.d")))
})

#'@include mtc.hy.prior.R

## mtc.model class methods

mtc.model.call <- function(fn, model, ...) {
  fn <- paste(fn, model[['type']], sep='.')
  do.call(fn, c(list(model), list(...)))
}

mtc.model.defined <- function(model) {
  fns <- c('mtc.model', 'mtc.model.name', 'func.param.matrix')
  fns <- paste(fns, model[['type']], sep='.')
  all(sapply(fns, function(fn) { exists(fn, mode='function') }))
}

mtc.model <- function(network, type="consistency",
    factor=2.5, n.chain=4,
    likelihood=NULL, link=NULL,
    linearModel="random",
    om.scale=NULL,
    hy.prior=mtc.hy.prior("std.dev", "dunif", 0, "om.scale"),
    re.prior.sd=15*om.scale,
    dic=TRUE,
    powerAdjust=NA,
    ...) {
  if (!inherits(network, "mtc.network")) {
    stop('Given network is not an mtc.network')
  }
  if (!is.null(network[['studies']]) && !is.data.frame(network[['studies']])) {
    stop('studies must be a data frame') # older versions did not enforce this
  }

  if (!(linearModel %in% c("random", "fixed"))) {
    stop('linearModel must be either "random" or "fixed"')
  }

  network <- fix.network(network)

  if (check.duplicated.treatments(network, warn=TRUE)) {
    stop("Studies with duplicated treatments are not supported.")
  }

  if (!igraph::is_connected(mtc.network.graph(network))) {
    stop("Network is disconnected - see plot(network)")
  }

  add.std.err <- function(data) {
    if (!is.null(data[['std.dev']]) &&
        !is.null(data[['sampleSize']]) &&
        is.null(data[['std.err']])) {
      data[['std.err']] <- data[['std.dev']] / sqrt(data[['sampleSize']])
    }
    data
  }

  # calculate std.err for "legacy" data sets:
  if (!is.null(network[['data.ab']])) {
    network[['data.ab']] <- add.std.err(network[['data.ab']])
  }
  if (!is.null(network[['data.re']])) {
    network[['data.re']] <- add.std.err(network[['data.re']])
  }

  if (!is.na(powerAdjust)) {
    if (!(powerAdjust %in% names(network[['studies']]))) {
      stop("The value of powerAdjust must be a column in the studies data frame")
    }
    if (any(network[['studies']][[powerAdjust]] > 1) || any(network[['studies']][[powerAdjust]] < 0)) {
      stop("All power adjustment values must in [0, 1]")
    }
  }

  model <- list(
    type = type,
    linearModel = linearModel,
    network = network,
    n.chain = n.chain,
    var.scale = factor,
    dic = dic,
    powerAdjust = powerAdjust)

  if (!mtc.model.defined(model)) {
    stop(paste(type, 'is not an MTC model type.'))
  }

  model[['likelihood']] <- likelihood
  model[['link']] <- link
  if (!is.null(network[['data.ab']]) && all(required.columns.ab.binom.logit() %in% colnames(network[['data.ab']]))) {
    if (is.null(likelihood)) {
      model[['likelihood']] = 'binom'
    }
    if (is.null(link)) {
      model[['link']] = 'logit'
    }
  } else if (!is.null(network[['data.ab']]) && all(required.columns.ab.normal.identity() %in% colnames(network[['data.ab']]))) {
    if (is.null(likelihood)) {
      model[['likelihood']] = 'normal'
    }
    if (is.null(link)) {
      model[['link']] = 'identity'
    }
  } else if (is.null(network[['data.ab']])) {
    if (is.null(likelihood)) {
      warning('Likelihood can not be inferred. Defaulting to normal.')
      model[['likelihood']] = 'normal'
    }
    if (is.null(link)) {
      warning('Link can not be inferred. Defaulting to identity.')
      model[['link']] = 'identity'
    }
  } else {
    if (is.null(likelihood)) {
      stop('No appropriate likelihood could be inferred. Please specify one.')
    }
    if (is.null(link)) {
      stop('No appropriate link could be inferred. Please specify one.')
    }
  }
  if (!ll.defined(model)) {
    stop(paste('likelihood = ', model[['likelihood']],
      ', link = ', model[['link']], ' not found!', sep=''))
  }

  ll.call('validate.data', model, network[['data.ab']])

  model[['om.scale']] <- if (!is.null(om.scale)) om.scale else guess.scale(model)
  model[['hy.prior']] <- hy.prior
  model[['re.prior.sd']] <- if(length(re.prior.sd) == 1) re.prior.sd else 15 * model[['om.scale']]

  model <- mtc.model.call('mtc.model', model, ...)
  if (!grepl('om\\.scale', model[['code']])) {
    model[['data']][['om.scale']] <- NULL # prevent 'unused data' warning
  }
  model
}

print.mtc.model <- function(x, ...) {
  cat("MTC ", x[['type']], " model: ", x[['description']], "\n", sep="")
}

summary.mtc.model <- function(object, ...) {
  list("Description"=paste("MTC ", object[['type']], " model: ", object[['description']], sep=""),
     "Parameters"=mtc.basic.parameters(object))
}

plot.mtc.model <- function(x, layout=igraph::layout.circle, ...) {
  igraph::plot.igraph(mtc.model.graph(x), layout=layout, ...)
}

mtc.model.graph <- function(model) {
  comparisons <- mtc.comparisons(model[['network']])
  g <- if (!is.null(model[['tree']])) model[['tree']] else model[['graph']]
  g <- g + edges(as.vector(unlist(non.edges(g, comparisons))), arrow.mode=0, lty=1, color="grey")
  g
}

# filters list of comparison by edges that are not yet present in graph g
non.edges <- function(g, comparisons) {
  sapply(1:nrow(comparisons), function(i) {
    x <- c(comparisons[['t1']][i], comparisons[['t2']][i])
    if (are.connected(g, x[1], x[2]) || are.connected(g, x[2], x[1])) c() else x
  })
}

mtc.basic.parameters <- function(model) {
  graph <- if (!is.null(model[['tree']])) model[['tree']] else model[['graph']]
  if (!is.null(graph)) {
    sapply(E(graph), function(e) {
      v <- ends(graph, e, names=FALSE)
      paste("d", V(graph)[v[1]]$name, V(graph)[v[2]]$name, sep=".")
    })
  } else {
    NULL
  }
}

context("mtc.model.consistency")

test_that("func.param.matrix was implemented correctly", {
  model <- list(
    'type'='consistency',
    'tree'=igraph::make_graph(c('A','B','A','C','B','D')))

  expect_equal(matrix(0, nrow=3, dimnames=list(NULL, 'd.A.A')),
               mtc.model.call('func.param.matrix', model, t1='A', t2='A'))
  expect_equal(matrix(c(1,0,0), nrow=3, dimnames=list(NULL, 'd.A.B')),
               mtc.model.call('func.param.matrix', model, t1='A', t2='B'))
  expect_equal(matrix(c(1,0,0,0,1,0), nrow=3, dimnames=list(NULL, c('d.A.B', 'd.A.C'))),
               mtc.model.call('func.param.matrix', model, t1='A', t2=c('B', 'C')))
  expect_equal(matrix(c(-1,0,0), nrow=3, dimnames=list(NULL, c('d.B.A'))),
               mtc.model.call('func.param.matrix', model, t1='B', t2='A'))
  expect_equal(matrix(c(1,0,1), nrow=3, dimnames=list(NULL, c('d.A.D'))),
               mtc.model.call('func.param.matrix', model, t1='A', t2='D'))
})

test_that("mtc.model reports disconnected networks", {
  network <- parkinson
  network$data.ab <- network$data.ab[1:11,]
  expect_error(mtc.model(network), "Network is disconnected")
})

test_that("mtc.model sets re.prior.sd", {
  expect_equal(mtc.model(parkinson)$re.prior.sd, 15 * 2.3)
  expect_equal(mtc.model(parkinson, om.scale=3)$re.prior.sd, 15 * 3)
  expect_equal(mtc.model(parkinson, re.prior.sd=3)$re.prior.sd, 3)
})

context("mtc.model.nodesplit")

test_that("func.param.matrix was implemented correctly", {
  model <- list(
    'type'='nodesplit',
    'tree.indirect'=igraph::make_graph(c('D', 'A', 'D', 'B', 'D', 'C', 'D', 'E')),
    'split'=list(t1='A', t2='C'))

  expect_equal(matrix(c(0,0,0,0,1), nrow=5, dimnames=list(NULL, 'd.A.C')),
               mtc.model.call('func.param.matrix', model, t1='A', t2='C'))
  expect_equal(matrix(c(0,0,0,0,-1), nrow=5, dimnames=list(NULL, 'd.C.A')),
               mtc.model.call('func.param.matrix', model, t1='C', t2='A'))
  expect_equal(matrix(c(0,0,0,0,0), nrow=5, dimnames=list(NULL, 'd.A.A')),
               mtc.model.call('func.param.matrix', model, t1='A', t2='A'))
  expect_equal(matrix(c(-1,1,0,0,0), nrow=5, dimnames=list(NULL, 'd.A.B')),
               mtc.model.call('func.param.matrix', model, t1='A', t2='B'))
})

context("mtc.model.graph")

test_that("Vertices agree between mtc.comparisons and the model tree", {
  network <- thrombolytic
  model <- mtc.model(network)
  graph <- mtc.network.graph(network)
  expect_that(V(model$tree)$name, equals(V(graph)$name))
  expect_that(V(mtc.model.graph(model))$name, equals(V(graph)$name))
})

#' @include likelihoods.R

# Arm-level effect estimate (given a one-row data frame)
mtc.arm.mle.normal.smd <- function(data, k=0.5) {
  c('mean'=as.numeric(data['mean']), 'sd'=as.numeric(data['std.dev'])/sqrt(as.numeric(data['sampleSize'])))
}

# Relative effect estimate (given a two-row data frame)
mtc.rel.mle.normal.smd <- function(data, correction.force=TRUE, correction.type="constant", correction.magnitude=1) {
  e1 <- mtc.arm.mle.normal.smd(data[1,])
  e2 <- mtc.arm.mle.normal.smd(data[2,])
  c(e2['mean'] - e1['mean'], sqrt(e1['sd']^2 + e2['sd']^2))
}

likelihood.code.normal.smd <- list(
  read.template("gemtc.likelihood.normal.smd.txt"),
  read.template("gemtc.likelihood.normal.smd.power.txt"))

mtc.code.likelihood.normal.smd <- function(powerAdjust) {
  paste("theta[i, k] <- ($armLinearModel$) * sqrt(sum((n[i,1:na[i]]-1)*pow(s[i,1:na[i]], 2))/(sum(n[i,1:na[i]])-na[i]))/(1-(3/((4*(sum(n[i,1:na[i]])-na[i]))-1)))", likelihood.code.normal.smd[powerAdjust + 1], sep="\n")
}

fitted.values.parameter.normal.smd <- fitted.values.parameter.normal
deviance_fn.normal.smd <- deviance_fn.normal

scale.log.normal.smd <- function() { FALSE }
scale.name.normal.smd <- function() { "Standardized Mean Difference" }

inits.info.normal.smd <- function() {
  list(
    limits=c(-Inf, +Inf),
    param='mu',
    transform=identity)
}

required.columns.ab.normal.smd <- function() {
  c('m'='mean', 's'='std.dev', 'n'='sampleSize')
}

validate.data.normal.smd <- function(data.ab) {
  stopifnot(all(data.ab[['std.dev']] > 0))
  stopifnot(all(data.ab[['sampleSize']] > 0))
}

study.baseline.priors.normal.smd <- function() {
"for (i in studies.a) {
  mu[i] ~ dnorm(0, prior.prec)
}
"
}

context("[validate] NICE TSD2 program 7a")

test_that("The summaries match", {
  result <- replicate.example("parkinson-diff", parkinson_diff, likelihood="normal", link="identity")
  compare.summaries(result$s1, result$s2)
})

context("[validate] NICE TSD2 program 2b")

test_that("The summaries match", {
  result <- replicate.example("dietfat.fe", dietfat, likelihood="poisson", link="log", linearModel="fixed")
  compare.summaries(result$s1, result$s2)
})

context("[validate] Efficacy data from Cipriani et al. Lancet 2009;373:746-758")

test_that("The summaries match", {
  result <- replicate.example("cipriani-efficacy", depression)
  compare.summaries(result$s1, result$s2)
})

context("[validate] Welton et al., Am J Epidemiol 2009;169:1158-1165 Systolic BP")

test_that("The summaries match", {
  network <- dget('../data/welton-systolic.txt')
  result <- replicate.example("welton-systolic", network, likelihood="normal", link="identity")
  compare.summaries(result$s1, result$s2)
})

context("[validate] Thrombolytic drugs data from Lu & Ades, J Am Stat Assoc 2006;101(474):447-459, Table 3")

test_that("The summaries match", {
  result <- replicate.example("luades-thrombolytic", thrombolytic)
  compare.summaries(result$s1, result$s2)
})

context("[validate] NICE TSD2 program 2a")

test_that("The summaries match", {
  result <- replicate.example("dietfat.re", dietfat, likelihood="poisson", link="log", linearModel="random")
  compare.summaries(result$s1, result$s2)
})

context("[validate] Welton et al., Am J Epidemiol 2009;169:1158-1165 Diastolic BP")

test_that("The summaries match", {
  network <- dget('../data/welton-diastolic.txt')
  result <- replicate.example("welton-diastolic", network, likelihood="normal", link="identity")
  compare.summaries(result$s1, result$s2)
})

context("[validate] NICE TSD2 program 5a")

test_that("The summaries match", {
  result <- replicate.example("parkinson", parkinson, likelihood="normal", link="identity")
  compare.summaries(result$s1, result$s2)
})

context("[validate] NICE TSD2 program 8a")

test_that("The summaries match", {
  result <- replicate.example("parkinson-shared", parkinson_shared, likelihood="normal", link="identity")
  compare.summaries(result$s1, result$s2)
})

context("[validate] Smoking cessation data from Lu & Ades, J Am Stat Assoc 2006;101(474):447-459, Table 1")

test_that("The summaries match", {
  result <- replicate.example("luades-smoking", smoking)
  compare.summaries(result$s1, result$s2)
})

\encoding{utf8}
\name{mtc.nodesplit}
\alias{mtc.nodesplit}
\alias{mtc.nodesplit.comparisons}
\alias{plot.mtc.nodesplit}
\alias{print.mtc.nodesplit}
\alias{summary.mtc.nodesplit}
\alias{plot.mtc.nodesplit.summary}
\alias{print.mtc.nodesplit.summary}

\title{Node-splitting analysis of inconsistency}
\description{
Generate and run an ensemble of node-splitting models, results of which can be jointly summarized and plotted.
}
\usage{
mtc.nodesplit(network, comparisons=mtc.nodesplit.comparisons(network), ...)
mtc.nodesplit.comparisons(network)
}
\arguments{
  \item{network}{An object of S3 class \code{\link{mtc.network}}.}
  \item{comparisons}{Data frame specifying the comparisons to be split. The frame has two columns: 't1' and 't2'.}
  \item{...}{Arguments to be passed to \code{\link{mtc.run}} or \code{\link{mtc.model}}. This can be used to set the likelihood/link or the number of iterations, for example.}
}
\details{
\code{mtc.nodesplit} returns the MCMC results for all relevant node-splitting models \link[=gemtc-package]{[van Valkenhoef et al. 2015]}. To get appropriate summary statistics, call \code{summary()} on the results object. The summary can be plotted.
See \code{\link{mtc.model}} for details on how the node-splitting models are generated.

To control parameters of the MCMC estimation, see \code{\link{mtc.run}}.
To specify the likelihood/link or to control other model parameters, see \code{\link{mtc.model}}.
The \code{...} arguments are first matched against \code{\link{mtc.run}}, and those that do not match are passed to \code{\link{mtc.model}}.

\code{mtc.nodesplit.comparisons} returns a data frame enumerating all comparisons that can reasonably be split (i.e. have independent indirect evidence).
}
\value{
For \code{mtc.nodesplit}:
an object of class \code{mtc.nodesplit}. This is a list with the following elements: 
\item{d.X.Y}{For each comparison (t1=X, t2=Y), the MCMC results}
\item{consistency}{The consistency model results}

For \code{summary}:
an object of class \code{mtc.nodesplit.summary}. This is a list with the following elements: 
\item{dir.effect}{Summary of direct effects for each split comparison}
\item{ind.effect}{Summary of indirect effects for each split comparison}
\item{cons.effect}{Summary of consistency model effects for each split comparison}
\item{p.value}{Inconsistency p-values for each split comparison}
\item{cons.model}{The generated consistency model}
}
\author{Gert van Valkenhoef, Joël Kuiper}
\seealso{
\code{\link{mtc.model}}
\code{\link{mtc.run}}
}
\examples{
# Run all relevant node-splitting models
\dontrun{ result.ns <- mtc.nodesplit(parkinson, thin=50) }
# (read results from file instead of running:)
result.ns <- readRDS(system.file('extdata/parkinson.ns.rds', package='gemtc'))

# List the individual models 
names(result.ns)

# Time series plots and convergence diagnostics for d.A.C model
plot(result.ns$d.A.C)
gelman.diag(result.ns$d.A.C, multivariate=FALSE)

# Overall summary and plot
summary.ns <- summary(result.ns)
print(summary.ns)
plot(summary.ns)
}

\encoding{utf8}
\name{mtc.network}
\alias{mtc.network}
\alias{plot.mtc.network}

\title{Create an mtc.network}
\description{Creates an object of class \code{mtc.network}}
\usage{
mtc.network(data.ab=data, treatments=NULL, description="Network",
            data.re=NULL, studies=NULL, data=NULL)

\method{plot}{mtc.network}(x, layout=igraph::layout.circle, dynamic.edge.width=TRUE, use.description=FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data.ab}{Arm-level data.
A data frame defining the arms of each study, containing the columns `study' and `treatment', where
`treatment' must refer to an existing treatment ID if treatments were specified.
Further columns define the data per arm, and depend on the likelihood/link to be used.
See \code{\link{mtc.model}} for supported likelihood/links and their data requirements.}
  \item{data.re}{Relative effect data.
A data frame defining the arms of each study, containing the columns `study' and `treatment', where
`treatment' must refer to an existing treatment ID if treatments were specified.
The column `diff' specifies the mean difference between the current arm and the baseline arm; set `diff=NA' for the baseline arm.
The column `std.err' specifies the standard error of the mean difference (for non-baseline arms).
For trials with more than two arms, specify the standard error of the mean of the baseline arm in
`std.err', as this determines the covariance of the differences.}
  \item{treatments}{Optional.
A data frame with columns `id' and `description' defining the treatments or a vector giving the treatment IDs.}
  \item{studies}{Optional.
  A data frame with a `study' column naming the studies and additional columns containing covariate values.}
  \item{description}{Optional. Short description of the network.}
  \item{data}{Deprecated. Arm-level data; automatically assigned to data.ab if it is not specified.
Present for compatibility with older versions.}
  
  \item{x}{An mtc.network object.}
  \item{layout}{An igraph-compatible layout.}
  \item{dynamic.edge.width}{If set to TRUE, dynamically set the edge width based on the number of studies.}
  \item{use.description}{Display treatment descriptions instead of treatment IDs.}
  \item{...}{Additional arguments passed to \code{plot.igraph}.}
}
\value{
For \code{mtc.network}, an object of the class \code{mtc.network} which is a list containing:
  \item{description}{A short description of the network}
  \item{treatments}{A data frame describing the treatments}
  \item{data.ab}{A data frame containing the network data (arm-level)}
  \item{data.re}{A data frame containing the network data (relative effects)}
  \item{studies}{A data frame containing study-level information (covariates)}
These are cleaned up and standardized versions of the arguments provided, or generated defaults for `treatments' if the argument was omitted.
}
\details{
One-arm trials are automatically removed, which results in a warning.

Also see \code{\link{mtc.data.studyrow}} for a convenient way to import data from the one-study-per-row format, which is very popular for BUGS code.
}
\author{Gert van Valkenhoef, Joël Kuiper}

\seealso{
\code{\link{mtc.data.studyrow}},
\code{\link{mtc.model}}
}
\examples{
# Create a new network by specifying all information.
treatments <- read.table(textConnection('
  id  description
  A   "Treatment A"
  B   "Treatment B"
  C   "Treatment C"'), header=TRUE)
data <- read.table(textConnection('
  study  treatment  responders  sampleSize
  01     A          2           100
  01     B          5           100
  02     B          6           110
  02     C          1           110
  03     A          3           60
  03     C          4           80
  03     B          7           80'), header=TRUE)
network <- mtc.network(data, description="Example", treatments=treatments)
plot(network)

# Create a new network by specifying only the data.
data <- read.table(textConnection('
  study  treatment  mean   std.dev  sampleSize
  01     A          -1.12  0.6      15
  01     B          -1.55  0.5      16
  02     A          -0.8   0.7      33
  02     B          -1.1   0.5      31'), header=TRUE)
network <- mtc.network(data)

# Print the network
print(network)
## MTC dataset: Network
##   study treatment  mean std.dev sampleSize
## 1     1         A -1.12     0.6         15
## 2     1         B -1.55     0.5         16
## 3     2         A -0.80     0.7         33
## 4     2         B -1.10     0.5         31
}

\encoding{utf8}
\name{mtc.hy.prior}
\alias{mtc.hy.prior}
\alias{mtc.hy.empirical.lor}

\title{Set priors for the heterogeneity parameter}
\description{
These functions generate priors for the heterogeneity parameter in \code{\link{mtc.model}}. 
Priors can be set explicitly or, for outcomes on the log odds-ratio scale, based on empirical research.
}
\usage{
mtc.hy.prior(type, distr, ...)

mtc.hy.empirical.lor(outcome.type, comparison.type)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{type}{Type of heterogeneity prior: 'std.dev', 'var', or 'prec' for standard deviation, variance, or precision respectively.}
  \item{distr}{Prior distribution name (JAGS syntax). Typical ones would be 'dunif' (uniform), 'dgamma' (Gamma), or 'dlnorm' (log-normal). Use 'dhnorm' for the half-normal. Note that, as in JAGS, the precision (and not the variance or standard deviation) is used for the normal distribution and its derivatives.}
  \item{...}{Arguments to the \code{distr}. Can be numerical values or "om.scale" for the estimated outcome measure scale (see \code{\link{mtc.model}})}
  \item{outcome.type}{The type of outcome to get an empirical prior for. Can be one of 'mortality' (all-cause mortality), 'semi-objective' (e.g. cause-specific mortality, major morbidity event, drop-outs), or 'subjective' (e.g. pain, mental health, dichotomous biomarkers).}
  \item{comparison.type}{The type of comparison to get an empirical prior for. Can be one of 'pharma-control' (pharmacological interventions versus control), 'pharma-pharma' (pharmacological versus pharmacological interventions) and 'non-pharma' (any other comparisons).}
}
\value{
A value to be passed to \code{\link{mtc.model}}. 
}
\details{
The generated prior is a list, the structure of which may change without notice. It can be converted to JAGS compatible code using \code{as.character}.

Empirical priors for the log odds-ratio (LOR) are taken from \link[=gemtc-package]{[Turner et al. 2012]}.
}
\author{Gert van Valkenhoef}
\seealso{
\code{\link{mtc.model}}
}
\examples{
# NOTE: the mtc.run commands below are for illustrative purposes, such a small
# number of iterations should obviously not be used in practice.

# set a uniform prior standard deviation
model1 <- mtc.model(smoking, hy.prior=mtc.hy.prior("std.dev", "dunif", 0, 2))
result <- mtc.run(model1, n.adapt=10, n.iter=10)

# set an empirical (log-normal) prior on the variance
model2 <- mtc.model(smoking, hy.prior=mtc.hy.empirical.lor("subjective", "non-pharma"))
result <- mtc.run(model2, n.adapt=10, n.iter=10)

# set a gamma prior on the precision
model3 <- mtc.model(smoking, hy.prior=mtc.hy.prior("prec", "dgamma", 0.01, 0.01))
result <- mtc.run(model3, n.adapt=10, n.iter=10)
}

\encoding{utf8}
\name{smoking}
\alias{smoking}
\title{Psychological treatments to aid smoking cessation}
\description{A dataset of 24 trials investigating four psychological treatments and no treatment for smoking cessation. The outcome is the number of people who stopped smoking.}
\format{A network meta-analysis dataset containing 50 rows of arm-based data (responders and sample size).}
\source{
Lu and Ades (2006),
\emph{Assessing Evidence Inconsistency in Mixed Treatment Comparisons},
Journal of the American Statistical Society, 101(474):447-459.
[\doi{10.1198/016214505000001302}]

Hasselblad (1998),
\emph{Meta-analysis of multitreatment studies},
Medical Decision Making 18(1):37-43.
[\doi{10.1177/0272989X9801800110}]
}

\encoding{utf8}
\name{relative.effect.table}
\alias{relative.effect.table}
\alias{forest.mtc.relative.effect.table}
\alias{print.mtc.relative.effect.table}
\alias{as.data.frame.mtc.relative.effect.table}

\title{Table of relative effects}
\description{Generates a table of the relative effects of all pairs of treatments. Unlike \code{\link{relative.effect}}, this method stores summaries only, not raw samples.}
\usage{
relative.effect.table(result, covariate=NA)

\method{print}{mtc.relative.effect.table}(x, ...)
\method{forest}{mtc.relative.effect.table}(x, t1, use.description=FALSE, ...)
\method{as.data.frame}{mtc.relative.effect.table}(x, ...)
}
\arguments{
  \item{result}{An object of S3 class \code{mtc.result} to derive the relative effects from.}
  \item{covariate}{(Regression analyses only) Value of the covariate at which to compute relative effects.}
  \item{x}{An object of S3 class \code{mtc.relative.effect.table}.}
  \item{t1}{Baseline treatment for the Forest plot.}
  \item{use.description}{Display treatment descriptions instead of treatment IDs.}
  \item{...}{Additional arguments.}
}
\value{
  Returns an \code{mtc.relative.effect.table} object containing the quantiles of the calculated relative effects of all pair-wise comparisons among the treatments.

  The result will be pretty printed as an n-by-n table of relative treatment effects.
  It can also be used to produce Forest plots against any arbitrary baseline.
  Finally, the \code{as.data.frame} generic method makes it possible to export the table for use in Excel or other spreadsheet software, using the core R methods \code{write.csv} or \code{write.csv2}.
}
\author{Gert van Valkenhoef}
\seealso{
  \code{\link{relative.effect}}
}
\examples{
model <- mtc.model(smoking)
# To save computation time we load the samples instead of running the model
\dontrun{results <- mtc.run(model)}
results <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))

# Creates a forest plot of the relative effects
tbl <- relative.effect.table(results)

# Print the n*n table
print(tbl)

# Plot effect relative to treatment "C"
forest(tbl, "C")

# Write to CSV (e.g. to import to Excel, then use in a Word table)
\dontrun{write.csv(tbl, "smoking-effects.csv")}
# Note: use write.csv2 for Western European locales
}

\encoding{utf8}
\name{mtc.anohe}
\alias{mtc.anohe}
\alias{plot.mtc.anohe}
\alias{print.mtc.anohe}
\alias{summary.mtc.anohe}
\alias{plot.mtc.anohe.summary}
\alias{print.mtc.anohe.summary}

\title{Analysis of heterogeneity (ANOHE)}
\description{
(EXPERIMENTAL)
Generate an analysis of heterogeneity for the given network. Three types of model are estimated: unrelated study effects, unrelated mean effects, and consistency. Output of the \code{summary} function can passed to \code{plot} for a visual representation.
}
\usage{
mtc.anohe(network, ...)
}
\arguments{
  \item{network}{An object of S3 class \code{\link{mtc.network}}.}
  \item{...}{Arguments to be passed to \code{\link{mtc.run}} or \code{\link{mtc.model}}. This can be used to set the likelihood/link or the number of iterations, for example.}
}
\details{
Analysis of heterogeneity is intended to be a unified set of statistics and a visual display that allows the simultaneous assessment of both heterogeneity and inconsistency in network meta-analysis \link[=gemtc-package]{[van Valkenhoef et al. 2014b (draft)]}.

\code{mtc.anohe} returns the MCMC results for all three types of model. To get appropriate summary statistics, call \code{summary()} on the results object. The summary can be plotted.

To control parameters of the MCMC estimation, see \code{\link{mtc.run}}.
To specify the likelihood/link or to control other model parameters, see \code{\link{mtc.model}}.
The \code{...} arguments are first matched against \code{\link{mtc.run}}, and those that do not match are passed to \code{\link{mtc.model}}.
}
\note{This method should not be considered stable. It is an experimental feature and heavily work in progress. The interface may change at any time.
}
\value{
For \code{mtc.anohe}:
an object of class \code{mtc.anohe}. This is a list with the following elements: 
\item{result.use}{The result for the USE model (see \code{\link{mtc.run}}).}
\item{result.ume}{The result for the UME model (see \code{\link{mtc.run}}).}
\item{result.cons}{The result for the consistency model (see \code{\link{mtc.run}}).}

For \code{summary}:
an object of class \code{mtc.anohe.summary}. This is a list with the following elements: 
\item{cons.model}{Generated consistency model.}
\item{studyEffects}{Study-level effect summaries (multi-arm trials downweighted).}
\item{pairEffects}{Pair-wise pooled effect summaries (from the UME model).}
\item{consEffects}{Consistency effect summaries.}
\item{indEffects}{Indirect effect summaries (back-calculated).}
\item{isquared.comp}{Per-comparison I-squared statistics.}
\item{isquared.glob}{Global I-squared statistics.}
}
\author{Gert van Valkenhoef, Joël Kuiper}
\seealso{
\code{\link{mtc.model}}
\code{\link{mtc.run}}
}

\encoding{utf8}
\name{hfPrevention}
\alias{hfPrevention}
\title{Statins versus placebo in primary and secondary prevention of heart failure}
\description{A dataset of 19 trials comparing statins versus placebo or usual care for cholesterol lowering. The main outcome is the number of deaths. Trials are either primary prevention (no previous heart disease; secondary = 0) or secondary prevention (previous heart disease; secondary = 1).}
\format{A meta-regression dataset containing 38 rows of arm-based data (responders and sample size).}
\source{
S. Dias, A.J. Sutton, N.J. Welton, and A.E. Ades (2013b),
\emph{Heterogeneity - Subgroups, Meta-Regression, Bias, and Bias-Adjustment},
Medical Decision Making 33(5):618-640. \cr
[\doi{10.1177/0272989X13485157}]
}
\examples{
# Build a model similar to Program 1(a) from Dias et al. (2013b):
regressor <- list(coefficient='shared',
                  variable='secondary',
                  control='control')

model <- mtc.model(hfPrevention,
                   type="regression",
                   regressor=regressor,
                   hy.prior=mtc.hy.prior("std.dev", "dunif", 0, 5))

\dontrun{
result <- mtc.run(model)}
}

\encoding{utf8}
\name{atrialFibrillation}
\alias{atrialFibrillation}
\title{Prevention of stroke in atrial fibrillation patients}
\description{A dataset of 25 trials investigating 17 treatments for stroke prevention in atrial fibrillation patients. The main outcome is the number of patients with a stroke, and a covariate captures the proportion of patients with a prior stroke.

Data are take from Table 1 of Cooper et al. (2009), with the following corrections applied: SPAF 3 and AFASAK 2 do not have a treatment 13 arm, and SPAF 1 does not contain treatment 5, but treatment 6. Thanks to prof. Cooper for providing the original analysis dataset.}
\format{A network meta-regression dataset containing 63 rows of arm-based data (responders and sample size).}
\source{
Cooper et al. (2009),
\emph{Adressing between-study heterogeneity and inconsistency in mixed treatment comparisons: Application to stroke prevention treatments in individuals with non-rheumatic atrial fibrillation},
Statistics in Medicine 28:1861-1881.
[\doi{10.1002/sim.3594}]
}
\examples{
# Build a model similar to Model 4(b) from Cooper et al. (2009):
classes <- list("control"=c("01"),
                "anti-coagulant"=c("02","03","04","09"),
                "anti-platelet"=c("05","06","07","08","10","11","12","16","17"),
                "mixed"=c("13","14","15"))

regressor <- list(coefficient='shared',
                  variable='stroke',
                  classes=classes)

model <- mtc.model(atrialFibrillation,
                   type="regression",
                   regressor=regressor,
                   om.scale=10)

\dontrun{
result <- mtc.run(model)}
}

\encoding{utf8}
\name{certolizumab}
\alias{certolizumab}
\title{Certolizumab Pegol (CZP) for Rheumatoid Arthritis}
\description{A dataset of 12 trials investigating 6 treatments and placebo for rheumatoid arthritis. The main outcome is the number of patients who improved by at least 50\% on the American College of Rheumatology scale (ACR50) at 6 Months. A covariate is present for the mean disease duration at baseline (years).}
\format{A network meta-regression dataset containing 24 rows of arm-based data (responders and sample size).}
\source{
S. Dias, A.J. Sutton, N.J. Welton, and A.E. Ades (2013b),
\emph{Heterogeneity - Subgroups, Meta-Regression, Bias, and Bias-Adjustment},
Medical Decision Making 33(5):618-640. \cr
[\doi{10.1177/0272989X13485157}]
}
\examples{
# Run RE regression model with informative heterogeneity prior

regressor <- list(coefficient='shared',
                  variable='diseaseDuration',
                  control='Placebo')

# sd ~ half-Normal(mean=0, sd=0.32)
hy.prior <- mtc.hy.prior(type="std.dev", distr="dhnorm", 0, 9.77)

model <- mtc.model(certolizumab,
                   type="regression",
                   regressor=regressor,
                   hy.prior=hy.prior)

\dontrun{
result <- mtc.run(model)}
}

\encoding{utf8}
\name{thrombolytic}
\alias{thrombolytic}
\title{Thrombolytic treatment after acute myocardial infarction}
\description{A dataset of 28 trials investigating eight thrombolytic treatments administered after a myocardial infarction. The outcome is mortality after 30-35 days.}
\format{A network meta-analysis dataset containing 58 rows of arm-based data (responders and sample size).}
\source{
Lu and Ades (2006),
\emph{Assessing Evidence Inconsistency in Mixed Treatment Comparisons},
Journal of the American Statistical Society, 101(474):447-459.
[\doi{10.1198/016214505000001302}]

Boland et al. (2003),
\emph{Early thrombolysis for the treatment of acute myocardial infarction: a systematic review and economic evaluation},
Health Technology Assessment 7(15):1-136.
[\doi{10.3310/hta7150}]
}

\encoding{utf8}
\name{blocker}
\alias{blocker}
\title{Beta blockers to prevent mortality after myocardial infarction}
\description{A dataset of 22 trials investigating beta blockers versus control to prevent mortality after myocardial infarction. Number of events and sample size.}
\format{A meta-analysis dataset containing 44 rows of arm-based data (responders and sample size).}
\source{
S. Dias, A.J. Sutton, A.E. Ades, and N.J. Welton (2013a),
\emph{A Generalized Linear Modeling Framework for Pairwise and Network Meta-analysis of Randomized Controlled Trials},
Medical Decision Making 33(5):607-617.
[\doi{10.1177/0272989X12458724}]

J.B. Carlin (1992),
\emph{Meta-analysis for 2 × 2 tables: a Bayesian approach},
Statistics in Medicine 11(2):141-158.
[\doi{10.1002/sim.4780110202}]
}

\encoding{utf8}
\name{dietfat}
\alias{dietfat}
\title{Effects of low-fat diets on mortality}
\description{A dataset of 10 trials investigating low-fat diet versus control diet for mortality. Number of events and exposure in person-years.}
\format{A meta-analysis dataset containing 20 rows of arm-based data (responders, exposure, and sample size).}
\source{
S. Dias, A.J. Sutton, A.E. Ades, and N.J. Welton (2013a),
\emph{A Generalized Linear Modeling Framework for Pairwise and Network Meta-analysis of Randomized Controlled Trials},
Medical Decision Making 33(5):607-617.
[\doi{10.1177/0272989X12458724}]

Hooper et al. (2000),
\emph{Reduced or modified dietary fat for preventing cardiovascular disease},
Cochrane Database of Systematic Reviews 2:CD002137.
[\doi{10.1002/14651858.CD002137}]
}

\encoding{utf8}
\name{blobbogram}
\alias{blobbogram}
\alias{forest}
\title{Plot a blobbogram (AKA forest plot)}
\description{
\code{blobbogram} is a flexible function for creating blobbograms (forest plots), making no specific assumptions about the data being plotted. It supports column and row grouping as well as pagination.
}
\usage{
blobbogram(data, id.label="Study", ci.label="Mean (95\% CI)",
    left.label=NULL, right.label=NULL, center.label=NULL,
    log.scale=FALSE, xlim=NULL, styles=NULL,
    grouped=TRUE, group.labels=NULL,
    columns=NULL, column.labels=NULL,
    column.groups=NULL, column.group.labels=NULL,
    digits=2,
    ask=dev.interactive(orNone=TRUE),
    draw.no.effect=TRUE)

forest(x, ...)
}
\arguments{
  \item{data}{A data frame containing one row for each confidence interval to be visualized.
    The data format is described below.}
  \item{id.label}{Label to show above the row-id column.}
  \item{ci.label}{Label to show above the confidence intervals.}
  \item{left.label}{Label to show on the left-hand side of the no-difference line.}
  \item{right.label}{Label to show on the right-hand side of the no-difference line.}
  \item{center.label}{Label to show center-aligned with the no-difference line.}
  \item{log.scale}{If TRUE, the confidence intervals are given on a log scale, and axis labels will be \code{exp()} transformed.}
  \item{xlim}{The scale limits of the plot, if the confidence interval exceeds these limits an arrow will be shown at the limit. If unspecified, limits will be chosen that encompass all confidence intervals.}
  \item{styles}{A data frame describing the different row styles. By default, the styles "normal", "pooled" and "group" are defined.}
  \item{grouped}{If TRUE, and group.labels are specified, rows will be grouped according to the "group" column given in the data argument.}
  \item{group.labels}{Vector of group labels.}
  \item{columns}{Additional user-defined columns to be shown (names of columns given in the data argument).}
  \item{column.labels}{A vector of labels for the user-defined columns.}
  \item{column.groups}{Column groups, a numeric vector specifying the column group for each column.}
  \item{column.group.labels}{A vector of labels for the column groups.}
  \item{digits}{The number of (significant) digits to print.}
  \item{ask}{If TRUE, a prompt will be displayed before generating the
    next page of a multi-page plot.}
  \item{draw.no.effect}{If TRUE, draw the no-effect line.}
  \item{x}{An object to create a forest plot of.}
  \item{...}{Additional arguments.}
}
\details{
The \code{forest} function is a generic S3 method (definition compatible with the \code{meta} package). This package defines methods for \code{mtc.result} and \code{mtc.relative.effect.table}.

The \code{blobbogram} function creates a blobbogram (forest plot) from the given data (point estimates and confidence intervals) and meta-data (labels, column specifications, column groups, row groups, styles) using the \code{grid} package.
If the plot would not fit the device's graphics region, the content is broken up into multiple plots generated in sequence (pagination).

The \code{data} argument is given as a data frame containing the following columns:
\itemize{
  \item{id: identifier (label) for this row.}
  \item{group (optional): row group this row belongs to (indexes into the \code{group.labels} argument).}
  \item{pe: point estimate.}
  \item{ci.l: lower confidence interval limit.}
  \item{ci.u: upper confidence interval limit.}
  \item{style: the style to apply to this row (defined in the \code{styles} argument).}
}
Additional user-defined columns can be specified using the \code{columns} and \code{column.labels} arguments.

The \code{styles} argument is given as a data frame containing the following columns:
\itemize{
  \item{style: name of the style.}
  \item{weight: font weight.}
  \item{pe.style: symbol to draw for the point estimate ("circle" or "square", currently).}
}
}
\value{
None.
}
\note{
This method should not be considered stable.
We intend to generalize it further and possibly provide it in a separate package.
The interface may change at any time.
}
\author{Gert van Valkenhoef, Joël Kuiper}
%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{meta::forest}, \code{grid::Grid}
}
\examples{
data <- read.table(textConnection('
id               group pe       ci.l ci.u style      value.A    value.B 
"Study 1"  1         0.35 0.08 0.92 "normal" "2/46"     "7/46" 
"Study 2"  1         0.43 0.15 1.14 "normal" "4/50"     "8/49" 
"Study 3"  2         0.31 0.07 0.74 "normal" "2/97"     "10/100"
"Study 4"  2         0.86 0.34 2.90 "normal" "9/104"    "6/105" 
"Study 5"  2         0.33 0.10 0.72 "normal" "4/74"     "14/74" 
"Study 6"  2         0.47 0.23 0.91 "normal" "11/120" "22/129"
"Pooled"     NA      0.42 0.15 1.04 "pooled" NA             NA 
'), header=TRUE)
data$pe <- log(data$pe)
data$ci.l <- log(data$ci.l)
data$ci.u <- log(data$ci.u)

blobbogram(data, group.labels=c('GROUP 1', 'GROUP 2'),
    columns=c('value.A', 'value.B'), column.labels=c('r/n', 'r/n'),
    column.groups=c(1, 2), grouped=TRUE,
    column.group.labels=c('Intervention', 'Control'),
    id.label="Trial", ci.label="Odds Ratio (95\% CrI)", log.scale=TRUE)
}

\encoding{utf8}
\name{mtc.model}
\alias{mtc.model}
\alias{plot.mtc.model}
\alias{summary.mtc.model}
\alias{print.mtc.model}

\title{Generate network meta-analysis models}
\description{
The \code{mtc.model} function generates network meta-analysis models from an \code{mtc.network} object.
}
\usage{
mtc.model(network, type = "consistency", factor = 2.5, n.chain = 4,
  likelihood=NULL, link=NULL, linearModel="random",
  om.scale=NULL, hy.prior=mtc.hy.prior("std.dev", "dunif", 0, "om.scale"),
  re.prior.sd=15 * om.scale, dic=TRUE, powerAdjust=NA, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{network}{An object of S3 class \code{mtc.network}}
  \item{type}{A string literal indicating the type of model (allowed values are "consistency", "regression", "nodesplit", "ume", or "use").}
  \item{factor}{Variance scaling factor for the starting values}
  \item{n.chain}{Number of chains in the model}
  \item{likelihood}{The likelihood to be used. If unspecified, a suitable likelihood will be inferred for the given data.}
  \item{link}{The link function to be used. If unspecified, a suitable link function will be inferred for the given data.}
  \item{linearModel}{The type of linear model to be generated. Can be "random" for a random effects model, or "fixed" for a fixed effect model.}
  \item{om.scale}{Outcome measure scale. Represents a "very large" difference on the analysis' outcome scale. This is used to set vague priors. For the log odds-ratio, values between 2 and 5 are considered reasonable. For continuous outcomes, this depends heavily on the specific outcome. If left unspecified, it is determined from the data.}
  \item{hy.prior}{Heterogeneity prior. See \code{\link{mtc.hy.prior}}.}
  \item{re.prior.sd}{Standard deviation for the relative effects prior (normal distribution).}
  \item{dic}{When set to TRUE, deviance and fitted values will be monitored to allow computation of the Deviance Information Criterion (DIC) at residual.}
  \item{powerAdjust}{Optional: the name of a column in the studies data frame of the \code{mtc.network}. This column must contain values between 0 and 1. The likelihood for each study will be adjusted by inflating the variance, where 0 means the study is excluded and 1 means it receives full weight. See details for more.}
  \item{...}{Additional arguments to be passed to the type-specific model generation function.}
}
\value{
An object of class \code{mtc.model}.
The following elements are descriptive:
  \item{type}{The type of model}
  \item{network}{Network the model was generated from}
  \item{tree}{Spanning tree formed by the basic parameters}
  \item{var.scale}{The scaling factor used to over-disperse starting values}
  \item{likelihood}{The likelihood used}
  \item{link}{The link function used}
  \item{om.scale}{The scale for the variance parameters}
  \item{regressor}{Regressor specification (regression models only): includes additional elements "center" and "scale" describing how the regressor input was standardized}
These elements determine the model run by JAGS:
  \item{n.chain}{The number of chains}
  \item{code}{Model code in JAGS syntax. Use \code{cat()} for proper formatting.}
  \item{data}{Data in JAGS compatible format}
  \item{inits}{Initial values in JAGS compatible format}
  \item{monitors}{The nodes of the JAGS model to monitor}
  \item{dic}{Whether to compute the DIC}
These latter fields can be modified to alter the statistical model, but such changes may break the model or assumptions made elsewhere in the package.
}
\details{
The \code{mtc.model} function generates an object of S3 class \code{mtc.model}, which can be
visualized by the generic \code{plot} function or summarized by the generic \code{summary} function.

These likelihood/links are supported:
\itemize{
  \item{normal/identity: for continuous (mean difference) data.

  Required columns: \code{[mean, std.err]} or \code{[mean, std.dev, sampleSize]}.

  Result: relative mean difference.}
  \item{normal/smd: for continuous (mean difference) data.

  Required columns: \code{[mean, std.dev, sampleSize]}.

  Result: relative standardized mean difference.}
  \item{binom/logit: for dichotomous data.

  Required columns \code{[responders, sampleSize]}.

  Result: (log) odds ratio.}
  \item{binom/log: for dichotomous data.

  Required columns \code{[responders, sampleSize]}.

  Result: (log) risk ratio.}
  \item{binom/cloglog: for rate (survival) data - equal follow-up in each arm.

  Required columns \code{[responders, sampleSize]}.

  Result: (log) hazard ratio.}
  \item{poisson/log: for rate (survival) data.

  Required columns \code{[responders, exposure]}.

  Result: (log) hazard ratio.}
}
Most likelihood/links follow \link[=gemtc-package]{[Dias et al. 2013a]}, and the binom/log model follows \link[=gemtc-package]{[Warn et al. 2002]}.

The following model types are supported:
\itemize{
  \item{\code{consistency}: ordinary consistency model. No additional parameters. \link[=gemtc-package]{[Dias et al. 2013a, van Valkenhoef et al. 2012]}}
  \item{\code{nodesplit}: node-splitting model. Removes both arms used to estimate the direct evidence from the network of indirect evidence, rather than just one of those arms. This means that three-arm trials do not contribute any evidence in the network of indirect evidence. When relative effect data are present, these are transformed appropriately (using an assumption of normality) to enable this direct/indirect evidence split. Additional parameters: \code{t1} and \code{t2}, which indicate the comparison to be split. \link[=gemtc-package]{[Dias et al. 2010, van Valkenhoef et al. 2015]}}
  \item{\code{regression}: meta-regression model. Additional parameters: \code{regressor}, which indicates how to structure the treatment-interaction model for the regression. See below for details.}
  \item{\code{use}: unrelated study effects. Models the effects within each study as if the studies are independent. No additional parameters. \link[=gemtc-package]{[van Valkenhoef et al. (draft)]}}
  \item{\code{ume}: unrelated mean effects. Models the effects within each comparison as if they are independent. Does not properly handle multi-arm trials, and warns when they are present in the network. No additional parameters. \link[=gemtc-package]{[Dias et al. 2013b, van Valkenhoef et al. (draft)]}}
}

Regressor specification: a list with elements: \code{variable}, \code{coefficient}, and either \code{control} or \code{classes}.
The \code{variable} is the name of the covariate to include in the regression analysis, and must be a column of the studies data frame in the network.
The regressor variable is automatically centered and standardized using the method recommended by \link[=gemtc-package]{Gelman et al. (2008)}.
The \code{coefficient} indicates the type of treatment-interaction model: "shared", "unrelated", or "exchangeable".
\code{control}, if specified, must be the ID of a treatment in the network.
All other treatments have a coefficient relative to the control, which can be the same for all treatments ("shared"), different for all treatments ("unrelated") or exchangeable between treatments ("exchangeable").
\code{classes} is a named list of treatment classes, the first of which will act as the control class.
Each class is a vector of treatment IDs from the network.
Only "shared" coefficients can currently be used, meaning a single coefficient per class (except the control class).
See also \code{\link{atrialFibrillation}}, \code{\link{certolizumab}}, and \code{\link{hfPrevention}} for examples of meta-regression analyses.

Studies can be downweighted by using the \code{powerAdjust} argument, which applies a variance inflation (also known as "power prior") to the likelihood.
This allows a weight \eqn{\alpha_i \in [0, 1]}{a[i] in [0, 1])} to be specified for each study \eqn{i}{i}.
The log-likelihood will be multiplied by a factor \eqn{\alpha_i}{a[i]}, or equivalently for normal distributions the variance will be multiplied by \eqn{1/\alpha_i}{1/a[i]}.
Setting \eqn{\alpha_i = 0}{a[i] = 0} will completely exclude that study, whereas setting \eqn{\alpha_i = 1}{a[i] = 1} will weight it fully.
Essentially, down-weighted models modify the data and hence model fit statistics such as DIC can not be compared between models with different weightings.
}

\section{Visualization}{Calling the generic plot method on an S3 \code{mtc.model} object
will show a graph with the treatments as vertices and the comparisons as edges.
The lines with solid arrows represent basic parameters, and the other lines represent comparisons that are not associated with any parameter but do have direct evidence from trials.

\if{html}{
The example code will generate the following graph:

\figure{parkinson.png}{Graph showing structure of a consistency model generated from parkison example data}
}
\if{latex}{
The example code will generate the following graph:

\figure{parkinson.pdf}{options: width=3in}
}
The default layout algorithm is \code{igraph::layout.circle}, other layout algorithms can be used by passing them as an optional argument called \code{layout} to \code{plot}.
The \code{igraph::layout.fruchterman.reingold} algorithm also seems to produce nice results and may be better for large graphs.
}

\section{Reproducible results}{
It is possible to get reproducible results, but as JAGS uses its own pseudo-random number generator, this is somewhat more involved.

\code{mtc.model} generates random numbers using the R functions (for example to generate initial values). Therefore, \code{set.seed} must be called before calling \code{mtc.model}.

Then, before calling \code{mtc.run}, the random number generator type and seed must be set for each of the chains. This can be done as documented in \code{jags.model}, by setting the \code{.RNG.name} and \code{.RNG.seed} for each chain in \code{model$inits}. See below for an example.}


\author{Gert van Valkenhoef, Joël Kuiper}
%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{mtc.network}}, \code{\link{mtc.run}}
}
\examples{
# Random effects consistency model for Parkinson network
model <- mtc.model(parkinson)
plot(model)
summary(model)

# Fixed effect meta-regression for heart failure prevention
regressor <- list(coefficient='shared',
                  variable='secondary',
                  control='control')

model <- mtc.model(hfPrevention,
                   type="regression",
                   regressor=regressor,
                   linearModel="fixed")

# Reproducible results
# Set the R RNG seed
set.seed(42)
model <- mtc.model(parkinson, likelihood='normal', link='identity')
# By default, the model will have 4 chains - generate a seed for each
seeds <- sample.int(4, n = .Machine$integer.max)
# Apply JAGS RNG settings to each chain
model$inits <- mapply(c, model$inits, list(
  list(.RNG.name="base::Wichmann-Hill", .RNG.seed=seeds[1]),
  list(.RNG.name="base::Marsaglia-Multicarry", .RNG.seed=seeds[2]),
  list(.RNG.name="base::Super-Duper", .RNG.seed=seeds[3]),
  list(.RNG.name="base::Mersenne-Twister", .RNG.seed=seeds[4])), SIMPLIFY=FALSE)
}

\encoding{utf8}
\name{rank.probability}
\alias{rank.probability}
\alias{print.mtc.rank.probability}
\alias{plot.mtc.rank.probability}
\alias{sucra}
\alias{rank.quantiles}
\title{Calculating rank-probabilities}
\description{
Rank probabilities indicate the probability for each treatment to be best, second best, etc.
}
\details{
For each MCMC iteration, the treatments are ranked by their effect relative to an arbitrary baseline.
A frequency table is constructed from these rankings and normalized by the number of iterations to give the rank probabilities.
}
\usage{
rank.probability(result, preferredDirection=1, covariate=NA)

\method{print}{mtc.rank.probability}(x, ...)
\method{plot}{mtc.rank.probability}(x, ...)

sucra(ranks)
rank.quantiles(ranks, probs=c("2.5\%"=0.025, "50\%"=0.5, "97.5\%"=0.975))
}
\arguments{
  \item{result}{Object of S3 class \code{mtc.result} to be used in creation of the rank probability table}
  \item{preferredDirection}{Preferential direction of the outcome. Set 1 if higher values are preferred, -1 if lower values are preferred.}
  \item{covariate}{(Regression analyses only) Value of the covariate at which to compute rank probabilities.}
  \item{x}{An object of S3 class \code{rank.probability}.}
  \item{...}{Additional arguments.}
  \item{ranks}{A ranking matrix where the treatments are the rows (e.g. the result of rank.probability).}
  \item{probs}{Probabilities at which to give quantiles.}
}
\value{\code{rank.probability}: A matrix (with class \code{mtc.rank.probability}) with the treatments as rows and the ranks as columns.
\code{sucra}: A vector of SUCRA values.
\code{rank.quantiles}: A matrix with treatments as rows and quantiles as columns, giving the quantile ranks (by default, the median and 2.5\% and 97.5\% ranks).
}

\author{Gert van Valkenhoef, Joël Kuiper}

\seealso{
\code{\link{relative.effect}}
}
\examples{
model <- mtc.model(smoking)
# To save computation time we load the samples instead of running the model
\dontrun{results <- mtc.run(model)}
results <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))

ranks <- rank.probability(results)
print(ranks)
## Rank probability; preferred direction = 1
##       [,1]     [,2]     [,3]     [,4]
## A 0.000000 0.003000 0.105125 0.891875
## B 0.057875 0.175875 0.661500 0.104750
## C 0.228250 0.600500 0.170875 0.000375
## D 0.713875 0.220625 0.062500 0.003000

print(sucra(ranks))
##          A          B          C          D
## 0.03670833 0.39591667 0.68562500 0.88175000

print(rank.quantiles(ranks))
##   2.5% 50% 97.5%
## A    3   4     4
## B    1   3     4
## C    1   2     3
## D    1   1     3

plot(ranks) # plot a cumulative rank plot
plot(ranks, beside=TRUE) # plot a 'rankogram'
}

\encoding{utf8}
\name{parkinson}
\alias{parkinson}
\alias{parkinson_diff}
\alias{parkinson_shared}
\title{Mean off-time reduction in Parkinson's disease}
\description{A dataset of seven trials investigating four treatments and placebo for Parkinson's disease. The outcome is mean off-time reduction.}
\format{ \describe{
\item{parkinson: }{A network meta-analysis dataset containing fifteen rows of arm-based data (mean, standard deviation, and sample size).}
\item{parkinson_diff: }{A network meta-analysis dataset containing fifteen rows of contrast-based data.}
\item{parkinson_shared: }{A network meta-analysis dataset containing mixed arm-based and contrast-based data.}
} }
\source{
Franchini et al. (2012),
\emph{Accounting for correlation in network meta-analysis with multi-arm trials},
Research Synthesis Methods, 3(2):142-160.
[\doi{10.1002/jrsm.1049}]
}

\encoding{utf8}
\name{mtc.run}
\alias{mtc.run}
\alias{mtc.result}
\alias{summary.mtc.result}
\alias{plot.mtc.result}
\alias{forest.mtc.result}
\alias{print.mtc.result}
\alias{as.mcmc.list.mtc.result}

\title{Running an \code{mtc.model} using an MCMC sampler}
\description{
The function \code{mtc.run} is used to generate samples from a object of type \code{mtc.model} using a MCMC sampler.
The resulting \code{mtc.result} object can be coerced to an \code{mcmc.list} for further analysis of the dataset using the \code{coda} package.
}
\usage{
mtc.run(model, sampler = NA, n.adapt = 5000, n.iter = 20000, thin = 1, n.burnin = 0)

\method{summary}{mtc.result}(object, ...)
\method{plot}{mtc.result}(x, ...)
\method{forest}{mtc.result}(x, use.description=FALSE, ...)
\method{print}{mtc.result}(x, ...)
\method{as.mcmc.list}{mtc.result}(x, ...)
}
\arguments{
  \item{model}{An object of S3 class \code{mtc.model} describing a network meta-analysis model.}
  \item{sampler}{Deprecated: gemtc now only supports the JAGS sampler. Specifying a sampler will result in a warning or error. This argument will be removed in future versions.}
  \item{n.adapt}{Amount of adaptation (or tuning) iterations.}
  \item{n.iter}{Amount of simulation iterations.}
  \item{thin}{Thinning factor.}
  \item{n.burnin}{Amount of burn-in iterations. Has to be positive number.}

  \item{object}{Object of S3 class \code{mtc.result}.}
  \item{x}{Object of S3 class \code{mtc.result}.}
  \item{use.description}{Display treatment descriptions instead of treatment IDs.}
  \item{...}{Additional arguments.}
}
\value{
An object of class \code{mtc.result}. This is a list with the following elements: 
\item{samples}{The samples resulting from running the MCMC model, in \code{mcmc.list} format.}
\item{model}{The \code{mtc.model} used to produce the samples.}
\item{deviance}{Residual deviance statistics, a list with the following elements. \code{DIC}: deviance information criterion at residual (Dbar + pD). \code{Dbar}: mean sum of residual deviance. \code{pD}: sum of leverage, also known as the effective number of parameters. \code{dev.ab}: mean posterior residual deviance of each arm (for arm-based data). \code{fit.ab}: deviance at the posterior mean of te fitted values (for arm-based data). \code{dev.re}: mean posterior residual deviance of each study (for relative-effect data). \code{fit.re}: deviance at the posterior mean of the fitted values (for relative-effect data).}

The object can be coerced to an \code{mcmc.list} from the \code{coda} package by the generic S3 method \code{as.mcmc.list}.
}

\section{Analysis of the results}{
Convergence of the model can be assessed using methods from the \code{coda} package.
For example the Brooks-Gelman-Rubin method (\code{coda::gelman.diag}, \code{coda::gelman.plot}).
The \code{summary} also provides useful information, such as the MCMC error and the time series and densities given by \code{plot} should also be inspected.

The \code{forest} function can provide forest plots for \code{mtc.result} objects.
This is especially useful in combination with the \code{\link{relative.effect}} function that can be used to calculate relative effects compared to any baseline for consistency models.
The \code{\link{rank.probability}} function calculates rank probabilities for consistency models.
}

\author{Gert van Valkenhoef, Joël Kuiper}
\seealso{
\code{\link{mtc.model}}

\code{\link{relative.effect.table}},
\code{\link{relative.effect}},
\code{\link{rank.probability}}

\code{coda::gelman.diag},
\code{coda::gelman.plot}
}

\examples{
model <- mtc.model(smoking)

\dontrun{results <- mtc.run(model, thin=10)}
results <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))

# Convergence diagnostics
gelman.plot(results)

# Posterior summaries
summary(results)
## Iterations = 5010:25000
## Thinning interval = 10 
## Number of chains = 4 
## Sample size per chain = 2000 
## 
## 1. Empirical mean and standard deviation for each variable,
##    plus standard error of the mean:
## 
##         Mean     SD Naive SE Time-series SE
## d.A.B 0.4965 0.4081 0.004563       0.004989
## d.A.C 0.8359 0.2433 0.002720       0.003147
## d.A.D 1.1088 0.4355 0.004869       0.005280
## sd.d  0.8465 0.1913 0.002139       0.002965
## 
## 2. Quantiles for each variable:
## 
##          2.5%    25%    50%    75% 97.5%
## d.A.B -0.2985 0.2312 0.4910 0.7530 1.341
## d.A.C  0.3878 0.6720 0.8273 0.9867 1.353
## d.A.D  0.2692 0.8197 1.0983 1.3824 2.006
## sd.d   0.5509 0.7119 0.8180 0.9542 1.283

plot(results) # Shows time-series and density plots of the samples
forest(results) # Shows a forest plot
}

\encoding{utf8}
\name{plotCovariateEffect}
\alias{plotCovariateEffect}
\title{Plot treatment effects versus covariate values}
\description{
The plot will show the median treatment effect and the 95\% credible interval on the y-axis and the covariate value on the x-axis.
One plot page will be generated per pair of treatments.
}
\usage{
plotCovariateEffect(result, t1, t2, xlim=NULL, ylim=NULL,
                    ask=dev.interactive(orNone=TRUE))
}
\arguments{
  \item{result}{Results object - created by \code{mtc.result}}
  \item{t1}{A list of baseline treatments to calculate treatment effects against. Will be extended to match the length of t2.}
  \item{t2}{A list of treatments to calculate the effects for. Will be extended to match the length of t1.
If left empty and t1 is a single treatment, effects of all treatments except t1 will be calculated.}
  \item{xlim}{The x-axis limits.}
  \item{ylim}{The y-axis limits.}
  \item{ask}{If TRUE, a prompt will be displayed before generating the next page of a multi-page plot.}
}
\details{
Default x-axis limits will be set to three standard deviations above and below the centering value of the covariate.
The y-axis limits will be set based on the minimum and maximum 95\% CrI limits among the set of effects computed.
}
\value{
None.
}
\author{Gert van Valkenhoef}
\seealso{
\code{\link{relative.effect}}, \code{\link{mtc.run}}
}

\encoding{utf8}
\name{mtc.deviance}
\alias{mtc.deviance}
\alias{plot.mtc.deviance}
\alias{mtc.devplot}
\alias{mtc.levplot}

\title{Inspect residual deviance}
\description{Inspect the posterior residual deviance and summarize it using plots}
\usage{
mtc.deviance(result)

mtc.devplot(x, ...)
mtc.levplot(x, ...)

\method{plot}{mtc.deviance}(x, auto.layout=TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{result}{An object of class \code{\link{mtc.result}}.}
  \item{x}{An object of class \code{\link{mtc.deviance}}.}
  \item{auto.layout}{If \code{TRUE}, the separate plots will be shown as panels on a single page.}
  \item{...}{Graphical parameters.}
}
\value{
\code{mtc.deviance} returns the deviance statistics of a \code{mtc.result}.
}
\details{
\code{mtc.devplot} will generate a stem plot of the posterior deviance per arm (if there are only arm-based data) or the mean per data point deviance per study (if there are contrast-based data).

\code{mtc.levplot} will plot the leverage versus the square root of the residual deviance (mean per data point for each study).

The generic plot function will display both on a single page (unless \code{auto.layout=FALSE}).
}
\author{Gert van Valkenhoef}

\seealso{
\code{\link{mtc.run}}
}

\encoding{utf8}
\name{ll.call}
\alias{ll.call}
\title{Call a likelihood/link-specific function}
\description{
GeMTC implements various likelihood/link combinations.
Functionality specific to the likelihood/link is handled by methods with names ending in
\code{.<likelihood>.<link>}. 
This convenience function calls such methods.
}
\usage{
ll.call(fnName, model, ...)
}
\arguments{
  \item{fnName}{The name of the function to call. See details for available functions.}
  \item{model}{An object of S3 class \code{mtc.model} describing a network meta-analysis model, or a
list containing elements named 'likelihood' and 'link'.}
  \item{...}{Additional arguments to be passed to the function.}
}
\value{
The return value of the called function.
}
\details{
The following methods currently need to be implemented to implement a likelihood/link:
\itemize{
  \item{\code{mtc.arm.mle}: calculates a (corrected) maximum likelihood estimate for an arm-level
  effect. Used to generate starting values.}
  \item{\code{mtc.rel.mle}: calculates a (corrected) maximum likelihood estimate for a relative
  effect. Used to generate starting values.}
  \item{\code{mtc.code.likelihood}: generates JAGS code implementing the likelihood.}
  \item{\code{scale.log}: returns TRUE if plots should use the log scale.}
  \item{\code{scale.name}: returns the user-facing name of the outcome metric.}
  \item{\code{scale.limit.inits}: returns an upper and lower bound for the initial values, because
  some initial values might trigger boundary conditions such as probability 0 or 1 for the binomial.}
  \item{\code{required.columns.ab}: returns the required columns for arm-based data.}
}
The first two methods can now also be used to selectively apply continuity corrections in case the maximum likelihood estimates are used for other purposes. \code{mtc.arm.mle} has an additional \code{k=0.5} argument to specify the correction factor. \code{mtc.rel.mle} has arguments \code{correction.force=TRUE} to force application of the continuity correction even if unnecessary, \code{correction.type="constant"} to specify the type of correction (specify \code{"reciprocal"}) for a correction proportional to the reciprocal of the size of the other arm, and \code{correction.magnitude=1} to specify the (total) magnitude of the correction. These corrections apply only for count data, and will be ignored for continuous likelihood/links.
}
\author{Gert van Valkenhoef}
\seealso{
\code{\link{mtc.model}}
}
\examples{
# The "model" may be a stub.
model <- list(likelihood="poisson", link="log")

ll.call("scale.name", model)
# "Hazard Ratio"

ll.call("mtc.arm.mle", model, c('responders'=12, 'exposure'=80))
#      mean         sd
#-1.8562980  0.1118034
}

\encoding{utf8}
\name{relative.effect}
\alias{relative.effect}

\title{Calculating relative effects}
\description{Calculates the relative effects of pairs of treatments.}
\usage{
relative.effect(result, t1, t2 = c(), preserve.extra = TRUE, covariate = NA)
}
\arguments{
  \item{result}{An object of S3 class \code{mtc.result} to derive the relative effects from.}
  \item{t1}{A list of baselines to calculate a relative effects against. Will be extended to match the length of t2.}
  \item{t2}{A list of treatments to calculate the relative effects for. Will be extended to match the length of t1.
If left empty and t1 is a single treatment, relative effects of all treatments except t1 will be calculated.}
  \item{preserve.extra}{Indicates whether to preserve extra parameters such as the sd.d.}
  \item{covariate}{(Regression analyses only) Value of the covariate at which to compute relative effects.}
}
\value{
  Returns an \code{mtc.results} object containing the calculated relative effects.

  Note that this method stores the raw samples, which may result in excessive memory usage. You may want to consider using \code{\link{relative.effect.table}} instead.
}
\author{Gert van Valkenhoef, Joël Kuiper}
\seealso{
  \code{\link{rank.probability}},
  \code{\link{relative.effect.table}}
}
\examples{
model <- mtc.model(smoking)
# To save computation time we load the samples instead of running the model
\dontrun{results <- mtc.run(model)}
results <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))

# Creates a forest plot of the relative effects
forest(relative.effect(results, "A"))

summary(relative.effect(results, "B", c("A", "C", "D")))
## Iterations = 5010:25000
## Thinning interval = 10 
## Number of chains = 4 
## Sample size per chain = 2000 
## 
## 1. Empirical mean and standard deviation for each variable,
##    plus standard error of the mean:
## 
##          Mean     SD Naive SE Time-series SE
## d.B.A -0.4965 0.4081 0.004563       0.004989
## d.B.C  0.3394 0.4144 0.004634       0.004859
## d.B.D  0.6123 0.4789 0.005354       0.005297
## sd.d   0.8465 0.1913 0.002139       0.002965
## 
## 2. Quantiles for each variable:
## 
##          2.5%     25%     50%     75%  97.5%
## d.B.A -1.3407 -0.7530 -0.4910 -0.2312 0.2985
## d.B.C -0.4809  0.0744  0.3411  0.5977 1.1702
## d.B.D -0.3083  0.3005  0.6044  0.9152 1.5790
## sd.d   0.5509  0.7119  0.8180  0.9542 1.2827
}

\name{gemtc-package}
\alias{mtc}
\alias{gemtc}
\alias{gemtc-package}
\docType{package} 

\title{GeMTC: Network meta-analysis in R}
\description{
An R package for performing network meta-analyses (mixed treatment comparisons).
}
\details{
Network meta-analysis, or mixed treatment comparison (MTC) is a technique to meta-analyze networks of trials comparing two or more treatments at the same time [Dias et al. 2013a].
Using a Bayesian hierarchical model, all direct and indirect comparisons are taken into account to arrive at a single consistent estimate of the effect of all included treatments based on all included studies.

This package allows the automated generation of network meta-analysis models [van Valkenhoef et al. 2012], inclusing both fixed effect and random effects network meta-analysis, node-splitting models to identify inconsistency, and network meta-regression models.
Models are estimated using JAGS (through the \code{rjags} package).

It is possible to get reproducible results, but as JAGS uses its own pseudo-random number generator, this is somewhat more involved. See \code{\link{mtc.model}} for details.

The source for GeMTC is available under the GPL-3 on \href{https://github.com/gertvv/gemtc}{Github}.
}

\author{Gert van Valkenhoef}

\references{
S. Dias, N.J. Welton, D.M. Caldwell, and A.E. Ades (2010),
\emph{Checking consistency in mixed treatment comparison meta-analysis},
Statistics in Medicine 29(7-8, Sp. Iss. SI):932-944.\cr
[\doi{10.1002/sim.3767}]

S. Dias, A.J. Sutton, A.E. Ades, and N.J. Welton (2013a),
\emph{A Generalized Linear Modeling Framework for Pairwise and Network Meta-analysis of Randomized Controlled Trials},
Medical Decision Making 33(5):607-617.
[\doi{10.1177/0272989X12458724}]

S. Dias, A.J. Sutton, N.J. Welton, and A.E. Ades (2013b),
\emph{Heterogeneity - Subgroups, Meta-Regression, Bias, and Bias-Adjustment},
Medical Decision Making 33(5):618-640. \cr
[\doi{10.1177/0272989X13485157}]

S. Dias, N.J. Welton, A.J. Sutton, D.M. Caldwell, G. Lu, and A.E. Ades (2013c),
\emph{Inconsistency in Networks of Evidence Based on Randomized Controlled Trials},
Medical Decision Making 33(5):641-656.
[\doi{10.1177/0272989X12455847}]

A. Gelman, A. Jakulin, M. Grazia Pittau, Y.-S. Su (2008),
\emph{A weakly informative default prior distribution for logistic and other regression models},
The Annals of Applied Statistics 2(4):1360-1383.
[\doi{10.1214/08-AOAS191}]

R.M. Turner, J. Davey, M.J. Clarke, S.G. Thompson, J.P.T. Higgins (2012),
\emph{Predicting the extent of heterogeneity in meta-analysis, using empirical data from the Cochrane Database of Systematic Reviews},
International Journal of Epidemiology 41(3):818-827.
[\doi{10.1093/ije/dys041}]

G. van Valkenhoef, G. Lu, B. de Brock, H. Hillege, A.E. Ades, and N.J. Welton (2012),
\emph{Automating network meta-analysis},
Research Synthesis Methods 3(4):285-299.
[\doi{10.1002/jrsm.1054}]

G. van Valkenhoef, S. Dias, A.E. Ades, and N.J. Welton (2015),
\emph{Automated generation of node-splitting models for assessment of inconsistency in network meta-analysis},
Research Synthesis Methods, accepted manuscript.
[\doi{10.1002/jrsm.1167}]

G. van Valkenhoef et al. (draft),
\emph{Modeling inconsistency as heterogeneity in network meta-analysis},
draft manuscript.

D.E. Warn, S.G. Thompson, and D.J. Spiegelhalter (2002),
\emph{Bayesian random effects meta-analysis of trials with binary outcomes: methods for the absolute risk difference and relative risk scales},
Statistics in Medicine 21(11):1601-1623.
[\doi{10.1002/sim.1189}]

}
\seealso{
\code{\link{mtc.network}},
\code{\link{mtc.model}},
\code{\link{mtc.run}}
}

\examples{
# Load the example network and generate a consistency model:
model <- mtc.model(smoking, type="consistency")

# Load pre-generated samples instead of runing the model:
\dontrun{results <- mtc.run(model, thin=10)}
results <- readRDS(system.file("extdata/luades-smoking-samples.rds", package="gemtc"))

# Print a basic statistical summary of the results:
summary(results)
## Iterations = 5010:25000
## Thinning interval = 10 
## Number of chains = 4 
## Sample size per chain = 2000 
## 
## 1. Empirical mean and standard deviation for each variable,
##    plus standard error of the mean:
## 
##         Mean     SD Naive SE Time-series SE
## d.A.B 0.4965 0.4081 0.004563       0.004989
## d.A.C 0.8359 0.2433 0.002720       0.003147
## d.A.D 1.1088 0.4355 0.004869       0.005280
## sd.d  0.8465 0.1913 0.002139       0.002965
## 
## 2. Quantiles for each variable:
## 
##          2.5%    25%    50%    75% 97.5%
## d.A.B -0.2985 0.2312 0.4910 0.7530 1.341
## d.A.C  0.3878 0.6720 0.8273 0.9867 1.353
## d.A.D  0.2692 0.8197 1.0983 1.3824 2.006
## sd.d   0.5509 0.7119 0.8180 0.9542 1.283
}

\encoding{utf8}
\name{depression}
\alias{depression}
\title{Treatment response in major depression}
\description{A dataset of 111 trials investigating 12 treatments for major depression on treatment response. Treatment response was defined as a reduction of at least 50\% from the baseline score on the HAM-D or MADRS at week 8 (or, if not available, another time between week 6 and 12).}
\format{A network meta-analysis dataset containing 224 rows of arm-based data (responders and sample size).}
\source{
Cipriani et al. (2009),
\emph{Comparative efficacy and acceptability of 12 new-generation antidepressants: a multiple-treatments meta-analysis},
Lancet 373(9665):746-758.
[\doi{10.1016/S0140-6736(09)60046-5}]
}

\encoding{utf8}
\name{mtc.data.studyrow}
\alias{mtc.data.studyrow}

\title{Convert one-study-per-row datasets}
\description{Converts datasets in the one-study-per-row format to one-arm-per-row format used by GeMTC}
\usage{
mtc.data.studyrow(data,
  armVars=c('treatment'='t', 'responders'='r', 'sampleSize'='n'),
  nArmsVar='na',
  studyVars=c(),
  studyNames=1:nrow(data),
  treatmentNames=NA,
  patterns=c('\%s..', '\%s..\%d.'))
}
\arguments{
  \item{data}{Data in one-study-per-row format.}
  \item{armVars}{Vector of per-arm variables. The name of each component will be the column name in the resulting dataset. The column name in the source dataset is derived from the value of each component.}
  \item{nArmsVar}{Variable holding the number of arms for each study.}
  \item{studyVars}{Vector of per-study variables. The name of each component will be the column name in the resulting dataset. The column name in the source dataset is derived from the value of each component.}
  \item{studyNames}{Vector of study names.}
  \item{treatmentNames}{Vector of treatment names.}
  \item{patterns}{Patterns to generate column names in the source dataset. The first is for per-study variables, the second for per-arm variables.}
}
\value{A data table with the requested columns.}
\details{
Maps the one-study-per-row format that is widely used and convenient for BUGS models to the one-arm-per-row format used by GeMTC.
As the primary purpose is to input datasets from BUGS models, the defaults work for the standard BUGS data table format.
In most cases, it should be possible to just copy/paste the BUGS data table (without the final 'END') and \code{read.table} it into R, then apply \code{mtc.data.studyrow}.
In many cases, the resulting table can be processed directly by \code{\link{mtc.network}}.
}
\author{Gert van Valkenhoef}
\seealso{\code{\link{mtc.network}}}
\examples{
## Example taken from the NICE DSU TSD series in Evidence Synthesis, #2
## Dopamine agonists for the treatment of Parkinson's

# Read the bugs-formatted data
data.src <- read.table(textConnection('
t[,1]	t[,2]	t[,3]	y[,1]	y[,2]	y[,3]	se[,1]	se[,2]	se[,3]	na[]
1	3	NA	-1.22	-1.53	NA	0.504	0.439	NA	2
1	2	NA	-0.7	-2.4	NA	0.282	0.258	NA	2
1	2	4	-0.3	-2.6	-1.2	0.505	0.510	0.478	3
3	4	NA	-0.24	-0.59	NA	0.265	0.354	NA	2
3	4	NA	-0.73	-0.18	NA	0.335	0.442	NA	2
4	5	NA	-2.2	-2.5	NA	0.197	0.190	NA	2
4	5	NA	-1.8	-2.1	NA	0.200	0.250	NA	2'), header=TRUE)

# Convert the data, setting treatment names
data <- mtc.data.studyrow(data.src,
  armVars=c('treatment'='t', 'mean'='y', 'std.err'='se'),
  treatmentNames=c('Placebo', 'DA1', 'DA2', 'DA3', 'DA4'))

# Check that the data are correct
print(data)

# Create a network
network <- mtc.network(data)
}
